#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fstream>
#include "SDL.h"
#include "SDL_ttf.h"
#include "SDL_image.h"
#include "SDL_mixer.h"
#include "gp2x.h"
#include <SDL_gp2x.h>


using namespace std;

enum GameStates {GSQuit,GSIntro,GSGame,GSGameOver};

const int FPS = 30;
SDL_Surface *Screen,*Background,*Player,*Enemy,*Tmp,*Buffer;
TTF_Font* font;
SDL_Joystick *Joystick;
bool GlobalSoundEnabled = true;
GameStates GameState = GSIntro;
Uint32 NextTime;
bool EnemyStates[3][3];
bool PlayerStates[3];
bool TvOutMode = false;
int HitPosition, LivesLost,StartScreenX=20,StartScreenY=24,InitialStartScreenX=20,InitialStartScreenY=24;
Mix_Chunk *TickSound, *CrashSound;
long Score = 0;
SDL_Color FG = {0,0,0,0};
SDL_Color BG = {172,172,172,0};

Uint32 WaitForFrame()
{
	Uint32 Now,Result;
	Now = SDL_GetTicks();
	if (Now >= NextTime)
	{
		Result = 0;
		NextTime = Now + 1000/FPS;
	}
	else
		Result = NextTime - Now;
	return Result;
}

void DrawGame()
{
	int X,Y;
	SDL_Rect DstRect;
	for (X=0;X<3;X++)
		for (Y=0;Y<3;Y++)
			if (EnemyStates[X][Y])
			{
				DstRect.x = 90 + (X *54);
				DstRect.y = 48 + (Y * 47);
				DstRect.w = Enemy->w;
				DstRect.h = Enemy->h;
				SDL_BlitSurface(Enemy,NULL,Buffer,&DstRect);
			}
	for (X=0;X<3;X++)
	{
		if (PlayerStates[X])
		{
			DstRect.x = 90 + (X * 54);
			DstRect.y = 180;
			DstRect.w = Player->w;
			DstRect.h = Player->h;
			SDL_BlitSurface(Player,NULL,Buffer,&DstRect);
		}
	}
}

void MoveEnemy()
{
	int X,Y;
	for (X=0;X<3;X++)
		for (Y=2;Y>=1;Y--)
			EnemyStates[X][Y] = EnemyStates[X][Y-1];
	for (X=0;X<3;X++)
		EnemyStates[X][0] = false;
	for (X=0;X<=1;X++)
		EnemyStates[rand()%3][0] = true;
}

void MoveLeft()
{
	int X;
	for (X=0;X<2;X++)
	{
		if (PlayerStates[X+1])
		{
			PlayerStates[X] = true;
			PlayerStates[X+1] = false;
		}
	}
}

void MoveRight()
{
	int X;
	for (X=2;X>=1;X--)
	{
		if (PlayerStates[X-1])
		{
			PlayerStates[X] = true;
			PlayerStates[X-1] = false;
		}
	}
}

bool IsCollided()
{
	int X;
	bool Temp;
	Temp=false;
	for (X = 0;X<3;X++)
		if (PlayerStates[X] && EnemyStates[X][2])
		{
			Temp = true;
			HitPosition = X;
		}
	return Temp;
}

void HitFlash()
{
	PlayerStates[HitPosition] = !PlayerStates[HitPosition];
	EnemyStates[HitPosition][2] = !EnemyStates[HitPosition][2];
}

void InitialiseStates()
{
	int X,Y;
	for(X = 0;X<3;X++)
		for(Y=0;Y<3;Y++)
			EnemyStates[X][Y] = false;
	for(X=0;X<3;X++)
		PlayerStates[X] = false;
	PlayerStates[1] = true;
}

void Game()
{
	SDL_Event Event;
	int Teller,FlashesDelay,Flashes,X,Delay;
	SDL_Surface* Text;
	SDL_Rect DstRect,Rect;
	bool CanMove,CrashSoundPlayed,RPressed=false;
	char Str[100];
	Teller = 25;
	FlashesDelay = 14;
	Flashes = 0;
	CanMove = true;
	Score = 0;
	Delay = 60;
	LivesLost = 0;
	CrashSoundPlayed = false;
	InitialiseStates();
	while (GameState == GSGame)
	{
		while (SDL_PollEvent(&Event))
		{
		    if (Event.type == SDL_JOYBUTTONUP)
                switch (Event.jbutton.button)
                {
                    case GP2X_BUTTON_R:
                        RPressed = false;
                        break;
                    case GP2X_BUTTON_LEFT:
                        if (RPressed)
                            if(StartScreenX - 2 >= 0)
                                StartScreenX -=2;
                        break;
                    case GP2X_BUTTON_RIGHT:
                        if (RPressed)
                            if(StartScreenX + 2 + Buffer->w <= Screen->w)
                                StartScreenX +=2;
                        break;
                    case GP2X_BUTTON_UP:
                        if(RPressed)
                            if(StartScreenY - 2 >=0)
                                StartScreenY -=2;
                        break;
                    case GP2X_BUTTON_DOWN:
                        if(RPressed)
                            if(StartScreenY + 2 + Buffer->h <= Screen->h)
                                StartScreenY +=2;
                        break;
                }

			if (Event.type == SDL_JOYBUTTONDOWN)
			{
				switch(Event.jbutton.button)
				{
				    case GP2X_BUTTON_R:
                        RPressed = true;
                        break;
					case GP2X_BUTTON_SELECT:
						GameState= GSQuit;
						break;
					case GP2X_BUTTON_A:
						if(CanMove)
							MoveLeft();
						break;
					case GP2X_BUTTON_B:
						if(CanMove)
							MoveRight();
						break;
				}
			}
		}
		SDL_BlitSurface(Background,NULL,Buffer,NULL);
		sprintf(Str,"%d",Score);
		Text = TTF_RenderText_Shaded(font,Str,FG,BG);
		DstRect.x = 230 - Text->w;
		DstRect.y = 30;
		DstRect.w = Text->w;
		DstRect.h = Text->h;
		SDL_BlitSurface(Text,NULL,Buffer,&DstRect);
		SDL_FreeSurface(Text);
		if (LivesLost >=1)
		{
			sprintf(Str,"");
			for(X = 0;X<LivesLost;X++)
				Str[X] = 'X';
			Str[LivesLost] = '\0';
			Text =TTF_RenderText_Shaded(font,Str,FG,BG);
			DstRect.x = 90;
			DstRect.y = 30;
			DstRect.w = Text->w;
			DstRect.h = Text->h;
			SDL_BlitSurface(Text,NULL,Buffer,&DstRect);
			SDL_FreeSurface(Text);
		}
		Teller++;
		if (Teller >= Delay)
		{
			if (!IsCollided() && CanMove)
			{
				Teller = 0;
				for (X = 0;X < 3;X++)
					if (EnemyStates[X][2])
					{
						Score += 10;
						if ((Score % 100 ==0) & (Delay > 18))
							Delay--;
					}
				MoveEnemy();
				if (GlobalSoundEnabled)
					Mix_PlayChannel(-1,TickSound,0);
			}
			else
			{
				if (!CrashSoundPlayed)
				{
					if (GlobalSoundEnabled)
						Mix_PlayChannel(-1,CrashSound,0);
                    CrashSoundPlayed = true;
				}
				CanMove = false;
				FlashesDelay++;
				if (FlashesDelay == 15)
				{
					Flashes++;
					HitFlash();
					FlashesDelay = 0;
					if (Flashes == 6)
					{
						Flashes = 0;
						CanMove = true;
						Teller = 0;
						CrashSoundPlayed=false;
						EnemyStates[HitPosition][2] = false;
						LivesLost++;
						FlashesDelay = 14;
						if (LivesLost ==3)
							GameState = GSGameOver;
					}
				}
			}
		}
		DrawGame();
		Rect.w = Buffer->w;
        Rect.h = Buffer->h;
        Rect.x = StartScreenX;
        Rect.y = StartScreenY;
        SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
        SDL_BlitSurface(Buffer,NULL,Screen,&Rect);
		SDL_Flip(Screen);
		SDL_Delay(WaitForFrame());
	}

}

void GameOver()
{
	SDL_Event Event;
	int X;
	SDL_Surface* Text;
	SDL_Rect DstRect,Rect;
	char Str[100];
	bool RPressed=false;
	while (GameState == GSGameOver)
	{
		while (SDL_PollEvent(&Event))
		{
			if (Event.type == SDL_JOYBUTTONUP)
                switch (Event.jbutton.button)
                {
                    case GP2X_BUTTON_R:
                        RPressed = false;
                        break;
                    case GP2X_BUTTON_LEFT:
                        if (RPressed)
                            if(StartScreenX - 2 >= 0)
                                StartScreenX -=2;
                        break;
                    case GP2X_BUTTON_RIGHT:
                        if (RPressed)
                            if(StartScreenX + 2 + Buffer->w <= Screen->w)
                                StartScreenX +=2;
                        break;
                    case GP2X_BUTTON_UP:
                        if(RPressed)
                            if(StartScreenY - 2 >=0)
                                StartScreenY -=2;
                        break;
                    case GP2X_BUTTON_DOWN:
                        if(RPressed)
                            if(StartScreenY + 2 + Buffer->h <= Screen->h)
                                StartScreenY +=2;
                        break;
                }

			if (Event.type == SDL_JOYBUTTONDOWN)
			{
				switch(Event.jbutton.button)
				{
				    case GP2X_BUTTON_R:
                        RPressed = true;
                        break;
					case GP2X_BUTTON_SELECT:
						GameState = GSQuit;
						break;
                    case GP2X_BUTTON_DOWN:
                    case GP2X_BUTTON_UP:
                    case GP2X_BUTTON_RIGHT:
                    case GP2X_BUTTON_LEFT:
                    break;
					default:
						GameState = GSGame;
						break;
				}
			}
		}
		SDL_BlitSurface(Background,NULL,Buffer,NULL);
		sprintf(Str,"%d",Score);
		Text = TTF_RenderText_Shaded(font,Str,FG,BG);
		DstRect.x = 230 - Text->w;
		DstRect.y = 30;
		DstRect.w = Text->w;
		DstRect.h = Text->h;
		SDL_BlitSurface(Text,NULL,Buffer,&DstRect);
		SDL_FreeSurface(Text);
		if (LivesLost >=1)
		{
			sprintf(Str,"");
			for(X = 0;X<LivesLost;X++)
				Str[X] = 'X';
			Str[LivesLost] = '\0';
			Text =TTF_RenderText_Shaded(font,Str,FG,BG);
			DstRect.x = 90;
			DstRect.y = 30;
			DstRect.w = Text->w;
			DstRect.h = Text->h;
			SDL_BlitSurface(Text,NULL,Buffer,&DstRect);
			SDL_FreeSurface(Text);
		}
		DrawGame();
		Rect.w = Buffer->w;
        Rect.h = Buffer->h;
        Rect.x = StartScreenX;
        Rect.y = StartScreenY;
        SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
        SDL_BlitSurface(Buffer,NULL,Screen,&Rect);
		SDL_Flip(Screen);
		SDL_Delay(WaitForFrame());
	}

}

void FlashIntro()
{
	int X,Y;
	for (X = 0;X < 3;X++)
 		for (Y = 0;Y<3;Y++)
 			EnemyStates[X][Y] = ! EnemyStates[X][Y];
 	for (X=0;X<3;X++)
 		PlayerStates[X] = ! PlayerStates[X];
}

void Intro()
{
	SDL_Event Event ;
	int X,FlashesDelay;
	SDL_Surface * Text;
	SDL_Rect DstRect,Rect;
	char ScoreStr[20],Str[20];
	bool RPressed=false;
	LivesLost = 0;
	FlashesDelay = 0;
	sprintf(ScoreStr,"");
	while (GameState == GSIntro)
	{
		while (SDL_PollEvent(&Event))
		{
			if (Event.type == SDL_JOYBUTTONUP)
                switch (Event.jbutton.button)
                {
                    case GP2X_BUTTON_R:
                        RPressed = false;
                        break;
                    case GP2X_BUTTON_LEFT:
                        if (RPressed)
                            if(StartScreenX - 2 >= 0)
                                StartScreenX -=2;
                        break;
                    case GP2X_BUTTON_RIGHT:
                        if (RPressed)
                            if(StartScreenX + 2 + Buffer->w <= Screen->w)
                                StartScreenX +=2;
                        break;
                    case GP2X_BUTTON_UP:
                        if(RPressed)
                            if(StartScreenY - 2 >=0)
                                StartScreenY -=2;
                        break;
                    case GP2X_BUTTON_DOWN:
                        if(RPressed)
                            if(StartScreenY + 2 + Buffer->h <= Screen->h)
                                StartScreenY +=2;
                        break;
                }

			if (Event.type == SDL_JOYBUTTONDOWN)
			{
				switch(Event.jbutton.button)
				{
				    case GP2X_BUTTON_R:
                        RPressed = true;
                        break;
					case GP2X_BUTTON_SELECT:
						GameState = GSQuit;
						break;
                    case GP2X_BUTTON_DOWN:
                    case GP2X_BUTTON_UP:
                    case GP2X_BUTTON_RIGHT:
                    case GP2X_BUTTON_LEFT:
                        break;
					default:
						GameState = GSGame;
						break;
				}
			}
		}
		SDL_BlitSurface(Background,NULL,Buffer,NULL);
		FlashesDelay++;
		if (FlashesDelay == 25)
		{
			FlashesDelay = 0;
			FlashIntro();
			if (PlayerStates[0])
			{
				LivesLost = 3;
				sprintf(ScoreStr,"888888");
			}
			else
			{
				LivesLost = 0;
				sprintf(ScoreStr,"");
			}
		}
		if (strlen(ScoreStr) > 0)
		{
			Text = TTF_RenderText_Shaded(font,ScoreStr,FG,BG);
			DstRect.x = 230 - Text->w;
			DstRect.y = 30;
			DstRect.w = Text->w;
			DstRect.h = Text->h;
			SDL_BlitSurface(Text,NULL,Buffer,&DstRect);
			SDL_FreeSurface(Text);
		}
		if (LivesLost >=1)
		{
			sprintf(Str,"");
			for(X = 0;X<LivesLost;X++)
			Str[X] = 'X';
			Str[LivesLost] = '\0';
			Text =TTF_RenderText_Shaded(font,Str,FG,BG);
			DstRect.x = 90;
			DstRect.y = 30;
			DstRect.w = Text->w;
			DstRect.h = Text->h;
			SDL_BlitSurface(Text,NULL,Buffer,&DstRect);
			SDL_FreeSurface(Text);
		}
		DrawGame();
		Rect.w = Buffer->w;
        Rect.h = Buffer->h;
        Rect.x = StartScreenX;
        Rect.y = StartScreenY;
        SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
        SDL_BlitSurface(Buffer,NULL,Screen,&Rect);
		SDL_Flip(Screen);
		SDL_Delay(WaitForFrame());
	}

}

void LoadSettings()
{
 	FILE *SettingsFile;
 	int Value=1;
 	SettingsFile = fopen("./settings.dat","r");
 	if(SettingsFile)
 	{
		if(TvOutMode)
		{
            fscanf(SettingsFile,"StartScreenX=%d\n",&StartScreenX);
            fscanf(SettingsFile,"StartScreenY=%d\n",&StartScreenY);
		}
		else
        {
            fscanf(SettingsFile,"StartScreenX=%d\n",&InitialStartScreenX);
            fscanf(SettingsFile,"StartScreenY=%d\n",&InitialStartScreenY);
            StartScreenX = 0;
            StartScreenY = 0;
        }
		fclose(SettingsFile);
 	}
 	else
 	{
		StartScreenX = (Screen->w - Buffer->w) / 2;
        StartScreenY = (Screen->h - Buffer->h) / 2;
	}
}

// Save the settings
void SaveSettings()
{
 	FILE *SettingsFile;
 	SettingsFile = fopen("./settings.dat","w");
 	if(SettingsFile)
 	{
        if(TvOutMode)
        {
            fprintf(SettingsFile,"StartScreenX=%d\n",StartScreenX);
            fprintf(SettingsFile,"StartScreenY=%d\n",StartScreenY);
        }
        else
        {
            fprintf(SettingsFile,"StartScreenX=%d\n",InitialStartScreenX);
            fprintf(SettingsFile,"StartScreenY=%d\n",InitialStartScreenY);
        }
		fclose(SettingsFile);
		sync();
 	}
}

int main(int argc, char **argv)
{
    printf("lives lost address: %X\n",&LivesLost);
    SDL_Rect Rect;
	if (SDL_Init(SDL_INIT_JOYSTICK | SDL_INIT_VIDEO | SDL_INIT_AUDIO ) == 0)
	{
		printf("SDL Succesfully initialized\n");
		if(SDL_GP2X_GetPhysicalScreenSize(&Rect) == 0)
            Screen = SDL_SetVideoMode( WINDOW_WIDTH, WINDOW_HEIGHT,16, SDL_SWSURFACE );
        else
        {
            TvOutMode = true;
            Screen = SDL_SetVideoMode( 360, 288,16, SDL_SWSURFACE );
        }
		if(Screen)
		{
		    Buffer = SDL_CreateRGBSurface(SDL_SWSURFACE,WINDOW_WIDTH,WINDOW_HEIGHT,16,Screen->format->Rmask,Screen->format->Gmask,Screen->format->Bmask,Screen->format->Amask);
			StartScreenX = (Screen->w - Buffer->w) / 2;
            StartScreenY = (Screen->h - Buffer->h) / 2;
			printf("Succesfully Set %dx%dx16\n",WINDOW_WIDTH,WINDOW_HEIGHT);
			SDL_ShowCursor(SDL_DISABLE);
			Joystick = SDL_JoystickOpen(0);
			SDL_JoystickEventState(SDL_ENABLE);
			if(Joystick)
			{
				if (Mix_OpenAudio(11025,AUDIO_S16,MIX_DEFAULT_CHANNELS,64) < 0)
				{
					GlobalSoundEnabled = false;
					printf("Failed to initialise sound!\n");
				}
				else
				{
			 		printf("Audio Succesfully initialised!\n");
				}
				if (TTF_Init() == 0)
				{
					printf("Succesfully initialized TTF\n");
					font = TTF_OpenFont("./font.ttf",14);
					if (font)
					{
						printf("Succesfully Loaded font\n");
						TTF_SetFontStyle(font,TTF_STYLE_NORMAL);
						srand(time(NULL));
						TickSound = Mix_LoadWAV("./sounds/tick.wav");
						CrashSound = Mix_LoadWAV("./sounds/crash.wav");
						Tmp = IMG_Load("./graphics/background.png");
						Background = SDL_DisplayFormat(Tmp);
						SDL_FreeSurface(Tmp);
						Tmp = IMG_Load("./graphics/player.png");
						Player = SDL_DisplayFormat(Tmp);
						SDL_FreeSurface(Tmp);
						Tmp = IMG_Load("./graphics/enemy.png");
						Enemy = SDL_DisplayFormat(Tmp);
						SDL_FreeSurface(Tmp);
						SDL_ShowCursor(0);
						LoadSettings();
						while (GameState != GSQuit)
						{
							switch(GameState)
							{
								case GSIntro :
									Intro();
									break;
								case GSGame :
									Game();
									break;
								case GSGameOver:
									GameOver();
									break;
							}
						}
						SaveSettings();
						TTF_CloseFont(font);
				    	font=NULL;
					}
					else
					{
						printf("Failed to Load font\n");
					}
					TTF_Quit();
				}
				else
				{
					printf("Failed to initialize TTF\n");
				}
				Mix_FreeChunk(CrashSound);
				Mix_FreeChunk(TickSound);
				SDL_FreeSurface(Screen);
				SDL_FreeSurface(Background);
				SDL_FreeSurface(Player);
				SDL_FreeSurface(Enemy);
				Screen=NULL;
				Mix_CloseAudio();
				if(SDL_JoystickOpened(0))
    					SDL_JoystickClose(Joystick);
			}
			else
			{
				printf("Failed to open joystick(0)!\n");
			}
		}
		else
		{
			printf("Failed to Set Videomode %dx%dx32\n",WINDOW_WIDTH, WINDOW_HEIGHT);
		}
		SDL_Quit();
	}
	else
	{
		printf("Couldn't initialise SDL!\n");
	}
	system("sync");
	//chdir("/usr/gp2x");
	//execl("/usr/gp2x/gp2xmenu", "/usr/gp2x/gp2xmenu", NULL);
	return 0;
}


