#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fstream>
#include <dirent.h>
#include <sys/stat.h>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_mixer.h>
#include "gp2x.h"
#include <SDL_gp2x.h>
#include <SDL/SDL_gfxPrimitives.h>


using namespace std;


SDL_Surface *IMGBackground=NULL,*IMGBlocks=NULL,*IMGCursor=NULL,*IMGReady=NULL,*IMGGo=NULL,*IMGTimeOver=NULL,*IMGIntro1=NULL, *IMGIntro2=NULL,*IMGIntro3=NULL,
            *IMGTitleScreen=NULL,*IMGQuit1=NULL,*IMGQuit2=NULL,*IMGPlay1=NULL,*IMGPlay2=NULL,*IMGHighScores1=NULL,*IMGHighScores2=NULL,*IMGCredits1=NULL,*IMGCredits2=NULL,
            *IMGCredits=NULL,*IMGSelectGame=NULL,*IMGFixedTimer1=NULL,*IMGFixedTimer2=NULL,*IMGRelativeTimer1=NULL,*IMGRelativeTimer2=NULL,*IMGHighScores;


enum GameStates {GSQuit,GSIntro,GSGame,GSTitleScreen,GSTimeOver,GSReadyGo,GSCredits,GSGameTypeMenu,GSShowHighScores};
enum GameTypes {Fixed=0,Relative=1};

const int   NrOfSounds = 14,
            NrOfRows = 10,
            NrOfCols = 10,
            NrOfBlockColors = 5,
            BlockBlue = 0,
            BlockYellow = 1,
            BlockOrange = 2,
            BlockGreen = 3,
            BlockRed = 4,
            MaxSkins = 100,
            MaxMusicFiles=26,
            TileWidth = 23,
            TileHeight = 23,
            MaxAlpha = 150,
            AlphaIncrease = 20,
            SND_READYGO = 0,
            SND_5MINUTE = 1,
            SND_1MINUTE = 2,
            SND_3 = 3,
            SND_2 = 4,
            SND_1 = 5,
            SND_TIMEOVER = 6,
            SND_MENU = 7,
            SND_SELECT = 8,
            SND_WELCOME = 9,
            SND_GOODBYE = 10,
            SND_DELETE = 11,
            SND_BLOCKSELECT = 12,
            SND_ERROR = 13;

Mix_Chunk *Sounds[NrOfSounds];
Mix_Music *Music[MaxMusicFiles];

bool GlobalSoundEnabled = true;

struct SHighScore
{
    char PName[21];
    long PScore;
};

class CGameTypeMenu
{
    private:
        int Selection; //Holds what we currently selected in the main menu, 1 = new game, 2= credits, 3 = quit
    public:
        CGameTypeMenu(); // constructor
        void Draw(SDL_Surface *Surface); //draw to a certain surface
        void NextItem(); //Select the next menuitem
        void PreviousItem(); //select the previous menuitem
        int GetSelection() { return Selection; } // return the current selection
        ~CGameTypeMenu(); // destructor
};
// constructor of main menu will Load the graphics and set the current selection to 1 (io newgame)
CGameTypeMenu::CGameTypeMenu()
{
    Selection = Fixed;
}

// Destructor will free the surface images
CGameTypeMenu::~CGameTypeMenu()
{

}

// Increase the selection if it goes to far set i to the first selection
void CGameTypeMenu::NextItem()
{
    Selection++;
    if (Selection == 2)
        Selection = 0;
    if (GlobalSoundEnabled)
        Mix_PlayChannel(-1,Sounds[SND_MENU],0);
}

// decrease the selection if it goes to low set it to the last selection
void CGameTypeMenu::PreviousItem()
{
    Selection--;
    if (Selection == -1)
        Selection = 1;
    if (GlobalSoundEnabled)
        Mix_PlayChannel(-1,Sounds[SND_MENU],0);
}

// Draw the main menu
void CGameTypeMenu::Draw(SDL_Surface *Surface)
{
    SDL_Rect Dest;

    // draw the title screen background
    SDL_BlitSurface(IMGTitleScreen,NULL,Surface,NULL);
    Dest.x = 68;
    Dest.y = 77;
    Dest.w = IMGSelectGame->w;
    Dest.h = IMGSelectGame->h;
    SDL_BlitSurface(IMGSelectGame,NULL,Surface,&Dest);

    if (Selection == Fixed)
    {
        Dest.x = 68;
        Dest.y = 114;
        Dest.w = IMGFixedTimer2->w;
        Dest.h = IMGFixedTimer2->h;
        SDL_BlitSurface(IMGFixedTimer2,NULL,Surface,&Dest);
    }
    else
    {
        Dest.x = 68 ;
        Dest.y = 114 ;
        Dest.w = IMGFixedTimer1->w;
        Dest.h = IMGFixedTimer1->h;
        SDL_BlitSurface(IMGFixedTimer1,NULL,Surface,&Dest);
    }

    if (Selection == Relative)
    {
        Dest.x = 68 ;
        Dest.y = 151;
        Dest.w = IMGRelativeTimer2->w;
        Dest.h = IMGRelativeTimer2->h;
        SDL_BlitSurface(IMGRelativeTimer2,NULL,Surface,&Dest);
    }
    else
    {
        Dest.x = 68 ;
        Dest.y = 151 ;
        Dest.w = IMGRelativeTimer1->w;
        Dest.h = IMGRelativeTimer1->h;
        SDL_BlitSurface(IMGRelativeTimer1,NULL,Surface,&Dest);
    }


}


class CMainMenu
{
    private:
        int Selection; //Holds what we currently selected in the main menu, 1 = new game, 2= credits, 3 = quit
    public:
        CMainMenu(); // constructor
        void Draw(SDL_Surface *Surface); //draw to a certain surface
        void NextItem(); //Select the next menuitem
        void PreviousItem(); //select the previous menuitem
        int GetSelection() { return Selection; } // return the current selection
        ~CMainMenu(); // destructor
};
// constructor of main menu will Load the graphics and set the current selection to 1 (io newgame)
CMainMenu::CMainMenu()
{
    Selection = 1;
}

CMainMenu::~CMainMenu()
{
}

// Increase the selection if it goes to far set i to the first selection
void CMainMenu::NextItem()
{
    Selection++;
    if (Selection == 5)
        Selection = 1;
    if (GlobalSoundEnabled)
        Mix_PlayChannel(-1,Sounds[SND_MENU],0);
}

// decrease the selection if it goes to low set it to the last selection
void CMainMenu::PreviousItem()
{
    Selection--;
    if (Selection == 0)
        Selection = 4;
    if (GlobalSoundEnabled)
        Mix_PlayChannel(-1,Sounds[SND_MENU],0);
}

// Draw the main menu
void CMainMenu::Draw(SDL_Surface *Surface)
{
    SDL_Rect Dest;


    SDL_BlitSurface(IMGTitleScreen,NULL,Surface,NULL);
    if (Selection == 1)
    {
        Dest.x = 98;
        Dest.y = 73;
        Dest.w = IMGPlay2->w;
        Dest.h = IMGPlay2->h;
        SDL_BlitSurface(IMGPlay2,NULL,Surface,&Dest);
    }
    else
    {
        Dest.x = 98 ;
        Dest.y = 73 ;
        Dest.w = IMGPlay1->w;
        Dest.h = IMGPlay1->h;
        SDL_BlitSurface(IMGPlay1,NULL,Surface,&Dest);
    }

    if (Selection == 2)
    {
        Dest.x = 98 ;
        Dest.y = 105 ;
        Dest.w = IMGHighScores2->w;
        Dest.h = IMGHighScores2->h;
        SDL_BlitSurface(IMGHighScores2,NULL,Surface,&Dest);
    }
    else
    {
        Dest.x = 98 ;
        Dest.y = 105 ;
        Dest.w = IMGHighScores1->w;
        Dest.h = IMGHighScores1->h;
        SDL_BlitSurface(IMGHighScores1,NULL,Surface,&Dest);
    }


    if (Selection == 3)
    {
        Dest.x = 98 ;
        Dest.y = 137 ;
        Dest.w = IMGCredits2->w;
        Dest.h = IMGCredits2->h;
        SDL_BlitSurface(IMGCredits2,NULL,Surface,&Dest);

    }
    else
    {
        Dest.x = 98 ;
        Dest.y = 137 ;
        Dest.w = IMGCredits1->w;
        Dest.h = IMGCredits1->h;
        SDL_BlitSurface(IMGCredits1,NULL,Surface,&Dest);

    }

    if (Selection == 4)
    {
        Dest.x = 98 ;
        Dest.y = 169 ;
        Dest.w = IMGQuit2->w;
        Dest.h = IMGQuit2->h;
        SDL_BlitSurface(IMGQuit2,NULL,Surface,&Dest);
    }
    else
    {
        Dest.x = 98 ;
        Dest.y = 169 ;
        Dest.w = IMGQuit1->w;
        Dest.h = IMGQuit1->h;
        SDL_BlitSurface(IMGQuit1,NULL,Surface,&Dest);
    }

}


struct SPoint
{
    int X,Y;
};

class CSelector
{
    private:
        SPoint SelectedPoint,CurrentPoint; //variables to hold the current position and the selected position

    public:
        bool HasSelection;
        // constructor, parameters define the starting position
        CSelector(const int PlayFieldXin,const int PlayFieldYin)
        {
            SelectedPoint.X = 0; // there's no selection set it 0
            SelectedPoint.Y = 0;
            CurrentPoint.X = PlayFieldXin; // set current position
            CurrentPoint.Y = PlayFieldYin;
            HasSelection = false; // No selection when it's created
        }
        // Will set the position only if it layes within the board boundary
        void SetPosition(const int PlayFieldXin,const int PlayFieldYin)
        {
            // check if the new position is inside the board, if so set the new position as the currentposition
            if ((PlayFieldYin>=0) && (PlayFieldYin < NrOfRows) && (PlayFieldXin >= 0) && (PlayFieldXin <NrOfCols))
            {
                CurrentPoint.X = PlayFieldXin;
                CurrentPoint.Y = PlayFieldYin;
            }
            else
            {
                if(PlayFieldYin < 0)
                {
                    CurrentPoint.X = PlayFieldXin;
                    CurrentPoint.Y = NrOfRows-1;
                }

                if(PlayFieldYin >= NrOfRows)
                {
                    CurrentPoint.X = PlayFieldXin;
                    CurrentPoint.Y = 0;
                }

                if(PlayFieldXin < 0)
                {
                    CurrentPoint.X = NrOfCols-1;
                    CurrentPoint.Y = PlayFieldYin;
                }

                if(PlayFieldXin >= NrOfCols)
                {
                    CurrentPoint.X = 0;
                    CurrentPoint.Y = PlayFieldYin;
                }



            }
        }
        // Select will set the current position as the selected position
        void Select()
        {
            SelectedPoint.X = CurrentPoint.X;
            SelectedPoint.Y = CurrentPoint.Y;
            HasSelection = true;
        }
        // return the selected position
        SPoint GetSelection()
        {
            return SelectedPoint;
        }
        // return the current position
        SPoint GetPosition()
        {
            return CurrentPoint;
        }
        void DeSelect()
        {
            HasSelection = false;
        }
        // Draw the blue box on the current position, with the offsets in mind
        void Draw(SDL_Surface *Surface)
        {
            SDL_Rect DstRect;
            DstRect.x = CurrentPoint.X * TileWidth+5;
            DstRect.y = CurrentPoint.Y * TileHeight+5;
            DstRect.w = TileWidth;
            DstRect.h = TileHeight;
            SDL_BlitSurface(IMGCursor,NULL,Surface,&DstRect);
        }
        ~CSelector()
        {
        }
};

class CBlock
{
    private:
        int Color,AnimCounter,AnimBase,AnimDelay,AnimDelayCounter,AnimPhases,AnimPhase,PlayFieldX,PlayFieldY;
        bool bNeedToKill,bSelected;
    public:
        CBlock(int PlayFieldXin,int PlayFieldYin,int ColorIn)
        {
            AnimBase = 0;
            AnimPhase = 0;
            AnimCounter = 0;
            AnimDelayCounter = 0;
            AnimPhases = 1;
            AnimDelay = 2;
            PlayFieldX = PlayFieldXin;
            PlayFieldY = PlayFieldYin;
            Color = ColorIn;
            bNeedToKill = false;
            bSelected = false;
        }
        void Select()
        {
            AnimBase = 0;
            AnimCounter = 0;
            AnimPhases = 6;
            bSelected = true;
        }
        void DeSelect()
        {
            AnimBase = 0;
            AnimCounter = 0;
            AnimPhases = 1;
            bSelected=false;

        }
        void Kill()
        {
            AnimBase = 6;
            AnimCounter = 0;
            AnimPhases = 1;
            bNeedToKill = true;
        }
        void Draw(SDL_Surface *Surface)
        {
            SDL_Rect Dst,Src;

            Src.x = AnimPhase *TileWidth;
            Src.y = Color *TileHeight;
            Src.w = TileWidth;
            Src.h = TileHeight;

            Dst.x = PlayFieldX * TileWidth+5;
            Dst.y = PlayFieldY * TileHeight+5;
            Dst.w = TileWidth;
            Dst.h = TileHeight;

            SDL_BlitSurface(IMGBlocks,&Src,Surface,&Dst);

            AnimPhase = AnimBase + AnimCounter;
            if(AnimPhase != AnimBase+ AnimPhases - 1)
            {
                AnimDelayCounter++;
                if (AnimDelayCounter >=AnimDelay)
                {
                    AnimDelayCounter = 0;
                    AnimCounter++;
                    if (AnimCounter == AnimPhases)
                        AnimCounter = 0;
                }
            }
        }
        int GetColor()
        {
            return Color;
        }
        bool IsSelected()
        {
            return bSelected;
        }
        bool NeedToKill()
        {
            return bNeedToKill;
        }
        ~CBlock()
        {
        }

};

class CWorldParts
{
    private:
        CBlock * Items[NrOfCols][NrOfRows];
        SPoint Selects [4];
        int NumSelected,SelectedColor;
        Uint32 Time;
        bool NeedToKillBlocks,NeedToAddBlocks;
        void KillBlocks()
        {
            int X,Y,Teller=0,StartX=NrOfCols,StartY=NrOfRows,EndX=-1,EndY=-1;
            for(Teller=0;Teller<NumSelected;Teller++)
            {
                if(Selects[Teller].X > EndX)
                    EndX = Selects[Teller].X;
                if(Selects[Teller].X < StartX)
                    StartX = Selects[Teller].X;
                if(Selects[Teller].Y > EndY)
                    EndY = Selects[Teller].Y;
                if(Selects[Teller].Y < StartY)
                    StartY = Selects[Teller].Y;
            }
            for(Y = StartY;Y<=EndY;Y++)
                for(X=StartX;X<=EndX;X++)
                    Items[X][Y]->Kill();
        }
        void AddBlocks()
        {
            int X,Y;
            for(Y=0;Y<NrOfRows;Y++)
                for(X=0;X<NrOfCols;X++)
                    if(Items[X][Y]->NeedToKill())
                    {
                        delete Items[X][Y];
                        Items[X][Y] = new CBlock(X,Y,rand()%NrOfBlockColors);
                    }
        }
    public:
        CWorldParts()
        {
            int X,Y;
            NeedToKillBlocks = false;
            NeedToAddBlocks = false;
            NumSelected = 0;
            SelectedColor = -1;
             for(Y=0;Y<NrOfRows;Y++)
                for(X=0;X<NrOfCols;X++)
                    Items[X][Y] = NULL;
        }
        void NewGame()
        {
            int X,Y;
            for(Y=0;Y<NrOfRows;Y++)
                for(X=0;X<NrOfCols;X++)
                {
                    if (Items[X][Y])
                        delete Items[X][Y];
                    Items[X][Y] = new CBlock(X,Y,rand()%NrOfBlockColors);
                }
            NeedToKillBlocks = false;
            NeedToAddBlocks = false;
            NumSelected = 0;
            SelectedColor = -1;
        }
        ~CWorldParts()
        {
            printf("Freed\n");
            int X,Y;
            for(Y=0;Y<NrOfRows;Y++)
                for(X=0;X<NrOfCols;X++)
                    if(Items[X][Y])
                        delete Items[X][Y];
        }
        void Draw(SDL_Surface * Surface)
        {
            if(NeedToKillBlocks && (Time < SDL_GetTicks()))
            {
                KillBlocks();
                NeedToKillBlocks = false;
                NeedToAddBlocks = true;
                Time = SDL_GetTicks() + 350;
                Mix_PlayChannel(-1,Sounds[SND_DELETE],0);
            }

            if (NeedToAddBlocks && (Time < SDL_GetTicks()))
            {
                AddBlocks();
                NeedToAddBlocks = false;
                NumSelected = 0;
            }

            SDL_BlitSurface(IMGBackground,NULL,Surface,NULL);
            int X,Y;
            for(Y=0;Y<NrOfRows;Y++)
                for(X=0;X<NrOfCols;X++)
                    Items[X][Y]->Draw(Surface);
        }
        void DeSelect(bool PlaySound)
        {
            if(GlobalSoundEnabled)
                if(PlaySound)
                    Mix_PlayChannel(-1,Sounds[SND_ERROR],0);
            int Teller = 0;
            for(Teller = 0;Teller<NumSelected;Teller++)
                Items[Selects[Teller].X][Selects[Teller].Y]->DeSelect();
            NumSelected = 0;
            SelectedColor = -1;
        }
        long Select(int PlayFieldX,int PlayFieldY)
        {

            int Score = 0,NumEqualY=0,NumEqualX = 0,Teller1=0,Teller2=0,StartX=NrOfCols,StartY=NrOfRows,EndX=-1,EndY=-1;;
            if(!NeedToKillBlocks && !NeedToAddBlocks)
            if(!Items[PlayFieldX][PlayFieldY]->IsSelected())
            {

                if (NumSelected == 0)
                {
                    Items[PlayFieldX][PlayFieldY]->Select();
                    SelectedColor = Items[PlayFieldX][PlayFieldY]->GetColor();
                    Selects[NumSelected].X = PlayFieldX;
                    Selects[NumSelected].Y = PlayFieldY;
                    NumSelected++;
                    Mix_PlayChannel(-1,Sounds[SND_BLOCKSELECT],0);

                }
                else
                {
                    if(Items[PlayFieldX][PlayFieldY]->GetColor() == SelectedColor)
                    {
                        Items[PlayFieldX][PlayFieldY]->Select();
                        Selects[NumSelected].X = PlayFieldX;
                        Selects[NumSelected].Y = PlayFieldY;
                        NumSelected++;
                        if(NumSelected > 2)
                        {
                            for(Teller1=0;Teller1 < NumSelected;Teller1++)
                            {
                                for(Teller2=0;Teller2<NumSelected;Teller2++)
                                if(Teller1 != Teller2)
                                {
                                    if(Selects[Teller1].X == Selects[Teller2].X)
                                        NumEqualX++;
                                    if(Selects[Teller1].Y == Selects[Teller2].Y)
                                        NumEqualY++;
                                }
                            }
                            if((NumEqualX > 4) || (NumEqualY > 4))
                                DeSelect(true);
                        }


                        if(NumSelected == 3)
                            if (!(((NumEqualX == 2) && (NumEqualY == 2)) ))
                                DeSelect(true);

                        if(NumSelected == 4)
                        {
                            if((NumEqualX == 4) && (NumEqualY == 4))
                            {
                               // Mix_PlayChannel(-1,Sounds[SND_BLOCKSELECT],0);
                                for(Teller1=0;Teller1<NumSelected;Teller1++)
                                {
                                    if(Selects[Teller1].X > EndX)
                                        EndX = Selects[Teller1].X;
                                    if(Selects[Teller1].X < StartX)
                                        StartX = Selects[Teller1].X;
                                    if(Selects[Teller1].Y > EndY)
                                        EndY = Selects[Teller1].Y;
                                    if(Selects[Teller1].Y < StartY)
                                        StartY = Selects[Teller1].Y;
                                }
                                Score = (EndY-StartY+1) * (EndX-StartX +1) * 100;
                                NeedToKillBlocks = true;
                                Time = SDL_GetTicks() + 350;
                            }
                            else
                                DeSelect(true);
                        }

                        if(((NumSelected <= 4) && (NumSelected > 0)) )
                            Mix_PlayChannel(-1,Sounds[SND_BLOCKSELECT],0);
                    }
                    else
                        DeSelect(true);
                }

            }
            else
                DeSelect(true);
            return Score;
        }

};


//Screen is het scherm, Buffer is de groote van het spelscherm (altijd 320x240)
//Buffer1 is de groote van het spel scherm of de groote de tv out mode
// er wordt dus altijd naar Buffer getekend, buffer wordt naar buffer1 geblit en buffer1 naar het scherm
SDL_Surface *Buffer,*Buffer1,*Screen;
TTF_Font* font,*BigFont,*MonoFont;
SDL_Joystick *Joystick;
bool TvOutMode=false;
GameStates GameState = GSIntro;
int MusicCount=0,SelectedMusic=0,Volume = 128,StartScreenX=0,StartScreenY=0,InitialStartScreenX=20,InitialStartScreenY=24,InstalledSkinsCount=0,SelectedSkin=-1,Score=0;
char InstalledSkins[MaxSkins][FILENAME_MAX];
char SkinName[FILENAME_MAX];
CWorldParts *World;
GameTypes GameType = Fixed;
SHighScore HighScores[2][10];

bool FileExists(char * FileName)
{
	FILE *Fp;
	Fp = fopen(FileName,"rb");
	if (Fp)
	{
		fclose(Fp);
		return true;
	}
	else
		return false;
}

void UnloadMusic()
{
	int Teller;
	if (GlobalSoundEnabled)
	{
		Mix_HaltMusic();
		for (Teller=0;Teller < MusicCount;Teller++)
			if (Music[Teller])
				Mix_FreeMusic(Music[Teller]);
	}
	//Mix_FreeMusic(Music[0]);
}

void SearchForMusic()
{
	struct dirent *Entry;
	DIR *Directory;
	struct stat Stats;
	int Teller;
	char FileName[FILENAME_MAX];
	if (GlobalSoundEnabled)
		Music[0] = Mix_LoadMUS("./music/title.mod");
	Teller=1;
	Directory = opendir("./music");
	if (Directory)
	{
		Entry=readdir(Directory);
		while(Entry)
		{
			sprintf(FileName,"./music/%s",Entry->d_name);
			stat(FileName,&Stats);
			if(!S_ISDIR(Stats.st_mode))
			{
				if(strncmp(".", Entry->d_name, 1) && (strcmp("title.mod",Entry->d_name) != 0) && (Teller< MaxMusicFiles))
				{
				//	printf("%s\n",Entry->d_name);
					if (GlobalSoundEnabled)
					{
						Music[Teller] = Mix_LoadMUS(FileName);
						Teller++;
					}
				}
			}
			Entry=readdir(Directory);
		}
		closedir(Directory);
	}
	MusicCount = Teller;
}

void SearchForSkins()
{
	struct dirent *Entry;
	DIR *Directory;
	struct stat Stats;
	int Teller=0;
	char FileName[FILENAME_MAX];
	Directory = opendir("./skins");
	if (Directory)
	{
		Entry=readdir(Directory);
		while(Entry)
		{
			sprintf(FileName,"./skins/%s",Entry->d_name);
			stat(FileName,&Stats);
			if(S_ISDIR(Stats.st_mode))
			{
				//printf("%s\n",Entry->d_name);
				if(strncmp(".", Entry->d_name, 1)  && (Teller< MaxSkins) && (strlen(Entry->d_name) < FILENAME_MAX-1))
				{
					sprintf(InstalledSkins[Teller],"%s",Entry->d_name);
					Teller++;
				}
			}
			Entry=readdir(Directory);
		}
		closedir(Directory);
	}
	InstalledSkinsCount = Teller;
	SelectedSkin=-1;
}

void UnLoadGraphics()
{
    if(IMGHighScores)
        SDL_FreeSurface(IMGHighScores);
    if(IMGSelectGame)
        SDL_FreeSurface(IMGSelectGame);
    if(IMGFixedTimer2)
        SDL_FreeSurface(IMGFixedTimer2);
    if(IMGRelativeTimer2)
        SDL_FreeSurface(IMGRelativeTimer2);
    if(IMGFixedTimer1)
        SDL_FreeSurface(IMGFixedTimer1);
    if(IMGRelativeTimer1)
        SDL_FreeSurface(IMGRelativeTimer1);
    if(IMGIntro1)
        SDL_FreeSurface(IMGIntro1);
    if(IMGIntro2)
        SDL_FreeSurface(IMGIntro2);
    if(IMGIntro3)
        SDL_FreeSurface(IMGIntro3);
    if(IMGTimeOver)
        SDL_FreeSurface(IMGTimeOver);
    if(IMGBlocks)
        SDL_FreeSurface(IMGBlocks);
    if(IMGBackground)
        SDL_FreeSurface(IMGBackground);
    if(IMGCursor)
        SDL_FreeSurface(IMGCursor);
    if(IMGReady)
        SDL_FreeSurface(IMGReady);
    if(IMGGo)
        SDL_FreeSurface(IMGGo);
    if(IMGPlay1)
        SDL_FreeSurface(IMGPlay1);
    if(IMGPlay2)
        SDL_FreeSurface(IMGPlay2);
    if(IMGCredits1)
        SDL_FreeSurface(IMGCredits1);
    if(IMGCredits2)
        SDL_FreeSurface(IMGCredits2);
    if(IMGQuit1)
        SDL_FreeSurface(IMGQuit1);
    if(IMGQuit2)
        SDL_FreeSurface(IMGQuit2);
    if(IMGHighScores1)
        SDL_FreeSurface(IMGHighScores1);
    if(IMGHighScores2)
        SDL_FreeSurface(IMGHighScores2);
    if(IMGTitleScreen)
        SDL_FreeSurface(IMGTitleScreen);
    if(IMGCredits)
        SDL_FreeSurface(IMGCredits);
}

void LoadGraphics()
{
    char FileName[FILENAME_MAX];
    SDL_Surface *Tmp;

    if(IMGHighScores)
        SDL_FreeSurface(IMGHighScores);
    if(IMGSelectGame)
        SDL_FreeSurface(IMGSelectGame);
    if(IMGFixedTimer2)
        SDL_FreeSurface(IMGFixedTimer2);
    if(IMGRelativeTimer2)
        SDL_FreeSurface(IMGRelativeTimer2);
    if(IMGFixedTimer1)
        SDL_FreeSurface(IMGFixedTimer1);
    if(IMGRelativeTimer1)
        SDL_FreeSurface(IMGRelativeTimer1);
    if(IMGIntro1)
        SDL_FreeSurface(IMGIntro1);
    if(IMGIntro2)
        SDL_FreeSurface(IMGIntro2);
    if(IMGIntro3)
        SDL_FreeSurface(IMGIntro3);
    if(IMGTimeOver)
        SDL_FreeSurface(IMGTimeOver);
    if(IMGBlocks)
        SDL_FreeSurface(IMGBlocks);
    if(IMGBackground)
        SDL_FreeSurface(IMGBackground);
    if(IMGCursor)
        SDL_FreeSurface(IMGCursor);
    if(IMGReady)
        SDL_FreeSurface(IMGReady);
    if(IMGGo)
        SDL_FreeSurface(IMGGo);
    if(IMGPlay1)
        SDL_FreeSurface(IMGPlay1);
    if(IMGPlay2)
        SDL_FreeSurface(IMGPlay2);
    if(IMGCredits1)
        SDL_FreeSurface(IMGCredits1);
    if(IMGCredits2)
        SDL_FreeSurface(IMGCredits2);
    if(IMGQuit1)
        SDL_FreeSurface(IMGQuit1);
    if(IMGQuit2)
        SDL_FreeSurface(IMGQuit2);
    if(IMGHighScores1)
        SDL_FreeSurface(IMGHighScores1);
    if(IMGHighScores2)
        SDL_FreeSurface(IMGHighScores2);
    if(IMGTitleScreen)
        SDL_FreeSurface(IMGTitleScreen);
    if(IMGCredits)
        SDL_FreeSurface(IMGCredits);


    sprintf(FileName,"./skins/%s/highscores.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/highscores.png");
    IMGHighScores= SDL_DisplayFormat(Tmp);
    SDL_FreeSurface(Tmp);

    sprintf(FileName,"./skins/%s/fixedtimer1.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/fixedtimer1.png");
    IMGFixedTimer1= SDL_DisplayFormat(Tmp);
    SDL_FreeSurface(Tmp);
    SDL_SetColorKey(IMGFixedTimer1, (SDL_SRCCOLORKEY|SDL_RLEACCEL),SDL_MapRGB(IMGFixedTimer1->format,255,0,255));

    sprintf(FileName,"./skins/%s/relativetimer1.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/relativetimer1.png");
    IMGRelativeTimer1= SDL_DisplayFormat(Tmp);
    SDL_FreeSurface(Tmp);
    SDL_SetColorKey(IMGRelativeTimer1, (SDL_SRCCOLORKEY|SDL_RLEACCEL),SDL_MapRGB(IMGRelativeTimer1->format,255,0,255));


    sprintf(FileName,"./skins/%s/fixedtimer2.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/fixedtimer2.png");
    IMGFixedTimer2= SDL_DisplayFormat(Tmp);
    SDL_FreeSurface(Tmp);
    SDL_SetColorKey(IMGFixedTimer2, (SDL_SRCCOLORKEY|SDL_RLEACCEL),SDL_MapRGB(IMGFixedTimer2->format,255,0,255));

    sprintf(FileName,"./skins/%s/relativetimer2.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/relativetimer2.png");
    IMGRelativeTimer2= SDL_DisplayFormat(Tmp);
    SDL_FreeSurface(Tmp);
    SDL_SetColorKey(IMGRelativeTimer2, (SDL_SRCCOLORKEY|SDL_RLEACCEL),SDL_MapRGB(IMGRelativeTimer2->format,255,0,255));

    sprintf(FileName,"./skins/%s/selectgame.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/selectgame.png");
    IMGSelectGame= SDL_DisplayFormat(Tmp);
    SDL_FreeSurface(Tmp);
    SDL_SetColorKey(IMGSelectGame, (SDL_SRCCOLORKEY|SDL_RLEACCEL),SDL_MapRGB(IMGSelectGame->format,255,0,255));



    sprintf(FileName,"./skins/%s/credits.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/credits.png");
    IMGCredits= SDL_DisplayFormat(Tmp);
    SDL_FreeSurface(Tmp);
    SDL_SetColorKey(IMGCredits, (SDL_SRCCOLORKEY|SDL_RLEACCEL),SDL_MapRGB(IMGCredits->format,255,0,255));

    sprintf(FileName,"./skins/%s/titlescreen.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/titlescreen.png");
    IMGTitleScreen= SDL_DisplayFormat(Tmp);
    SDL_FreeSurface(Tmp);
    SDL_SetColorKey(IMGTitleScreen, (SDL_SRCCOLORKEY|SDL_RLEACCEL),SDL_MapRGB(IMGTitleScreen->format,255,0,255));

    sprintf(FileName,"./skins/%s/play1.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/play1.png");
    IMGPlay1 = SDL_DisplayFormat(Tmp);
    SDL_FreeSurface(Tmp);
    SDL_SetColorKey(IMGPlay1, (SDL_SRCCOLORKEY|SDL_RLEACCEL),SDL_MapRGB(IMGPlay1->format,255,0,255));

    sprintf(FileName,"./skins/%s/play2.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/play2.png");
    IMGPlay2 = SDL_DisplayFormat(Tmp);
    SDL_FreeSurface(Tmp);
    SDL_SetColorKey(IMGPlay2, (SDL_SRCCOLORKEY|SDL_RLEACCEL),SDL_MapRGB(IMGPlay2->format,255,0,255));

    sprintf(FileName,"./skins/%s/quit1.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/quit1.png");
    IMGQuit1 = SDL_DisplayFormat(Tmp);
    SDL_FreeSurface(Tmp);
    SDL_SetColorKey(IMGQuit1, (SDL_SRCCOLORKEY|SDL_RLEACCEL),SDL_MapRGB(IMGQuit1->format,255,0,255));

    sprintf(FileName,"./skins/%s/quit2.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/quit2.png");
    IMGQuit2 = SDL_DisplayFormat(Tmp);
    SDL_FreeSurface(Tmp);
    SDL_SetColorKey(IMGQuit2, (SDL_SRCCOLORKEY|SDL_RLEACCEL),SDL_MapRGB(IMGQuit2->format,255,0,255));

    sprintf(FileName,"./skins/%s/credits2.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/credits2.png");
    IMGCredits2 = SDL_DisplayFormat(Tmp);
    SDL_FreeSurface(Tmp);
    SDL_SetColorKey(IMGCredits2, (SDL_SRCCOLORKEY|SDL_RLEACCEL),SDL_MapRGB(IMGCredits2->format,255,0,255));

    sprintf(FileName,"./skins/%s/credits1.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/credits1.png");
    IMGCredits1 = SDL_DisplayFormat(Tmp);
    SDL_FreeSurface(Tmp);
    SDL_SetColorKey(IMGCredits1, (SDL_SRCCOLORKEY|SDL_RLEACCEL),SDL_MapRGB(IMGCredits1->format,255,0,255));

    sprintf(FileName,"./skins/%s/highscores2.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/highscores2.png");
    IMGHighScores2 = SDL_DisplayFormat(Tmp);
    SDL_FreeSurface(Tmp);
    SDL_SetColorKey(IMGHighScores2, (SDL_SRCCOLORKEY|SDL_RLEACCEL),SDL_MapRGB(IMGHighScores2->format,255,0,255));

    sprintf(FileName,"./skins/%s/highscores1.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/highscores1.png");
    IMGHighScores1 = SDL_DisplayFormat(Tmp);
    SDL_FreeSurface(Tmp);
    SDL_SetColorKey(IMGHighScores1, (SDL_SRCCOLORKEY|SDL_RLEACCEL),SDL_MapRGB(IMGHighScores1->format,255,0,255));





    sprintf(FileName,"./skins/%s/intro1.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/intro1.png");
    IMGIntro1 = SDL_DisplayFormat(Tmp);
    SDL_FreeSurface(Tmp);

    sprintf(FileName,"./skins/%s/intro2.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/intro2.png");
    IMGIntro2 = SDL_DisplayFormat(Tmp);
    SDL_FreeSurface(Tmp);

    sprintf(FileName,"./skins/%s/intro3.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/intro3.png");
    IMGIntro3 = SDL_DisplayFormat(Tmp);
    SDL_FreeSurface(Tmp);

    sprintf(FileName,"./skins/%s/timeover.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/timeover.png");
    IMGTimeOver = SDL_DisplayFormatAlpha(Tmp);
    SDL_FreeSurface(Tmp);

    sprintf(FileName,"./skins/%s/ready.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/ready.png");
    IMGReady = SDL_DisplayFormatAlpha(Tmp);
    SDL_FreeSurface(Tmp);

    sprintf(FileName,"./skins/%s/go.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/go.png");
    IMGGo = SDL_DisplayFormatAlpha(Tmp);
    SDL_FreeSurface(Tmp);

    sprintf(FileName,"./skins/%s/cursor.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/cursor.png");
    IMGCursor = SDL_DisplayFormat(Tmp);
    SDL_FreeSurface(Tmp);
    SDL_SetColorKey(IMGCursor, (SDL_SRCCOLORKEY|SDL_RLEACCEL),SDL_MapRGB(IMGCursor->format,255,0,255));

    sprintf(FileName,"./skins/%s/blocks.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/blocks.png");
    IMGBlocks = SDL_DisplayFormat(Tmp);
    SDL_FreeSurface(Tmp);
    SDL_SetColorKey(IMGBlocks, (SDL_SRCCOLORKEY|SDL_RLEACCEL),SDL_MapRGB(IMGBlocks->format,255,0,255));

    sprintf(FileName,"./skins/%s/background.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/background.png");
    IMGBackground = SDL_DisplayFormat(Tmp);
    SDL_FreeSurface(Tmp);
}

// set the volume of the music and both channels (left/right)
void SetVolume(const int VolumeIn)
{
	Mix_Volume(0,VolumeIn);
	Mix_Volume(1,VolumeIn);
	Mix_VolumeMusic(VolumeIn);
}

// increase the volume with 8
void IncVolume()
{
	if (Volume < 128)
	{
		Volume += 8;
		SetVolume(Volume);
	}
}

// decrease the volume with 8
void DecVolume()
{
	if (Volume > 0)
	{
		Volume-= 8;
		SetVolume(Volume);
	}
}

void LoadSettings()
{
 	FILE *SettingsFile;
 	int Value=1,Teller=0;
 	SettingsFile = fopen("./settings.dat","r");
 	if(SettingsFile)
 	{
		fscanf(SettingsFile,"Volume=%d\n",&Volume);
		SetVolume(Volume);
		if(TvOutMode)
		{
            fscanf(SettingsFile,"StartScreenX=%d\n",&StartScreenX);
            fscanf(SettingsFile,"StartScreenY=%d\n",&StartScreenY);
		}
		else
        {
            fscanf(SettingsFile,"StartScreenX=%d\n",&InitialStartScreenX);
            fscanf(SettingsFile,"StartScreenY=%d\n",&InitialStartScreenY);
            StartScreenX = 0;
            StartScreenY = 0;
        }
        fscanf(SettingsFile,"Skin=%s\n",&SkinName);
        for(Teller=0;Teller<InstalledSkinsCount;Teller++)
        {
            if (strcmp(InstalledSkins[Teller],SkinName) == 0)
            {
                SelectedSkin = Teller;
                break;
            }
            else
                SelectedSkin = -1;
        }
		fclose(SettingsFile);
 	}
 	else
 	{
		SetVolume(128);
		StartScreenX = (Screen->w - Buffer->w) / 2;
        StartScreenY = (Screen->h - Buffer->h) / 2;
	}
}

void SaveSettings()
{
 	FILE *SettingsFile;
 	SettingsFile = fopen("./settings.dat","w");
 	if(SettingsFile)
 	{
		fprintf(SettingsFile,"Volume=%d\n",Volume);
        if(TvOutMode)
        {
            fprintf(SettingsFile,"StartScreenX=%d\n",StartScreenX);
            fprintf(SettingsFile,"StartScreenY=%d\n",StartScreenY);
        }
        else
        {
            fprintf(SettingsFile,"StartScreenX=%d\n",InitialStartScreenX);
            fprintf(SettingsFile,"StartScreenY=%d\n",InitialStartScreenY);
        }
        fprintf(SettingsFile,"Skin=%s\n",SkinName);
		fclose(SettingsFile);
		sync();
 	}
}


void WriteText(SDL_Surface* Surface,TTF_Font* FontIn,char* Tekst,int NrOfChars,int X,int Y,int YSpacing,SDL_Color ColorIn)
{
	char List[100][255];
	int Lines,Teller,Chars;
	SDL_Rect DstRect;
	SDL_Surface *TmpSurface1,*TmpSurface2;
	Lines = 0;
	Chars = 0;
	for (Teller=0;Teller<NrOfChars;Teller++)
	{
		if(Lines > 100)
			break;
		if((Tekst[Teller] == '\n') || (Chars==255))
		{
			List[Lines][Chars]='\0';
			Lines++;
			Chars = 0;
		}
		else
		{
		 	List[Lines][Chars]=Tekst[Teller];
		 	Chars++;
		}
	}
	List[Lines][Chars] = '\0';
	for (Teller=0;Teller <= Lines;Teller++)
	{
		if(strlen(List[Teller]) > 0)
		{
			TmpSurface1 = TTF_RenderText_Blended(FontIn,List[Teller],ColorIn);
			if (TmpSurface1)
			{
				DstRect.x = X;
				DstRect.y = Y + (Teller) * TTF_FontLineSkip(FontIn) + (Teller*YSpacing);
				DstRect.w = TmpSurface1->w;
				DstRect.h = TmpSurface1->h;
				SDL_BlitSurface(TmpSurface1,NULL,Surface,&DstRect);
				SDL_FreeSurface(TmpSurface1);
			}
		}
	}
}

void NextSkin()
{
    char Text[500];
    SDL_Color Color = {255,255,255,255};
    SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
    if(InstalledSkinsCount > 0)
    {
        SelectedSkin++;
        if (SelectedSkin >=InstalledSkinsCount)
            SelectedSkin = -1;
        if(SelectedSkin == -1)
        {
            sprintf(SkinName,"%s","");
            sprintf(Text,"Please Wait Loading Skin:\nDefault Skin");
        }
        else
        {
            sprintf(SkinName,"%s",InstalledSkins[SelectedSkin]);
            sprintf(Text,"Please Wait Loading Skin:\n%s",SkinName);
        }
        boxRGBA(Buffer,0,0,319,48,255,255,255,200);
        boxRGBA(Buffer,1,1,318,47,0,0,0,200);
        WriteText(Buffer,BigFont,Text,strlen(Text),5,0,0,Color);
        SDL_FillRect(Buffer1,NULL,SDL_MapRGB(Buffer1->format,0,0,0));
        SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
       /* #ifdef WIN32
            ScreenBufferZoom = zoomSurface(Buffer1,2,2,0);
            SDL_BlitSurface(ScreenBufferZoom,NULL,Screen,NULL);
            SDL_FreeSurface(ScreenBufferZoom);
        #else*/
        SDL_BlitSurface(Buffer1,NULL,Screen,NULL);
       // #endif
        SDL_Flip(Screen);
        LoadGraphics();
    }
}

void PreviousSkin()
{
    char Text[500];
    SDL_Color Color = {255,255,255,255};
    SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
    if(InstalledSkinsCount > 0)
    {
        SelectedSkin--;
        if (SelectedSkin <=-2)
            SelectedSkin = InstalledSkinsCount-1;
        if(SelectedSkin == -1)
        {
            sprintf(SkinName,"%s","");
            sprintf(Text,"Please Wait Loading Skin:\nDefault Skin");
        }
        else
        {
            sprintf(SkinName,"%s",InstalledSkins[SelectedSkin]);
            sprintf(Text,"Please Wait Loading Skin:\n%s",SkinName);
        }
        boxRGBA(Buffer,0,0,319,48,255,255,255,200);
        boxRGBA(Buffer,1,1,318,47,0,0,0,200);
        WriteText(Buffer,BigFont,Text,strlen(Text),5,0,0,Color);
        SDL_FillRect(Buffer1,NULL,SDL_MapRGB(Buffer1->format,0,0,0));
        SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
       /* #ifdef WIN32
            ScreenBufferZoom = zoomSurface(Buffer1,2,2,0);
            SDL_BlitSurface(ScreenBufferZoom,NULL,Screen,NULL);
            SDL_FreeSurface(ScreenBufferZoom);
        #else*/
        SDL_BlitSurface(Buffer1,NULL,Screen,NULL);
       // #endif
        SDL_Flip(Screen);
        LoadGraphics();
    }
}

char chr(int ascii)
{
	return((char)ascii);
}

int ord(char chr)
{
	return((int)chr);
}


void GameTypeMenu()
{
    SDL_Rect Rect;
    SDL_Event Event;
    Uint8 alpha = 0;
    SDL_Surface *Tmp;
    Tmp = SDL_DisplayFormat(Buffer);
    CGameTypeMenu *Menu = new CGameTypeMenu();
    while (GameState == GSGameTypeMenu)
    {
        if(MusicCount > 0)
            if(GlobalSoundEnabled)
                if(!Mix_PlayingMusic())
                {
                    SelectedMusic = 0;
                    Mix_PlayMusic(Music[SelectedMusic],0);
                    SetVolume(Volume);
                }
        /* if(GlobalSoundEnabled)
            if(!Mix_PlayingMusic())
                Mix_PlayMusic(Music[MUS_TITLE],0);*/
        while (SDL_PollEvent(&Event))
		{

            if (Event.type == SDL_JOYBUTTONDOWN)
                switch (Event.jbutton.button)
                {
                   case GP2X_BUTTON_L:
                        NextSkin();
                        break;
                   case GP2X_BUTTON_VOLUP:
						IncVolume();
						break;
					case GP2X_BUTTON_VOLDOWN:
						DecVolume();
						break;
                    case GP2X_BUTTON_DOWN:
                        Menu->PreviousItem();
                        break;
                    case GP2X_BUTTON_UP:
                        Menu->NextItem();
                        break;
                    case GP2X_BUTTON_B:
                    case GP2X_BUTTON_X:
                    case GP2X_BUTTON_Y:
                    case GP2X_BUTTON_A:
                    case GP2X_BUTTON_START:
                        if (GlobalSoundEnabled)
                            Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
                        switch(Menu->GetSelection())
                        {
                            case Fixed:
                                GameType = Fixed;
                                break;
                            case Relative:
                                GameType = Relative;
                                break;
                        }
                        GameState = GSReadyGo;
                        break;
                    case GP2X_BUTTON_SELECT:
                        GameState = GSTitleScreen;
                        break;
                }
        }
        Menu->Draw(Tmp);
        //tekenen naar buffer
        Rect.w = Buffer->w;
        Rect.h = Buffer->h;
        Rect.x = StartScreenX;
        Rect.y = StartScreenY;
        if (alpha < 255)
        {
            if(alpha+AlphaIncrease > MaxAlpha)
            {
                alpha = 255;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
            else
            {
                alpha+=10;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
        }
        SDL_BlitSurface(Tmp,NULL,Buffer,NULL);
        SDL_FillRect(Buffer1,NULL,SDL_MapRGB(Buffer1->format,0,0,0));
        SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
        SDL_BlitSurface(Buffer1,NULL,Screen,NULL);
        SDL_Flip(Screen);
    }
    delete Menu;
    SDL_FreeSurface(Tmp);
}


void Credits()
{
    SDL_Rect Rect,Rect1;
    SDL_Event Event;
    Uint8 alpha = 0;
    Uint32 Time = 0;
    int CreditsInfoNr = 1;
    SDL_Surface *Tmp;
    Tmp = SDL_DisplayFormat(Buffer);
    Rect1.x = 48;
    Rect1.y = 76;
    Rect1.w = IMGCredits->w;
    Rect1.h = IMGCredits->h;
    while (GameState == GSCredits)
    {
        if(MusicCount > 0)
            if(GlobalSoundEnabled)
                if(!Mix_PlayingMusic())
                {
                    SelectedMusic = 0;
                    Mix_PlayMusic(Music[SelectedMusic],0);
                    SetVolume(Volume);
                }
        /*if(GlobalSoundEnabled)
            if(!Mix_PlayingMusic())
                Mix_PlayMusic(Music[MUS_TITLE],0);*/
        while (SDL_PollEvent(&Event))
		{

            if (Event.type == SDL_QUIT)
                GameState=GSQuit;

            if (Event.type == SDL_JOYBUTTONDOWN)
                switch (Event.jbutton.button)
                {
                    case GP2X_BUTTON_L:
                        NextSkin();
                        break;
                    case GP2X_BUTTON_VOLUP:
						IncVolume();
						break;
					case GP2X_BUTTON_VOLDOWN:
						DecVolume();
						break;
                    default :
                        GameState = GSTitleScreen;
                        break;
                }

        }

        //tekenen naar buffer


        SDL_BlitSurface(IMGTitleScreen,NULL,Tmp,NULL);
        SDL_BlitSurface(IMGCredits,NULL,Tmp,&Rect1);

        if (alpha < 255)
        {
            if(alpha+AlphaIncrease > MaxAlpha)
            {
                alpha = 255;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
            else
            {
                alpha+=10;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
        }




        Rect.w = Buffer->w;
        Rect.h = Buffer->h;
        Rect.x = StartScreenX;
        Rect.y = StartScreenY;
        SDL_FillRect(Buffer1,NULL,SDL_MapRGB(Buffer1->format,0,0,0));
        SDL_BlitSurface(Tmp,NULL,Buffer,NULL);

        SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
      /* #ifdef WIN32
            ScreenBufferZoom = zoomSurface(Buffer1,2,2,0);
            SDL_BlitSurface(ScreenBufferZoom,NULL,Screen,NULL);
            SDL_FreeSurface(ScreenBufferZoom);
        #else*/
        SDL_BlitSurface(Buffer1,NULL,Screen,NULL);
       // #endif
        SDL_Flip(Screen);
    }
    SDL_FreeSurface(Tmp);

}

void TitleScreen()
{
    SDL_Rect Rect;
    SDL_Event Event;
    bool RPressed = false;
    SDL_Surface *Tmp;
    Tmp = SDL_DisplayFormat(Buffer);
    CMainMenu *Menu = new CMainMenu;
    Uint8 alpha = 0;
    while (GameState == GSTitleScreen)
    {
       if(MusicCount > 0)
            if(GlobalSoundEnabled)
                if(!Mix_PlayingMusic())
                {
                    SelectedMusic = 0;
                    Mix_PlayMusic(Music[SelectedMusic],0);
                    SetVolume(Volume);
                }
        while (SDL_PollEvent(&Event))
		{
		    if (Event.type == SDL_JOYBUTTONUP)
                switch (Event.jbutton.button)
                {
                    case GP2X_BUTTON_R:
                        RPressed = false;
                        break;
                    case GP2X_BUTTON_LEFT:
                        if (RPressed)
                            if(StartScreenX - 2 >= 0)
                                StartScreenX -=2;
                        break;
                    case GP2X_BUTTON_RIGHT:
                        if (RPressed)
                            if(StartScreenX + 2 + Buffer->w <= Screen->w)
                                StartScreenX +=2;
                        break;
                    case GP2X_BUTTON_UP:
                        if(RPressed)
                            if(StartScreenY - 2 >=0)
                                StartScreenY -=2;
                        break;
                    case GP2X_BUTTON_DOWN:
                        if(RPressed)
                            if(StartScreenY + 2 + Buffer->h <= Screen->h)
                                StartScreenY +=2;
                        break;
#ifdef SCREENSHOTS
                    case GP2X_BUTTON_CLICK:
                        SDL_SaveBMP (Screen,"./znax.bmp");
                        sync();
                        break;
#endif
                }

            if (Event.type == SDL_JOYBUTTONDOWN)
                switch (Event.jbutton.button)
                {
                    case GP2X_BUTTON_L:
                        NextSkin();
                        break;
                    case GP2X_BUTTON_R:
                        RPressed=true;
                        break;
                    case GP2X_BUTTON_VOLUP:
						IncVolume();
						break;
					case GP2X_BUTTON_VOLDOWN:
						DecVolume();
						break;
                    case GP2X_BUTTON_UP:
                        Menu->PreviousItem();
                        break;
                    case GP2X_BUTTON_DOWN:
                        Menu->NextItem();
                        break;
                    case GP2X_BUTTON_B:
                    case GP2X_BUTTON_X:
                    case GP2X_BUTTON_Y:
                    case GP2X_BUTTON_A:
                    case GP2X_BUTTON_START:
                        if(Menu->GetSelection() != 4)
                            if (GlobalSoundEnabled)
                                Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
                        switch(Menu->GetSelection())
                        {
                            case 1:
                                GameState = GSGameTypeMenu;
                                break;
                            case 2:
                                GameState = GSShowHighScores;
                                break;
                            case 3:
                                GameState = GSCredits;
                                break;
                            case 4:
                                GameState = GSQuit;
                                Mix_HaltMusic();
                                if (GlobalSoundEnabled)
                                    Mix_PlayChannel(-1,Sounds[SND_GOODBYE],0);
                                SDL_Delay(750);
                                break;
                        }
                        break;
                    case GP2X_BUTTON_SELECT:
                        GameState = GSQuit;
                        Mix_HaltMusic();
                        if (GlobalSoundEnabled)
                            Mix_PlayChannel(-1,Sounds[SND_GOODBYE],0);
                        SDL_Delay(750);
                        break;
                }
        }
        Menu->Draw(Tmp);
        //tekenen naar buffer
        Rect.w = Buffer->w;
        Rect.h = Buffer->h;
        Rect.x = StartScreenX;
        Rect.y = StartScreenY;
        if (alpha < 255)
        {
            if(alpha+AlphaIncrease > MaxAlpha)
            {
                alpha = 255;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
            else
            {
                alpha+=10;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
        }
        SDL_BlitSurface(Tmp,NULL,Buffer,NULL);
        SDL_FillRect(Buffer1,NULL,SDL_MapRGB(Buffer1->format,0,0,0));

        SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);

        SDL_BlitSurface(Buffer1,NULL,Screen,NULL);
        SDL_Flip(Screen);


    }
    delete Menu;
    SDL_FreeSurface(Tmp);
}

void GetHighScoreName(char NameIn[21],int Place, long PScore)
{
	char Name[21];
	char Msg[500];
	bool End=false,SubmitChanges=false;
	int Teller,MaxSelection=0, Selection = 0,asci=97;
	SDL_Event Event;
	SDL_Rect Rect;
	SDL_Color TextColor = {102,115,152,255}, TextColor1 = {255,115,152,255};;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
	SDL_PollEvent(&Event);
	sprintf(Name,"%s",NameIn);
	MaxSelection = strlen(NameIn);
	Name[Selection+1]='\0';
	//sprintf(Msg,Congratulations
	if (MaxSelection == 0)
		Name[MaxSelection]=chr(asci);
	char Tekst[100];
	while (!End)
	{
        if(MusicCount > 0)
            if(GlobalSoundEnabled)
                if(!Mix_PlayingMusic())
                {
                    SelectedMusic = 0;
                    Mix_PlayMusic(Music[SelectedMusic],0);
                    SetVolume(Volume);
                }
	   /* if(GlobalSoundEnabled)
		if(!Mix_PlayingMusic())
        {
            Mix_PlayMusic(Music[SelectedMusic],0);
            SetVolume(Volume);
        }*/
		while (SDL_PollEvent(&Event))
		{
			if (Event.type == SDL_JOYBUTTONDOWN)
			{
				switch(Event.jbutton.button)
				{
					case GP2X_BUTTON_LEFT:
						if (Selection > 0)
						{	Selection--;
							asci = ord(Name[Selection]);
						}
						break;
					case GP2X_BUTTON_RIGHT:
						if (Selection < 19)
						{
							Selection++;
							if (Selection > MaxSelection)
							{
								Name[Selection] = chr(97);
								Name[Selection+1] = '\0';
								MaxSelection=Selection;
							}
							asci = ord(Name[Selection]);
						}
						break;
					case GP2X_BUTTON_UP:
						asci++;
						if (asci==123)
							asci=32;
						if (asci==33)
							(asci=48);
						if (asci==58)
							asci=97;
							Name[Selection] = chr(asci);
						break;
					case GP2X_BUTTON_DOWN:
						asci--;
						if(asci==96)
								asci=57;
						if(asci==47)
							asci=32;
						if(asci==31)
							asci=122;
						Name[Selection] = chr(asci);
						break;
					case GP2X_BUTTON_A:
						if (GlobalSoundEnabled)
							Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
						End = true;
						SubmitChanges=true;
						break;
                    case GP2X_BUTTON_SELECT:
					case GP2X_BUTTON_X:
						End=true;
						SubmitChanges=false;
						break;
                    case GP2X_BUTTON_L:
                        NextSkin();
                        break;
                    case GP2X_BUTTON_VOLUP:
						IncVolume();
						break;
					case GP2X_BUTTON_VOLDOWN:
						DecVolume();
						break;
				}
			}

#ifdef SCREENSHOTS
            if (Event.type == SDL_JOYBUTTONUP)
                switch(Event.jbutton.button)
                {
                    case GP2X_BUTTON_CLICK:
                        SDL_SaveBMP (Screen,"./znax.bmp");
                        sync();
                        break;
                }
#endif
		}
        SDL_BlitSurface(IMGHighScores,NULL,Buffer,NULL);

        for(Teller = 0;Teller<9;Teller++)
        {
            if(Teller < Place)
            {
                sprintf(Msg,"%2d.%s",Teller+1,HighScores[GameType][Teller].PName);
                WriteText(Buffer,MonoFont,Msg,strlen(Msg),50,62+Teller*16,0,TextColor);
                sprintf(Msg,"%7d",HighScores[GameType][Teller].PScore);
                WriteText(Buffer,MonoFont,Msg,strlen(Msg),220,62+Teller*16,0,TextColor);
            }
            else
            {
                sprintf(Msg,"%2d.%s",Teller+2,HighScores[GameType][Teller].PName);
                WriteText(Buffer,MonoFont,Msg,strlen(Msg),50,62+(Teller+1)*16,0,TextColor);
                sprintf(Msg,"%7d",HighScores[GameType][Teller].PScore);
                WriteText(Buffer,MonoFont,Msg,strlen(Msg),220,62+(Teller+1)*16,0,TextColor);
            }
        }
		sprintf(Msg,"%2d.%s",Place+1,Name);
		WriteText(Buffer,MonoFont,Msg,strlen(Msg),50,62+Place*16,0,TextColor1);
		sprintf(Msg,"%7d",PScore);
        WriteText(Buffer,MonoFont,Msg,strlen(Msg),220,62+Place*16,0,TextColor1);
		if (Selection > 0)
		{
			sprintf(Tekst,"    ");
			for (Teller=1;Teller<Selection;Teller++)
				sprintf(Tekst,"%s ",Tekst);
			sprintf(Tekst,"%s_",Tekst);
		}
		else
			sprintf(Tekst,"   _");
		WriteText(Buffer,MonoFont,Tekst,strlen(Tekst),50,(62+(Place)*16),2,TextColor1);
		sprintf(Tekst,"Use Up,Down,Left,right. A = Ok X = Cancel" );
		WriteText(Buffer,font,Tekst,strlen(Tekst),45,227,0,TextColor);
        SDL_FillRect(Buffer1,NULL,SDL_MapRGB(Buffer1->format,0,0,0));
        SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
        SDL_BlitSurface(Buffer1,NULL,Screen,NULL);
        SDL_Flip(Screen);
	}
	Name[MaxSelection+1] = '\0';
	while ((Name[0] == ' ') && (MaxSelection>-1))
	{
		for (Teller=0;Teller<MaxSelection;Teller++)
			Name[Teller] = Name[Teller+1];
		MaxSelection--;
	}
	if (MaxSelection>-1)
		while ((Name[MaxSelection] == ' ') && (MaxSelection>0))
		{
			Name[MaxSelection] = '\0';
			MaxSelection--;
		}
	if (!SubmitChanges)
		sprintf(NameIn,"%s"," ");
    else
        sprintf(NameIn,"%s",Name);
}


void TimeOver()
{
    char Name[21];
    int X,Y,Teller1,Teller2;
    SDL_Rect Rect,Dst;
    SDL_Event Event;
    SDL_Surface *Tmp;
    Uint8 alpha = 255;
    Uint32 Time2 = SDL_GetTicks() + 1250;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
    Tmp = SDL_DisplayFormat(Buffer);
    SDL_Color TextColor = {255,255,255,255};
    int AddToScore = 0,Timer = 0,Counter=0;
    char Text[500];
    World->DeSelect(false);

    while (GameState == GSTimeOver)
    {
        while (SDL_PollEvent(&Event))
		{
			if (Event.type == SDL_JOYBUTTONDOWN)
			{
				switch (Event.jbutton.button)
				{
					case GP2X_BUTTON_SELECT:
						GameState = GSTitleScreen;
						break;
                    case GP2X_BUTTON_L:
                        NextSkin();
                        break;
                    case GP2X_BUTTON_VOLUP:
						IncVolume();
						break;
					case GP2X_BUTTON_VOLDOWN:
						DecVolume();
						break;
				}
			}
		}
		World->Draw(Tmp);
		switch(Counter)
		{
		    case 0 :
                Dst.x = 23;
                Dst.y = 85;
                Dst.w = IMGTimeOver->w;
                Dst.h = IMGTimeOver->h;
                SDL_BlitSurface(IMGTimeOver,NULL,Tmp,&Dst);
                break;
		}
		if (alpha < 255)
        {
            if(alpha+AlphaIncrease > MaxAlpha)
            {
                alpha = 255;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
            else
            {
                alpha+=10;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
        }
        sprintf(Text,"Timer:\n%2d:%02d\n\nScore:\n%d",Timer/60,Timer%60,Score);
        WriteText(Tmp,BigFont,Text,strlen(Text),247,10,0,TextColor);
        SDL_FillRect(Buffer1,NULL,SDL_MapRGB(Buffer1->format,0,0,0));
        SDL_BlitSurface(Tmp,NULL,Buffer,NULL);
        SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
        SDL_BlitSurface(Buffer1,NULL,Screen,NULL);
        SDL_Flip(Screen);
        if (Time2 < SDL_GetTicks())
        {
            GameState = GSShowHighScores;
        }

    }
    for(Teller1 =0;Teller1<10;Teller1++)
        if(HighScores[GameType][Teller1].PScore < Score)
        {
            GetHighScoreName(Name,Teller1,Score);
            for(Teller2=8;Teller2>=Teller1;Teller2--)
                HighScores[GameType][Teller2+1] = HighScores[GameType][Teller2];
            if((strcmp(Name," ") == 0))
                sprintf(HighScores[GameType][Teller1].PName,"%s","anonymous");
            else
                sprintf(HighScores[GameType][Teller1].PName,"%s",Name);
            HighScores[GameType][Teller1].PScore = Score;
            break;
        }
    SDL_FreeSurface(Tmp);
}


void ReadyGo()
{
    SDL_Rect Rect,Dst;
    SDL_Event Event;
    SDL_Surface *Tmp;
    Uint8 alpha = 0;
    Uint32 Time = SDL_GetTicks(),Time2 = SDL_GetTicks()+500;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
    World->NewGame();
    Score = 0;
    Tmp = SDL_DisplayFormat(Buffer);
    SDL_Color TextColor = {255,255,255,255};
    int AddToScore = 0,Timer = 600,Counter=0;
#ifdef DEBUG
    int FrameCount=0,Fps=0;
#endif
    char Text[500];
    if(GameType == Relative)
        Timer = 200;
    else
        Timer = 600;
    if (MusicCount > 1)
    {
        SelectedMusic =	1+rand()%(MusicCount-1);
        Mix_PlayMusic(Music[SelectedMusic],0);
        SetVolume(Volume);
    }
    else
        if(MusicCount ==1)
        {
            if(!Mix_PlayingMusic())
            {
                SelectedMusic =	0;
                Mix_PlayMusic(Music[SelectedMusic],0);
                SetVolume(Volume);
            }
        }
    while (GameState == GSReadyGo)
    {
        if(!Mix_PlayingMusic())
        if(GlobalSoundEnabled)
        {
            if (MusicCount > 1)
            {
                SelectedMusic =	1+rand()%(MusicCount-1);
                Mix_PlayMusic(Music[SelectedMusic],0);
                SetVolume(Volume);
            }
            else
                if(MusicCount ==1)
                {
                    SelectedMusic =	0;
                    Mix_PlayMusic(Music[SelectedMusic],0);
                    SetVolume(Volume);
                }

        }
        while (SDL_PollEvent(&Event))
		{
			if (Event.type == SDL_JOYBUTTONDOWN)
			{
				switch (Event.jbutton.button)
				{
					case GP2X_BUTTON_SELECT:
						GameState = GSTitleScreen;
						break;
                    case GP2X_BUTTON_L :
                        NextSkin();
                        break;
                    case GP2X_BUTTON_VOLUP:
						IncVolume();
						break;
					case GP2X_BUTTON_VOLDOWN:
						DecVolume();
						break;
				}
			}
		}
		World->Draw(Tmp);
		switch(Counter)
		{
		    case 1 :
                Dst.x = 46;
                Dst.y = 85;
                Dst.w = IMGReady->w;
                Dst.h = IMGReady->h;
                SDL_BlitSurface(IMGReady,NULL,Tmp,&Dst);
                break;
            case 2 :
                Dst.x = 93;
                Dst.y = 85;
                Dst.w = IMGGo->w;
                Dst.h = IMGGo->h;
                SDL_BlitSurface(IMGGo,NULL,Tmp,&Dst);
                break;
            case 3 :
                GameState = GSGame;
                break;
		}
		if (alpha < 255)
        {
            if(alpha+AlphaIncrease > MaxAlpha)
            {
                alpha = 255;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
            else
            {
                alpha+=10;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
        }
#ifdef DEBUG
        sprintf(Text,"FPS : %d",Fps);
        WriteText(Tmp,BigFont,Text,strlen(Text),247,140,0,TextColor);
#endif
        sprintf(Text,"Timer:\n%2d:%02d\n\nScore:\n%d",Timer/60,Timer%60,Score);
        WriteText(Tmp,BigFont,Text,strlen(Text),247,10,0,TextColor);
        SDL_FillRect(Buffer1,NULL,SDL_MapRGB(Buffer1->format,0,0,0));
        SDL_BlitSurface(Tmp,NULL,Buffer,NULL);
        SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
        SDL_BlitSurface(Buffer1,NULL,Screen,NULL);
        SDL_Flip(Screen);
#ifdef DEBUG
        FrameCount++;
        if (Time +1000 < SDL_GetTicks())
        {
            Fps = FrameCount;
            FrameCount = 0;
            Time = SDL_GetTicks();
        }
#endif
        if (Time2 < SDL_GetTicks())
        {
            Counter++;
            if(Counter == 1)
                if(GlobalSoundEnabled)
                    Mix_PlayChannel(-1,Sounds[SND_READYGO],0);
            if (Counter == 2)
            {
                Time2 = SDL_GetTicks() + 400;
            }
            else
                Time2 = SDL_GetTicks() + 900;
        }

    }
    SDL_FreeSurface(Tmp);
}

void Intro()
{
    SDL_Rect Rect;
    SDL_Event Event;
    Uint8 alpha = 0;
    Uint32 Time=0;
    bool RPressed=false;
    int IntroScreenNr = 1;
    SDL_Surface *Tmp;
    Tmp = SDL_DisplayFormat(Buffer);
    Time = SDL_GetTicks();
    while (GameState == GSIntro)
    {
        while (SDL_PollEvent(&Event))
		{
            if (Event.type == SDL_JOYBUTTONUP)
                switch (Event.jbutton.button)
                {
                    case GP2X_BUTTON_R:
                        RPressed = false;
                        break;
                    case GP2X_BUTTON_LEFT:
                        if (RPressed)
                            if(StartScreenX - 2 >= 0)
                                StartScreenX -=2;
                        break;
                    case GP2X_BUTTON_RIGHT:
                        if (RPressed)
                            if(StartScreenX + 2 + Buffer->w <= Screen->w)
                                StartScreenX +=2;
                        break;
                    case GP2X_BUTTON_UP:
                        if(RPressed)
                            if(StartScreenY - 2 >=0)
                                StartScreenY -=2;
                        break;
                    case GP2X_BUTTON_DOWN:
                        if(RPressed)
                            if(StartScreenY + 2 + Buffer->h <= Screen->h)
                                StartScreenY +=2;
                        break;
                }

            if (Event.type == SDL_JOYBUTTONDOWN)
                switch (Event.jbutton.button)
                {
                    case GP2X_BUTTON_L:
                        NextSkin();
                        break;
                    case GP2X_BUTTON_R:
                        RPressed = true;
                        break;
                   case GP2X_BUTTON_VOLUP:
						IncVolume();
						break;
					case GP2X_BUTTON_VOLDOWN:
						DecVolume();
						break;
                    case GP2X_BUTTON_A:
                    case GP2X_BUTTON_B:
                    case GP2X_BUTTON_X:
                    case GP2X_BUTTON_Y:
                    case GP2X_BUTTON_SELECT:
                    case GP2X_BUTTON_START:
                        GameState = GSTitleScreen;
                        break;
                }
        }

        //tekenen naar buffer

        switch(IntroScreenNr)
        {
            case 1 :
                SDL_BlitSurface(IMGIntro1,NULL,Tmp,NULL);
                break;
            case 2 :
                SDL_BlitSurface(IMGIntro2,NULL,Tmp,NULL);
                break;
            case 3 :
                SDL_BlitSurface(IMGIntro3,NULL,Tmp,NULL);
                break;
        }

        if (alpha < 255)
        {
            if(alpha+AlphaIncrease > MaxAlpha)
            {
                alpha = 255;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
            else
            {
                alpha+=AlphaIncrease;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
        }




        Rect.w = Buffer->w;
        Rect.h = Buffer->h;
        Rect.x = StartScreenX;
        Rect.y = StartScreenY;


        SDL_FillRect(Buffer1,NULL,SDL_MapRGB(Buffer1->format,0,0,0));
        SDL_BlitSurface(Tmp,NULL,Buffer,NULL);
        SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);

        SDL_BlitSurface(Buffer1,NULL,Screen,NULL);
        SDL_Flip(Screen);

        if(Time + 3700 < SDL_GetTicks())
        {
            alpha = 0;
            IntroScreenNr++;
            if(IntroScreenNr > 3)
                GameState = GSTitleScreen;
            Time = SDL_GetTicks();
        }

    }
    SDL_FreeSurface(Tmp);
    if (GlobalSoundEnabled)
        Mix_PlayChannel(-1,Sounds[SND_WELCOME],0);
}



void Game()
{
    CSelector *Selector;
    SDL_Rect Rect;
    SDL_Event Event;
    SDL_Surface *Tmp;
    Uint8 alpha = 255;
    Uint32 Time = SDL_GetTicks();
    Uint32 ScoreTimer = 0;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
    Selector = new CSelector(5,5);
    Tmp = SDL_DisplayFormat(Buffer);
    SDL_Color TextColor = {255,255,255,255};
    int AddToScore = 0,Timer = 150;
#ifdef DEBUG
    int FrameCount=0,Fps=0;
#endif
    char Text[500];
    if(GameType == Relative)
        Timer =200;
    else
        Timer = 600;
    while (GameState == GSGame)
    {
        if(!Mix_PlayingMusic())
        if(GlobalSoundEnabled)
        {
            if (MusicCount > 1)
            {
                SelectedMusic =	1+rand()%(MusicCount-1);
                Mix_PlayMusic(Music[SelectedMusic],0);
                SetVolume(Volume);
            }
            else
                if(MusicCount ==1)
                {
                    SelectedMusic =	0;
                    Mix_PlayMusic(Music[SelectedMusic],0);
                    SetVolume(Volume);
                }

        }
        while (SDL_PollEvent(&Event))
		{
			if (Event.type == SDL_JOYBUTTONDOWN)
			{
				switch (Event.jbutton.button)
				{
					case GP2X_BUTTON_SELECT:
						GameState = GSTitleScreen;
						break;
                    case GP2X_BUTTON_A :
                    case GP2X_BUTTON_B :
                    case GP2X_BUTTON_X :
                    case GP2X_BUTTON_Y :
                        AddToScore+= World->Select(Selector->GetPosition().X,Selector->GetPosition().Y);
                        if(AddToScore != 0)
                        {
                            ScoreTimer = SDL_GetTicks() + 700;
                        }
                        break;
                     case GP2X_BUTTON_LEFT:
                        Selector->SetPosition(Selector->GetPosition().X -1,Selector->GetPosition().Y);
                        break;
                    case GP2X_BUTTON_RIGHT:
                        Selector->SetPosition(Selector->GetPosition().X +1,Selector->GetPosition().Y);
                        break;
                    case GP2X_BUTTON_UP:
                        Selector->SetPosition(Selector->GetPosition().X,Selector->GetPosition().Y-1);
                        break;
                    case GP2X_BUTTON_DOWN:
                        Selector->SetPosition(Selector->GetPosition().X,Selector->GetPosition().Y+1);
                        break;
                    case GP2X_BUTTON_L:
                        NextSkin();
                        break;
                    case GP2X_BUTTON_R:
						if (MusicCount > 1)
                        {
                            SelectedMusic++;
                            if (SelectedMusic >= MusicCount)
                                SelectedMusic = 1;
                            if(GlobalSoundEnabled)
                            {
                                Mix_HaltMusic();
                                Mix_PlayMusic(Music[SelectedMusic],0);
                                //Mix_HookMusicFinished(MusicFinished);
                                SetVolume(Volume);
                            }
                        }
						break;
                    case GP2X_BUTTON_VOLUP:
						IncVolume();
						break;
					case GP2X_BUTTON_VOLDOWN:
						DecVolume();
						break;
                    //case GP2X_BUTTON_B :
				}
			}
#ifdef SCREENSHOTS
            if (Event.type == SDL_JOYBUTTONUP)
                switch(Event.jbutton.button)
                {
                    case GP2X_BUTTON_CLICK:
                        SDL_SaveBMP (Screen,"./znax.bmp");
                        sync();
                        break;
                }
#endif
		}
		World->Draw(Tmp);
        Selector->Draw(Tmp);
		if (alpha < 255)
        {
            if(alpha+AlphaIncrease > MaxAlpha)
            {
                alpha = 255;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
            else
            {
                alpha+=10;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
        }
        if(AddToScore !=0)
        {
            if(ScoreTimer > SDL_GetTicks())
            {
                sprintf(Text,"+%d",AddToScore);
                WriteText(Tmp,font,Text,strlen(Text),247,115,0,TextColor);
                if(GameType == Relative)
                {
                    sprintf(Text,"+%d",AddToScore/200);
                    WriteText(Tmp,font,Text,strlen(Text),247,50,0,TextColor);
                }
            }
            else
            {
                ScoreTimer = 0;
                Score +=AddToScore;
                if (GameType == Relative)
                    Timer += AddToScore / 200;
                AddToScore = 0;
            }
        }
#ifdef DEBUG
        sprintf(Text,"FPS:\n%d",Fps);
        WriteText(Tmp,BigFont,Text,strlen(Text),247,140,0,TextColor);
#endif
        sprintf(Text,"Timer:\n%2d:%02d\n\nScore:\n%d",Timer/60,Timer%60,Score);
        WriteText(Tmp,BigFont,Text,strlen(Text),247,10,0,TextColor);
        SDL_FillRect(Buffer1,NULL,SDL_MapRGB(Buffer1->format,0,0,0));
        SDL_BlitSurface(Tmp,NULL,Buffer,NULL);
        SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
        SDL_BlitSurface(Buffer1,NULL,Screen,NULL);
        SDL_Flip(Screen);
        //FrameCount++;
        if (Time +1000 < SDL_GetTicks())
        {
            Timer-= 1;
            switch (Timer)
            {
                case 300 :
                    if (GlobalSoundEnabled)
                        Mix_PlayChannel(-1,Sounds[SND_5MINUTE],0);
                    break;
                case 60:
                    if (GlobalSoundEnabled)
                        Mix_PlayChannel(-1,Sounds[SND_1MINUTE],0);
                    break;
                case 3 :
                    if (GlobalSoundEnabled)
                        Mix_PlayChannel(-1,Sounds[SND_3],0);
                    break;
                case 2 :
                    if (GlobalSoundEnabled)
                        Mix_PlayChannel(-1,Sounds[SND_2],0);
                    break;
                case 1 :
                    if (GlobalSoundEnabled)
                        Mix_PlayChannel(-1,Sounds[SND_1],0);
                    break;
                case 0 :
                    if(ScoreTimer != 0)
                    {
                        Score += AddToScore;
                        if(GameType == Relative)
                            Timer += AddToScore / 200;
                    }
                    if(Timer == 0)
                    {
                        if (GlobalSoundEnabled)
                            Mix_PlayChannel(-1,Sounds[SND_TIMEOVER],0);
                        GameState = GSTimeOver;
                    }
                    break;
            }
#ifdef DEBUG
            Fps = FrameCount;
            FrameCount = 0;
#endif
            Time = SDL_GetTicks();
        }

    }
    delete Selector;
    SDL_FreeSurface(Tmp);
    Mix_HaltMusic();
}

void LoadHighScores()
{
    FILE *FHighScores;
 	FHighScores = fopen("./highscore.dat","rb");
 	int Teller = 0;
 	if(FHighScores)
 	{
        fread(HighScores[Fixed],sizeof(HighScores[Fixed][0]),10,FHighScores);
        fread(HighScores[Relative],sizeof(HighScores[Relative][0]),10,FHighScores);
		fclose(FHighScores);
 	}
 	else
 	{
 	    for (Teller = 0;Teller<10;Teller++)
 	    {
 	        sprintf(HighScores[Fixed][Teller].PName,"%s","joyrider");
 	        sprintf(HighScores[Relative][Teller].PName,"%s","joyrider");
 	        HighScores[Fixed][Teller].PScore = 0;
 	        HighScores[Relative][Teller].PScore = 0;
 	    }
 	}
}

void SaveHighScores()
{
    FILE *FHighScores;
 	FHighScores = fopen("./highscore.dat","wb");
 	if(FHighScores)
 	{
        fwrite(HighScores[Fixed],sizeof(HighScores[Fixed][0]),10,FHighScores);
        fwrite(HighScores[Relative],sizeof(HighScores[Relative][0]),10,FHighScores);
		fclose(FHighScores);
		sync();
 	}
}





void ShowHighScores()
{
    SDL_Rect Rect;
    SDL_Event Event;
    bool RPressed = false;
    SDL_Surface *Tmp;
    Tmp = SDL_DisplayFormat(Buffer);
    Uint8 alpha = 0;
    SDL_Color TextColor = {102,115,152,255};
    int Teller = 0;
    char Text[250];
    int ScoreType = GameType;
    while (GameState == GSShowHighScores)
    {
        if(MusicCount > 0)
            if(GlobalSoundEnabled)
                if(!Mix_PlayingMusic())
                {
                    SelectedMusic = 0;
                    Mix_PlayMusic(Music[SelectedMusic],0);
                    SetVolume(Volume);
                }
       /* if(GlobalSoundEnabled)
            if(!Mix_PlayingMusic())
                Mix_PlayMusic(Music[MUS_TITLE],0);*/
        while (SDL_PollEvent(&Event))
		{
		    if (Event.type == SDL_JOYBUTTONDOWN)
                switch (Event.jbutton.button)
                {
                    case GP2X_BUTTON_L:
                        PreviousSkin();
                        break;
                    case GP2X_BUTTON_R:
                        NextSkin();
                        break;
                    case GP2X_BUTTON_VOLUP:
						IncVolume();
						break;
					case GP2X_BUTTON_VOLDOWN:
						DecVolume();
						break;
                    case GP2X_BUTTON_DOWN:
                    case GP2X_BUTTON_UP:
                    case GP2X_BUTTON_RIGHT:
                    case GP2X_BUTTON_LEFT:
                    case GP2X_BUTTON_X:
                    case GP2X_BUTTON_Y:
                        if (ScoreType == Fixed)
                            ScoreType = Relative;
                        else
                            ScoreType = Fixed;
                        break;
                    case GP2X_BUTTON_B:
                    case GP2X_BUTTON_A:
                    case GP2X_BUTTON_START:
                    case GP2X_BUTTON_SELECT:
                        GameState = GSTitleScreen;
                        break;
                }
        }
        SDL_BlitSurface(IMGHighScores,NULL,Tmp,NULL);
        switch(ScoreType)
        {
            case Fixed :
                sprintf(Text,"Gametype: Fixed Timer" );
                break;
            case Relative :
                sprintf(Text,"Gametype: Relative Timer" );
                break;
        }

		WriteText(Tmp,font,Text,strlen(Text),85,227,0,TextColor);
        for(Teller = 0;Teller<10;Teller++)
        {
            sprintf(Text,"%2d.%s",Teller+1,HighScores[ScoreType][Teller].PName);
            WriteText(Tmp,MonoFont,Text,strlen(Text),50,62+Teller*16,0,TextColor);
            sprintf(Text,"%7d",HighScores[ScoreType][Teller].PScore);
            WriteText(Tmp,MonoFont,Text,strlen(Text),220,62+Teller*16,0,TextColor);
        }
        //tekenen naar buffer
        Rect.w = Buffer->w;
        Rect.h = Buffer->h;
        Rect.x = StartScreenX;
        Rect.y = StartScreenY;
        if (alpha < 255)
        {
            if(alpha+AlphaIncrease > MaxAlpha)
            {
                alpha = 255;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
            else
            {
                alpha+=10;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
        }
        SDL_BlitSurface(Tmp,NULL,Buffer,NULL);
        SDL_FillRect(Buffer1,NULL,SDL_MapRGB(Buffer1->format,0,0,0));

        SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);

        SDL_BlitSurface(Buffer1,NULL,Screen,NULL);
        SDL_Flip(Screen);


    }
    SDL_FreeSurface(Tmp);
}

void LoadSounds()
{
    if(GlobalSoundEnabled)
    {
		Sounds[SND_READYGO] = Mix_LoadWAV("./sound/readygo.wav");
		Sounds[SND_1] = Mix_LoadWAV("./sound/one.wav");
		Sounds[SND_2] = Mix_LoadWAV("./sound/two.wav");
		Sounds[SND_3] = Mix_LoadWAV("./sound/three.wav");
		Sounds[SND_1MINUTE] = Mix_LoadWAV("./sound/oneminute.wav");
		Sounds[SND_5MINUTE] = Mix_LoadWAV("./sound/fiveminute.wav");
		Sounds[SND_TIMEOVER] = Mix_LoadWAV("./sound/timeover.wav");
		Sounds[SND_MENU] = Mix_LoadWAV("./sound/menu.wav");
		Sounds[SND_SELECT] = Mix_LoadWAV("./sound/select.wav");
		Sounds[SND_WELCOME] = Mix_LoadWAV("./sound/welcome.wav");
		Sounds[SND_GOODBYE] = Mix_LoadWAV("./sound/goodbye.wav");
		Sounds[SND_DELETE] = Mix_LoadWAV("./sound/delete.wav");
		Sounds[SND_BLOCKSELECT] = Mix_LoadWAV("./sound/blockselect.wav");
		Sounds[SND_ERROR] = Mix_LoadWAV("./sound/error.wav");
    }
}

void UnloadSounds()
{
    if (GlobalSoundEnabled)
    {
        if (!Mix_PlayingMusic())
            Mix_HaltMusic();

        int Teller;
        for (Teller=0;Teller<NrOfSounds;Teller++)
            if(Sounds[Teller])
                Mix_FreeChunk(Sounds[Teller]);
    }

}

int main(int argc, char **argv)
{
    SDL_Rect Rect;
	//mtrace();
	SDL_Surface *Tmp;
	if (SDL_Init(SDL_INIT_JOYSTICK | SDL_INIT_VIDEO | SDL_INIT_AUDIO ) == 0)
	{
		printf("SDL Succesfully initialized\n");

        if(SDL_GP2X_GetPhysicalScreenSize(&Rect) == 0)
            Screen = SDL_SetVideoMode( WINDOW_WIDTH, WINDOW_HEIGHT,16, SDL_HWSURFACE);
        else
        {
            TvOutMode = true;
            Screen = SDL_SetVideoMode( 360, 288,16, SDL_HWSURFACE );
        }
		if(Screen)
		{
		    printf("Succesfully Set %dx%dx16\n",WINDOW_WIDTH,WINDOW_HEIGHT);
		    Tmp = SDL_CreateRGBSurface(SDL_HWSURFACE,WINDOW_WIDTH,WINDOW_HEIGHT,16,Screen->format->Rmask,Screen->format->Gmask,Screen->format->Bmask,Screen->format->Amask);
			Buffer = SDL_DisplayFormat(Tmp);
			SDL_FreeSurface(Tmp);
			if (TvOutMode)
                Tmp = SDL_CreateRGBSurface(SDL_HWSURFACE,360,288,16,Screen->format->Rmask,Screen->format->Gmask,Screen->format->Bmask,Screen->format->Amask);
            else
                Tmp = SDL_CreateRGBSurface(SDL_HWSURFACE,WINDOW_WIDTH,WINDOW_HEIGHT,16,Screen->format->Rmask,Screen->format->Gmask,Screen->format->Bmask,Screen->format->Amask);
			Buffer1 = SDL_DisplayFormat(Tmp);
			SDL_FreeSurface(Tmp);
			//Buffer = SDL_DisplayFormat(Buffer);
			if (Buffer && Buffer1)
			{
			    SDL_FillRect(Buffer,NULL,SDL_MapRGB(Buffer->format,0,0,0));
                printf("Succesfully Created Buffer\n");
                SDL_ShowCursor(SDL_DISABLE);
                Joystick = SDL_JoystickOpen(0);
                SDL_JoystickEventState(SDL_ENABLE);
                if(Joystick)
                {
                    if (Mix_OpenAudio(22050,AUDIO_S16,MIX_DEFAULT_CHANNELS,128) < 0)
                    {
                        GlobalSoundEnabled = false;
                        printf("Failed to initialise sound!\n");
                        printf("%s\n",Mix_GetError());
                    }
                    else
                    {
                        printf("Audio Succesfully initialised!\n");
                    }
                    if (TTF_Init() == 0)
                    {
                        printf("Succesfully initialized TTF\n");
                        font = TTF_OpenFont("./font.ttf",11);
                        BigFont = TTF_OpenFont("./font.ttf",16);
                        MonoFont = TTF_OpenFont("./font1.ttf",17);
                        if (font&&BigFont&&MonoFont)
                        {
                            sprintf(SkinName,"%s","");
                            srand(time(NULL));
                            printf("Succesfully Loaded fonts\n");
                            TTF_SetFontStyle(font,TTF_STYLE_NORMAL);
                            // Main game loop that loops untile the gamestate = GSQuit
                            // and calls the procedure according to the gamestate.
                            SearchForMusic();
                            SearchForSkins();
                            LoadSettings();
                            LoadGraphics();
                            LoadSounds();
                            LoadHighScores();
                            World = new CWorldParts();
                            while (GameState != GSQuit)
                            {
                                switch(GameState)
                                {
                                    case GSGame :
                                        Game();
                                        break;
                                    case GSTitleScreen:
                                        TitleScreen();
                                        break;
                                    case GSIntro :
                                        Intro();
                                        break;
                                    case GSReadyGo:
                                        ReadyGo();
                                        break;
                                    case GSTimeOver:
                                        TimeOver();
                                        break;
                                    case GSCredits:
                                        Credits();
                                    case GSGameTypeMenu:
                                        GameTypeMenu();
                                    case GSShowHighScores:
                                        ShowHighScores();
                                    default :
                                        break;
                                }
                            }
                            delete World;
                            SaveHighScores();
                            UnloadSounds();
                            UnloadMusic();
                            UnLoadGraphics();
                            SaveSettings();
                            Mix_CloseAudio();
                            TTF_CloseFont(font);
                            TTF_CloseFont(MonoFont);
                            TTF_CloseFont(BigFont);
                            font=NULL;
                        }
                        else
                        {
                            printf("Failed to Load fonts\n");
                        }
                        TTF_Quit();
                    }
                    else
                    {
                        printf("Failed to initialize TTF\n");
                    }
                    if(SDL_JoystickOpened(0))
    					SDL_JoystickClose(Joystick);
                }
                else
                {
                    printf("Failed to open joystick(0)!\n");
                }
                SDL_FreeSurface(Buffer);
                Buffer=NULL;
			}
			else
                printf("Failed To Create Buffer\n");
            SDL_FreeSurface(Screen);
            Screen=NULL;
		}
		else
		{
			printf("Failed to Set Videomode %dx%dx16\n",WINDOW_WIDTH, WINDOW_HEIGHT);
		}
		SDL_Quit();
	}
	else
	{
		printf("Couldn't initialise SDL!\n");
	}
	// Shouldn't be needed but i just placed it again to be sure all the boardparts are free
	// Call the menu
	//chdir("/usr/gp2x");
	//execl("/usr/gp2x/gp2xmenu", "/usr/gp2x/gp2xmenu", NULL);

	return 0;

}
