#ifdef GP2X
    #include <SDL/SDL_Gp2x.h>
    #define SCREEN_BPP 16
#endif

#ifdef WIN32
    #define SCREEN_BPP 32
#endif
#include <sys/stat.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fstream>
#include <dirent.h>
#include <unistd.h>
#include <SDL/SDL_keysym.h>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_mixer.h>
#include "gp2x.h"
#include <SDL/SDL_gfxPrimitives.h>
#include <SDL/SDL_rotoZoom.h>
#include "CInput.h"
#include "CUsbJoystickSetup.h"
#include "Functions.h"
#include "defines.h"
#include <SDL/SDL_framerate.h>

#include <vector>







const int AlphaInc =20,
          NrOfRows = 50,
		  NrOfCols = 50,
		  MaxHistory = 1000,
		  IDEmpty = 1,
		  IDPlayer = 2,
		  IDBox = 3,
          IDFloor = 4,
          IDBomb = 5,
          IDWall = 6,
          IDDiamond = 7,
          IDExplosion = 8,
          ZPlayer= 10,
          ZDiamond = 3,
          ZWall = 5,
          ZBox = 5,
          ZBomb = 3,
          ZFloor = 1,
          ZExplosion = 15,
		  FPS = 40,
		  MaxSkins = 100,
		  MaxLevelPacks=200,
		  MaxMusicFiles=26,
		  NrOfSounds=6,
		  SND_MENU=0,
		  SND_SELECT=1,
		  SND_ERROR=2,
		  SND_STAGEEND=3,
		  SND_EXPLODE=4,
		  SND_COLLECT=5,
		  MaxAlpha=150,
          InputDelay = 13;


using namespace std;

enum GameStates {GSTitleScreen,GSCredits,GSIntro,GSQuit,GSGame,GSStageClear,GSStageSelect,GSLevelEditor,GSLevelEditorMenu,GSTest,GSSetupUsbJoystickButtons};

bool GlobalSoundEnabled = true;
Mix_Chunk *Sounds[NrOfSounds];

SDL_Surface *IMGBackground=NULL,*IMGFloor=NULL,*IMGPlayer=NULL,*IMGBox=NULL,*IMGEmpty=NULL,*IMGTitleScreen=NULL,*IMGWall=NULL,
            *IMGIntro1=NULL,*IMGIntro2=NULL,*IMGIntro3=NULL,*IMGBomb=NULL,*IMGDiamond=NULL,*IMGExplosion=NULL,*IMGGrid=NULL;

int WINDOW_WIDTH=320,WINDOW_HEIGHT=240,SCREENPOSMULTIPLY=2,TileWidth = 16,TileHeight =16;

CUsbJoystickSetup *JoystickSetup;

class CWorldPart;

struct SPoint
{
	int X,Y;
};



class CHistory
{
 private:
 	SPoint Items[MaxHistory];
 	CWorldPart *Part;
	int ItemCount;
 public:
 	CHistory(CWorldPart *Partin);
 	void Add (int X, int Y);
 	void GoBack();
 	void Print();
};

class CViewPort
{
    public:
        int VPMinX,VPMinY,VPMaxX,VPMaxY,MinScreenX,MinScreenY,MaxScreenX,MaxScreenY,Width,Height,VPLimitMinX,VPLimitMaxX,VPLimitMinY,VPLimitMaxY;
        CViewPort(int MinX,int MinY, int MaxX,int MaxY,int MinX2,int MinY2,int MaxX2,int MaxY2)
        {
            if ((MinX  < NrOfCols) && (MinX  >=0) && (MaxX < NrOfCols) && (MaxX  >=0) &&
                (MinY  < NrOfRows) && (MinY  >=0) && (MaxY  < NrOfRows) && (MaxY  >=0))
            {
                VPMinX = MinX;
                VPMinY = MinY;
                VPMaxX = MaxX;
                VPMaxY = MaxY;
                MinScreenX = VPMinX * TileWidth;
                MinScreenY = VPMinY * TileHeight;
                MaxScreenX = VPMaxX * TileHeight;
                MaxScreenY = VPMaxY * TileWidth;
            }
            else
            {
                VPMinX = 0;
                VPMinY = 0;
                VPMaxX = (WINDOW_WIDTH/TileWidth)-1;
                VPMaxY = (WINDOW_HEIGHT/TileHeight)-1;
                MinScreenX =0;
                MinScreenY = 0;
                MaxScreenX = WINDOW_WIDTH;
                MaxScreenY = WINDOW_HEIGHT;
            }
            if ((MinX2  < NrOfCols) && (MinX2  >=0) && (MaxX2 < NrOfCols) && (MaxX2  >=0) &&
                (MinY2  < NrOfRows) && (MinY2  >=0) && (MaxY2  < NrOfRows) && (MaxY2  >=0))
                {
                    VPLimitMinX = MinX2;
                    VPLimitMinY = MinY2;
                    VPLimitMaxX = MaxX2;
                    VPLimitMaxY = MaxY2;
                }
                else
                {
                    VPLimitMinX = 0;
                    VPLimitMinY = 0;
                    VPLimitMaxX = NrOfCols-1;
                    VPLimitMaxY = NrOfRows-1;
                }

            Width = VPMaxX - VPMinX + 1;
            Height = VPMaxY - VPMinY + 1;
        }
        void SetVPLimit(int MinX,int MinY, int MaxX,int MaxY)
        {
            if ((MinX  < NrOfCols) && (MinX  >=0) && (MaxX < NrOfCols) && (MaxX  >=0) &&
                (MinY  < NrOfRows) && (MinY  >=0) && (MaxY  < NrOfRows) && (MaxY  >=0))
                {
                    //printf("Limit Success\n");
                    VPLimitMinX = MinX;
                    VPLimitMinY = MinY;
                    VPLimitMaxX = MaxX;
                    VPLimitMaxY = MaxY;
                }
                else
                {
                   // printf("Limit Failed\n");
                    VPLimitMinX = 0;
                    VPLimitMinY = 0;
                    VPLimitMaxX = NrOfCols-1;
                    VPLimitMaxY = NrOfRows-1;
                }
                if(VPLimitMaxX - VPLimitMinX < Width-1)
                {
                    if( VPLimitMaxX - Width-1 >= 0)
                        VPLimitMinX = VPLimitMaxX - (Width-1);
                    else
                    {
                        VPLimitMinX = 0;
                        VPLimitMaxX = VPLimitMinX + (Width -1);
                    }
                }
                if(VPLimitMaxY - VPLimitMinY < Height-1)
                {
                    if( VPLimitMaxY - Height-1 >= 0)
                        VPLimitMinY = VPLimitMaxY - (Height-1);
                    else
                    {
                        VPLimitMinY = 0;
                        VPLimitMaxY = VPLimitMinY +(Height -1);
                    }
                }

//                printf("vp limit : %d %d %d %d\n",VPLimitMinX,VPLimitMinY,VPLimitMaxX,VPLimitMaxY);
        }
        void Move(int Xi,int Yi)
        {
//            printf("Screen : %d %d %d %d Limit:MinX: %d MinY : %d MaxX: %d MaxY: %d \n",MinScreenX +Xi,MinScreenY+Yi,MaxScreenX+Xi,MaxScreenY+Yi,VPLimitMinX*TileWidth,VPLimitMinY*TileHeight,TileWidth*(VPLimitMaxX),TileHeight*(VPLimitMaxY));
            if ((MinScreenX + Xi <= TileWidth*(VPLimitMaxX)) && (MinScreenX + Xi >=VPLimitMinX*TileWidth) && (MaxScreenX + Xi <= TileWidth*(VPLimitMaxX)) && (MaxScreenX + Xi >=VPLimitMinX *TileWidth) &&
                (MinScreenY + Yi <= TileHeight*(VPLimitMaxY)) && (MinScreenY + Yi >=VPLimitMinY*TileHeight) && (MaxScreenY + Yi <= TileHeight*(VPLimitMaxY)) && (MaxScreenY + Yi >=VPLimitMinY*TileHeight))
                {

                    MinScreenX +=Xi;
                    MaxScreenX +=Xi;
                    MinScreenY +=Yi;
                    MaxScreenY +=Yi;
                    VPMinX = MinScreenX / TileWidth;
                    VPMinY = MinScreenY  / TileHeight;
                    VPMaxX = MaxScreenX  / TileHeight;
                    VPMaxY = MaxScreenY  / TileWidth;
                }
        }
        void SetViewPort(int MinX,int MinY, int MaxX,int MaxY)
        {
           if ((MinX  <= VPLimitMaxX) && (MinX  >=VPLimitMinX) && (MaxX <= VPLimitMaxX) && (MaxX  >=VPLimitMinX) &&
               (MinY  <= VPLimitMaxY) && (MinY  >=VPLimitMinY) && (MaxY <= VPLimitMaxY) && (MaxY  >=VPLimitMinY))
            {
//                printf("Correct\n");
                VPMinX = MinX;
                VPMinY = MinY;
                VPMaxX = MaxX;
                VPMaxY = MaxY;
                MinScreenX = VPMinX * TileWidth;
                MinScreenY = VPMinY * TileHeight;
                MaxScreenX = VPMaxX * TileHeight;
                MaxScreenY = VPMaxY * TileWidth;
            }
            else
            {
                VPMinX = MinX;
                VPMinY = MinY;
                VPMaxX = MaxX;
                VPMaxY = MaxY;
               /* if(VPMinX < 0)
                {
                    VPMinX = 0;
                    VPMaxX = (WINDOW_WIDTH/TileWidth)-1;
                }
                if(VPMaxX >= NrOfCols)
                {
                    VPMinX = NrOfCols - (WINDOW_WIDTH/TileWidth);
                    VPMaxX = NrOfCols-1;
                }
                if(VPMinY < 0)
                {
                    VPMinY = 0;
                    VPMaxY = (WINDOW_HEIGHT / TileHeight) - 1;
                }
                if(VPMaxY >=NrOfRows)
                {
                    VPMinY = NrOfRows - (WINDOW_HEIGHT/TileHeight);
                    VPMaxY = NrOfRows-1;
                }
                */
                if(VPMinX < VPLimitMinX)
                {
                    VPMinX = VPLimitMinX;
                    VPMaxX = VPLimitMinX+(WINDOW_WIDTH/TileWidth)-1;

                }
                if(VPMaxX > VPLimitMaxX)
                {
                    VPMinX = VPLimitMaxX-(WINDOW_WIDTH/TileWidth)+1;
                    VPMaxX = VPLimitMaxX;
                }
                if(VPMinY < VPLimitMinY)
                {
                    VPMaxY = VPLimitMinY + (WINDOW_HEIGHT/TileHeight)-1;
                    VPMinY = VPLimitMinY;
                }
                if(VPMaxY >VPLimitMaxY)
                {
                    VPMinY = VPLimitMaxY- (WINDOW_HEIGHT/TileHeight)+1;
                    VPMaxY = VPLimitMaxY;
                }
//                printf("Not Correct : %d %d %d %d %d %d %d %d\n",MinX,MinY,MaxX,MaxY,VPMinX,VPMinY,VPMaxX,VPMaxY);
                MinScreenX = VPMinX * TileWidth;
                MinScreenY = VPMinY * TileHeight;
                MaxScreenX = VPMaxX * TileHeight;
                MaxScreenY = VPMaxY * TileWidth;
            }
            Width = VPMaxX - VPMinX + 1;
            Height = VPMaxY - VPMinY + 1;
        }
        ~CViewPort()
        {
        }


};

class CWorldParts
{
 private:

 	void Sort();
 	bool DisableSorting;
 public:
    CViewPort *ViewPort;
 	CWorldPart *Items[NrOfRows*NrOfCols*2];
 	int ItemCount;
 	CWorldParts();
 	void Add(CWorldPart *WorldPart);
 	void CenterVPOnPlayer();
 	void LimitVPLevel();
 	void Move();
 	void HistoryAdd();
 	void HistoryGoBack();
 	void Draw(SDL_Surface *Surface);
 	void Remove(int PlayFieldXin,int PlayFieldYin);
 	void Remove(int PlayFieldXin,int PlayFieldYin,int Type);
 	void RemoveAll();
	void Save(char *Filename);
	void Load(char *Filename);
 	~CWorldParts();
};

class CWorldPart
{
 protected:
    int MoveDelayCounter;
    bool FirstArriveEventFired,PNeedToKill;
 	int Type,MoveSpeed,MoveDelay,Xi,Yi,X,Y,AnimPhase,PlayFieldX,PlayFieldY,Z,Group;
 	bool BHistory,BHide;
 	SDL_Surface ** Image;
 	vector<SPoint> MoveQue  ;



 public:
    void Hide() {BHide = true;}
    void AddToMoveQue(int PlayFieldXIn,int PlayFieldYIn)
    {
        SPoint Temp;
        Temp.X = PlayFieldXIn;
        Temp.Y = PlayFieldYIn;
        if (MoveQue.size() == 0)
            MoveQue.push_back(Temp);
        else
            MoveQue.insert(MoveQue.begin(),Temp);
    }
    void Kill()
    {
        PNeedToKill = true;
    }
    bool NeedToKill() {return PNeedToKill;}
    bool MovesInQue() { return (MoveQue.size() > 0);};
 	CWorldParts *ParentList;
 	bool IsMoving;
 	CHistory *History;
 	bool Selected;
 	CWorldPart(const int PlayFieldXin,const int PlayFieldYin,bool CreateHistory)
 	{
 		BHistory = CreateHistory;
 		if (BHistory)
 			History = new CHistory(this);
 		PlayFieldX=PlayFieldXin;
 		PlayFieldY=PlayFieldYin;
 		Xi=0;
 		Yi=0;
 		X=PlayFieldXin*TileWidth;
 		Y=PlayFieldYin*TileHeight;
 		Type=0;
 		MoveDelay=0;
 		MoveDelayCounter=0;
 		IsMoving = false;
 		MoveSpeed=0;
 		Image = NULL;
 		ParentList = 0;
 		AnimPhase=0;
 		Selected = false;
 		FirstArriveEventFired = false;
 		Z=0;
 		Group=0;
 		PNeedToKill = false;
 		BHide = false;
 	}
 	int GetGroup() {return Group;};
 	int GetType() {return Type;};
 	int GetX() {return X;}
 	int GetY() {return Y;}
 	int GetPlayFieldX() {return PlayFieldX;}
 	int GetPlayFieldY() {return PlayFieldY;}
 	int GetZ() {return Z;}
 	int GetAnimPhase() {return AnimPhase;}
 	bool HasHistory() { return BHistory;}
 	void HistoryAdd() { if (BHistory) History->Add(PlayFieldX,PlayFieldY); }
 	void HistoryGoBack() { if(BHistory) History->GoBack();}
 	void SetAnimPhase(int AnimPhaseIn) { AnimPhase = AnimPhaseIn;}
 	virtual void Event_ArrivedOnNewSpot(){}
 	virtual void Event_BeforeDraw() {}
 	virtual void Event_LeaveCurrentSpot() {}
 	virtual void Event_Moving(int ScreenPosX,int ScreenPosY,int ScreenXi, int ScreenYi) {}
 	void SetPosition(const int PlayFieldXin,const int PlayFieldYin)
 	{
 		if ((PlayFieldXin >= 0) && (PlayFieldXin < NrOfCols) && (PlayFieldYin >= 0) && (PlayFieldYin < NrOfRows))
 		{
 			PlayFieldX=PlayFieldXin;
 			PlayFieldY=PlayFieldYin;
 			X=PlayFieldXin*TileWidth;
 			Y=PlayFieldYin*TileHeight;
 			//Event_ArrivedOnNewSpot();
 		}
 	}
 	virtual void MoveTo(const int PlayFieldXin,const int PlayFieldYin,bool BackWards)
 	{
 		if(!IsMoving)
 		{
 			if((PlayFieldXin != PlayFieldX) || (PlayFieldYin != PlayFieldY))
 				if(this->CanMoveTo(PlayFieldXin,PlayFieldYin) || BackWards)
 				{
 					PlayFieldX = PlayFieldXin;
 					PlayFieldY = PlayFieldYin;
 					if(X < PlayFieldX*TileWidth)
 						Xi = MoveSpeed;
 					if(X > PlayFieldX*TileWidth)
 						Xi = -MoveSpeed;
 					if(Y > PlayFieldY*TileHeight)
 						Yi = -MoveSpeed;
 					if(Y < PlayFieldY*TileHeight)
 						Yi = MoveSpeed;
 					IsMoving = true;
 					Event_LeaveCurrentSpot();
 				}
 		}
 	}
 	virtual bool CanMoveTo(const int PlayFieldXin,const int PlayFieldYin) {return false;}
 	virtual void Move()
 	{
 		if (!FirstArriveEventFired)
 		{
			Event_ArrivedOnNewSpot();
			FirstArriveEventFired=true;
 		}
 		if (IsMoving)
		{
			if (MoveDelayCounter == MoveDelay)
			{
				X += Xi;
				Y += Yi;
				Event_Moving(X,Y,Xi,Yi);
				if ((X == PlayFieldX * TileWidth) && (Y == PlayFieldY * TileHeight))
				{
					IsMoving = false;
					Xi = 0;
					Yi = 0;
					Event_ArrivedOnNewSpot();
				}
				MoveDelayCounter = -1;
			}
			MoveDelayCounter++;
		}
		else
            if (MoveQue.size() > 0)
            {
                if (CanMoveTo(MoveQue.back().X,MoveQue.back().Y))
                {
                    MoveTo(MoveQue.back().X,MoveQue.back().Y,false);
                    MoveQue.pop_back();
                }
                else
                    MoveQue.clear();
            }
 	}
 	virtual void Draw(SDL_Surface* Surface)
 	{
 		if(!BHide)
 		{
 		//printf("Start draw type:%d\n",Type);
 		if (*Image)
 		{
 			Event_BeforeDraw();
 			SDL_Rect SrcRect,DstRect;
			SrcRect.x = AnimPhase * TileWidth;
			SrcRect.y = 0;
			SrcRect.w = TileWidth;
			SrcRect.h = TileHeight;
			if(ParentList)
            {
                DstRect.x = X- ParentList->ViewPort->MinScreenX;
                DstRect.y = Y- ParentList->ViewPort->MinScreenY;
            }
            else
            {
                DstRect.x = X;
                DstRect.y = Y;
            }
			DstRect.w = TileWidth;
			DstRect.h = TileHeight;
			SDL_BlitSurface((*Image),&SrcRect,Surface,&DstRect);
		}
		if (Selected)
		{
		    if (ParentList)
			{
                boxRGBA(Surface,X- ParentList->ViewPort->MinScreenX,Y- ParentList->ViewPort->MinScreenY,X- ParentList->ViewPort->MinScreenX+TileWidth-1,Y- ParentList->ViewPort->MinScreenY+TileHeight-1,0,0,200,15);
                rectangleRGBA(Surface,X- ParentList->ViewPort->MinScreenX,Y- ParentList->ViewPort->MinScreenY,X- ParentList->ViewPort->MinScreenX+TileWidth-1,Y- ParentList->ViewPort->MinScreenY+TileHeight-1,0,0,255,50);
			}
			else
			{
			    boxRGBA(Surface,X,Y,X+TileWidth-1,Y+TileHeight-1,0,0,200,15);
                rectangleRGBA(Surface,X,Y,X+TileWidth-1,Y+TileHeight-1,0,0,255,50);
			}
		}
 		}
		//printf("End draw type:%d\n",Type);
 	}
 	virtual ~CWorldPart()
 	{
 		if(BHistory)
			delete History;
 	}
};

class CEmpty : public CWorldPart
{
 public:
	CEmpty(const int PlayFieldXin,const int PlayFieldYin) : CWorldPart(PlayFieldXin,PlayFieldYin,false)
	{
		Image = &IMGEmpty;
		Type = IDEmpty;
	}
};

class CWall : public CWorldPart
{
 public:
	CWall(const int PlayFieldXin,const int PlayFieldYin) : CWorldPart(PlayFieldXin,PlayFieldYin,false)
	{
		Image = &IMGWall;
		Type = IDWall;
		Z = ZWall;
	}
};

class CFloor : public CWorldPart
{
 public:
	CFloor(const int PlayFieldXin,const int PlayFieldYin) : CWorldPart(PlayFieldXin,PlayFieldYin,false)
	{
		Image = &IMGFloor;
		Type = IDFloor;
		Z = ZFloor;
	}
};

class CBomb : public CWorldPart
{
 public:
	CBomb(const int PlayFieldXin,const int PlayFieldYin) : CWorldPart(PlayFieldXin,PlayFieldYin,false)
	{
		Image = &IMGBomb;
		Type = IDBomb;
		Z = ZBomb;
	}
};

class CDiamond : public CWorldPart
{
 public:
	CDiamond(const int PlayFieldXin,const int PlayFieldYin) : CWorldPart(PlayFieldXin,PlayFieldYin,false)
	{
		Image = &IMGDiamond;
		Type = IDDiamond;
		Z = ZDiamond;
	}
};

class CExplosion : public CWorldPart
{
 private:
  int AnimCounter,AnimDelay,AnimDelayCounter,AnimPhases;
  void Event_BeforeDraw();
 public:
	CExplosion(const int PlayFieldXin,const int PlayFieldYin) : CWorldPart(PlayFieldXin,PlayFieldYin,false)
	{
	    if(GlobalSoundEnabled)
            Mix_PlayChannel(-1,Sounds[SND_EXPLODE],0);
		Image = &IMGExplosion;
		Type = IDExplosion;
		Z = ZExplosion;
		AnimPhase=0;
		AnimPhases=8;
		AnimCounter = 1;
		AnimDelay = 2;
		MoveDelay = 0;
		MoveSpeed = 2;
		AnimDelayCounter =0;
	}
};

void CExplosion::Event_BeforeDraw()
{
		AnimDelayCounter++;
		if (AnimDelayCounter ==AnimDelay)
		{
			AnimDelayCounter = 0;
			AnimPhase++;
			if (AnimPhase == AnimPhases)
				Kill();
		}

}


class CPlayer : public CWorldPart
{
 private:
 	int AnimCounter,AnimBase,AnimDelay,AnimDelayCounter,AnimPhases;
 	void Event_BeforeDraw();
 public:
    bool IsDeath;
 	bool CanMoveTo(const int PlayFieldXin,const int PlayFieldYin);
 	void Event_Moving(int ScreenPosX,int ScreenPosY,int ScreenXi,int ScreenYi);
 	void MoveTo(const int PlayFieldXin,const int PlayFieldYin,bool BackWards);
 	void Event_ArrivedOnNewSpot();
 	CPlayer(const int PlayFieldXin,const int PlayFieldYin) : CWorldPart(PlayFieldXin,PlayFieldYin,true)
 	{
		Image=&IMGPlayer;
		AnimBase=4;
		AnimPhase=4;
		AnimPhases=2;
		AnimCounter = 0;
		AnimDelay = 8;
		MoveDelay = 0;
		MoveSpeed = 2*SCREENPOSMULTIPLY;
		AnimDelayCounter =0;
		Type = IDPlayer;
		Z = ZPlayer;
		IsDeath=false;
 	}
 	void MoveTo(const int PlayFieldXin,const int PlayFieldYin);
};


bool CPlayer::CanMoveTo(const int PlayFieldXin,const int PlayFieldYin)
{
	int Teller;
	bool Result = true;
	if ((PlayFieldXin >= 0) && (PlayFieldXin < NrOfCols) && (PlayFieldYin >= 0) && (PlayFieldYin < NrOfRows))
	{
		if (ParentList)
		{
			for (Teller=0;Teller<ParentList->ItemCount;Teller++)
				if((ParentList->Items[Teller]->GetPlayFieldX() == PlayFieldXin) && (ParentList->Items[Teller]->GetPlayFieldY() == PlayFieldYin))
				{
					if((ParentList->Items[Teller]->GetType() == IDWall))
					{
						Result = false;
						break;
					}
					if((ParentList->Items[Teller]->GetType() == IDBox))
					{
						if (PlayFieldX > PlayFieldXin)
						{
							Result = ParentList->Items[Teller]->CanMoveTo(PlayFieldXin-1,PlayFieldYin);
						}
						if (PlayFieldX < PlayFieldXin)
						{
							Result = ParentList->Items[Teller]->CanMoveTo(PlayFieldXin+1,PlayFieldYin);
						}
						if (PlayFieldY > PlayFieldYin)
						{
							Result = ParentList->Items[Teller]->CanMoveTo(PlayFieldXin,PlayFieldYin-1);
						}
						if (PlayFieldY < PlayFieldYin)
						{
							Result = ParentList->Items[Teller]->CanMoveTo(PlayFieldXin,PlayFieldYin+1);
						}
						break;
					}

				}
		}
	}
	else
		Result = false;
	return Result;
}

void CPlayer::Event_ArrivedOnNewSpot()
{
    int Teller=0;
    if (ParentList)
    {
        for (Teller=0;Teller<ParentList->ItemCount;Teller++)
            if((ParentList->Items[Teller]->GetPlayFieldX() == PlayFieldX) && (ParentList->Items[Teller]->GetPlayFieldY() == PlayFieldY))
            {
                if((ParentList->Items[Teller]->GetType() == IDBomb))
                {
                    ParentList->Add(new CExplosion(PlayFieldX,PlayFieldY));
                    ParentList->Items[Teller]->Kill();
                    IsDeath = true;
                    Hide();

                    break;
                }

                if((ParentList->Items[Teller]->GetType() == IDDiamond))
                {
                    ParentList->Items[Teller]->Kill();
                    if(GlobalSoundEnabled)
                        Mix_PlayChannel(-1,Sounds[SND_COLLECT],0);
                }

            }
    }
}

void CPlayer::Event_BeforeDraw()
{
	if (IsMoving)
	{
		AnimPhase = AnimBase + AnimCounter;
		AnimDelayCounter++;
		if (AnimDelayCounter ==AnimDelay)
		{
			AnimDelayCounter = 0;
			AnimCounter++;
			if (AnimCounter == AnimPhases)
				AnimCounter = 0;
		}
	}
/*	else
		AnimPhase = AnimBase;*/
}

void CPlayer::Event_Moving(int ScreenPosX,int ScreenPosY,int ScreenXi,int ScreenYi)
{
    if((ScreenPosX > (ParentList->ViewPort->MaxScreenX) - WINDOW_WIDTH/2) && (Xi > 0))
        ParentList->ViewPort->Move(Xi,Yi);
    if((ScreenPosX < (ParentList->ViewPort->MaxScreenX) - WINDOW_WIDTH/2) && (Xi < 0))
        ParentList->ViewPort->Move(Xi,Yi);
    if((ScreenPosY > (ParentList->ViewPort->MaxScreenY) - WINDOW_HEIGHT/2) && (Yi > 0))
        ParentList->ViewPort->Move(Xi,Yi);
    if((ScreenPosY < (ParentList->ViewPort->MaxScreenY) - WINDOW_HEIGHT/2) && (Yi < 0))
        ParentList->ViewPort->Move(Xi,Yi);

}


void CPlayer::MoveTo(const int PlayFieldXin,const int PlayFieldYin,bool BackWards)
{
 	int Teller;
 	if(!IsMoving)
	{
		if(this->CanMoveTo(PlayFieldXin,PlayFieldYin) || BackWards)
		{
			PlayFieldX = PlayFieldXin;
			PlayFieldY = PlayFieldYin;
			if(X < PlayFieldX*TileWidth)
			{
				Xi = MoveSpeed;
				if(BackWards)
				{
					AnimBase = 0;
				}
				else
					if (ParentList)
					{
						for(Teller=0;Teller<ParentList->ItemCount;Teller++)
						{
							if(((ParentList->Items[Teller]->GetType() == IDBox) || (ParentList->Items[Teller]->GetType() == IDWall)) && ((ParentList->Items[Teller]->GetPlayFieldX() == PlayFieldX) && (ParentList->Items[Teller]->GetPlayFieldY() == PlayFieldY)))
							{
								ParentList->Items[Teller]->MoveTo(PlayFieldX+1,PlayFieldY,false);
								AnimBase = 6;
								break;
							}
							else
								AnimBase = 4;
						}
					}
					else
						AnimBase = 4;

			}
			if(X > PlayFieldX*TileWidth)
			{
				Xi = -MoveSpeed;
				if(BackWards)
				{
					AnimBase = 4;
				}
				else
					if (ParentList)
					{
						for(Teller=0;Teller<ParentList->ItemCount;Teller++)
						{
							if(((ParentList->Items[Teller]->GetType() == IDBox) || (ParentList->Items[Teller]->GetType() == IDWall)) && ((PlayFieldX == ParentList->Items[Teller]->GetPlayFieldX() )  && (ParentList->Items[Teller]->GetPlayFieldY() == PlayFieldY)))
							{
								ParentList->Items[Teller]->MoveTo(PlayFieldX-1,PlayFieldY,false);
								AnimBase = 2;
								break;
							}
							else
								AnimBase = 0;
						}
					}
					else
						AnimBase = 0;
			}
			if(Y > PlayFieldY*TileHeight)
			{
				Yi = -MoveSpeed;
				if(BackWards)
				{
					AnimBase = 12;
				}
				else
					if (ParentList)
					{
						for(Teller=0;Teller<ParentList->ItemCount;Teller++)
						{
							if(((ParentList->Items[Teller]->GetType() == IDBox) || (ParentList->Items[Teller]->GetType() == IDWall)) && ((PlayFieldY == ParentList->Items[Teller]->GetPlayFieldY())  && (ParentList->Items[Teller]->GetPlayFieldX() == PlayFieldX)))
							{
								ParentList->Items[Teller]->MoveTo(PlayFieldX,PlayFieldY-1,false);
								AnimBase = 10;
								break;
							}
							else
								AnimBase = 8;
						}
					}
					else
						AnimBase = 8;
			}
			if(Y < PlayFieldY*TileHeight)
			{
				Yi = MoveSpeed;
				if(BackWards)
				{
					AnimBase = 8;
				}
				else
					if(ParentList)
					{
						for(Teller=0;Teller<ParentList->ItemCount;Teller++)
						{
							if(((ParentList->Items[Teller]->GetType() == IDBox) || (ParentList->Items[Teller]->GetType() == IDWall)) && ((ParentList->Items[Teller]->GetPlayFieldY() == PlayFieldY)  && (ParentList->Items[Teller]->GetPlayFieldX() == PlayFieldX )))
							{
								ParentList->Items[Teller]->MoveTo(PlayFieldX,PlayFieldY+1,false);
								AnimBase = 14;
								break;
							}
							else
								AnimBase = 12;
						}
					}
					else
						AnimBase = 12;
			}
			IsMoving = true;
		}
		else
		{
			if (PlayFieldXin > PlayFieldX)
			{
				AnimBase= 6;
			}
			if (PlayFieldXin < PlayFieldX)
			{
				AnimBase = 2;
			}
			if (PlayFieldYin > PlayFieldY)
			{
				AnimBase = 14;
			}
			if (PlayFieldYin < PlayFieldY)
			{
				AnimBase = 10;
			}
			AnimPhase = AnimBase + AnimCounter;
			AnimDelayCounter++;
			if (AnimDelayCounter ==AnimDelay)
			{
				AnimDelayCounter = 0;
				AnimCounter++;
				if (AnimCounter == AnimPhases)
					AnimCounter = 0;
			}
		}

 	}
 }

class CBox : public CWorldPart
{
 private:
 	void Event_ArrivedOnNewSpot();
 	bool CanMoveTo(const int PlayFieldXin,const int PlayFieldYin);
 public:
	CBox(const int PlayFieldXin,const int PlayFieldYin) : CWorldPart(PlayFieldXin,PlayFieldYin,true)
	{
		Image = &IMGBox;
		Type = IDBox;
		MoveDelay = 0;
		MoveSpeed = 2*SCREENPOSMULTIPLY;
		Z = ZBox;
	}
};


bool CBox::CanMoveTo(const int PlayFieldXin,const int PlayFieldYin)
{
	int Teller;
	bool Result = true;
	if ((PlayFieldXin >= 0) && (PlayFieldXin < NrOfCols) && (PlayFieldYin >= 0) && (PlayFieldYin < NrOfRows))
	{
		if (ParentList)
		{
			for (Teller=0;Teller<ParentList->ItemCount;Teller++)
				if((ParentList->Items[Teller]->GetType() == IDWall) || (ParentList->Items[Teller]->GetType() == IDBox) || (ParentList->Items[Teller]->GetType() == IDDiamond))
					if((ParentList->Items[Teller]->GetPlayFieldX() == PlayFieldXin) && (ParentList->Items[Teller]->GetPlayFieldY() == PlayFieldYin))
					{
						Result = false;
						break;
					}
		}
	}
	else
		Result = false;
	return Result;
}

void CBox::Event_ArrivedOnNewSpot()
{
	int Teller;
	AnimPhase = 0;
	//printf("Arrive Event fired\n");
	if (ParentList)
	{
		//printf("Parent List Set\n");
		for (Teller=0;Teller< ParentList->ItemCount;Teller++)
		{
           if((ParentList->Items[Teller]->GetPlayFieldX() == PlayFieldX) && (ParentList->Items[Teller]->GetPlayFieldY() == PlayFieldY) &&
              (ParentList->Items[Teller]->GetType() == IDBomb))
              {
                  ParentList->Add(new CExplosion(PlayFieldX,PlayFieldY));
                  ParentList->Items[Teller]->Kill();
                  Kill();
              }

		}
	}
}



class CSelector
{
 private:
    CWorldParts *PList;
 	CWorldPart *Part;
 	int Selection;
 public:
	CSelector(CWorldParts *List);
	void MoveUp();
	void MoveDown();
	void MoveLeft();
	void MoveRight();
	void IncSelection();
	void DecSelection();
	void SetPosition(const int PlayFieldXin,const int PlayFieldYin);
	void Draw(SDL_Surface *Surface);
	int GetSelection();
	int GetPlayFieldX();
	int GetPlayFieldY();
	~CSelector();
};

CSelector::CSelector(CWorldParts *List)
{
    PList = List;
	Selection = IDEmpty;
	Part = new CEmpty(0,0);
	Part->ParentList = PList;
	Part->Selected = true;
}

void CSelector::SetPosition(const int PlayFieldXin,const int PlayFieldYin)
{
    Part->SetPosition(PlayFieldXin,PlayFieldYin);
}

void CSelector::MoveUp()
{

	Part->SetPosition(Part->GetPlayFieldX(),Part->GetPlayFieldY()-1);
}

void CSelector::MoveDown()
{
	Part->SetPosition(Part->GetPlayFieldX(),Part->GetPlayFieldY()+1);
}

void CSelector::MoveRight()
{
	Part->SetPosition(Part->GetPlayFieldX() +1,Part->GetPlayFieldY());
}

void CSelector::MoveLeft()
{
	Part->SetPosition(Part->GetPlayFieldX() -1,Part->GetPlayFieldY());
}

void CSelector::IncSelection()
{
	int X,Y;
	Selection++;
	if (Selection > 7)
		Selection = 1;
	X = Part->GetPlayFieldX();
	Y = Part->GetPlayFieldY();
	delete Part;
	switch (Selection)
	{
		case IDEmpty:
			Part = new CEmpty(X,Y);
			break;
		case IDBox:
			Part = new CBox(X,Y);
			break;
		case IDPlayer:
			Part = new CPlayer(X,Y);
			break;
		case IDBomb:
			Part = new CBomb(X,Y);
			break;
		case IDWall:
			Part = new CWall(X,Y);
			break;
		case IDFloor:
			Part = new CFloor(X,Y);
			break;
        case IDDiamond:
			Part = new CDiamond(X,Y);
			break;
	}
	Part->ParentList = PList;
	Part->Selected = true;
}

void CSelector::DecSelection()
{
	int X,Y;
	Selection--;
	if (Selection < 1)
		Selection = 7;
	X = Part->GetPlayFieldX();
	Y = Part->GetPlayFieldY();
	delete Part;
    switch (Selection)
	{
		case IDEmpty:
			Part = new CEmpty(X,Y);
			break;
		case IDBox:
			Part = new CBox(X,Y);
			break;
		case IDPlayer:
			Part = new CPlayer(X,Y);
			break;
		case IDBomb:
			Part = new CBomb(X,Y);
			break;
		case IDWall:
			Part = new CWall(X,Y);
			break;
		case IDFloor:
			Part = new CFloor(X,Y);
			break;
         case IDDiamond:
			Part = new CDiamond(X,Y);
			break;
	}

	Part->ParentList = PList;
	Part->Selected = true;
}

void CSelector::Draw(SDL_Surface *Surface)
{
	Part->Draw(Surface);
}

int CSelector::GetSelection()
{
	return Selection;
}

int CSelector::GetPlayFieldX()
{
	return Part->GetPlayFieldX();
}

int CSelector::GetPlayFieldY()
{
	return Part->GetPlayFieldY();
}

CSelector::~CSelector()
{
	delete Part;
}


CHistory::CHistory(CWorldPart *Partin)
{
	ItemCount = 0;
	Part = Partin;
}
void CHistory::Add(int X, int Y)
{
	SPoint Temp;
	int Teller;
	Temp.X = X;
	Temp.Y = Y;
	if (ItemCount < MaxHistory)
	{
		Items[ItemCount] = Temp;
		ItemCount++;
	}
	else
	{
		for (Teller=0;Teller < ItemCount - 1 ;Teller++)
		{
			Items[Teller] = Items[Teller+1];
		}
		Items[ItemCount-1] = Temp;
	}
}

void CHistory::GoBack()
{
	if(ItemCount > 0)
	{
		ItemCount--;
		Part->MoveTo(Items[ItemCount].X,Items[ItemCount].Y,true);
	}
}


void CHistory::Print()
{
	int Teller;
	for (Teller=0;Teller<ItemCount;Teller++)
	{
		printf("History nr:%d X:%d Y:%d\n",Teller,Items[Teller].X,Items[Teller].Y);
	}
}

CWorldParts::CWorldParts()
{
	ItemCount = 0;
	DisableSorting = false;
	ViewPort = new CViewPort(0,0,19,14,0,0,NrOfCols-1,NrOfRows-1);
}

void CWorldParts::CenterVPOnPlayer()
{
    int Teller=0,PlayerX=-1,PlayerY=-1;
    for (Teller=0;Teller<ItemCount;Teller++)
        if(Items[Teller]->GetType() == IDPlayer)
        {

            PlayerX = Items[Teller]->GetPlayFieldX();
            PlayerY = Items[Teller]->GetPlayFieldY();
//            printf("Player Found at %d %d \n",PlayerX,PlayerY);
            break;
        }
    ViewPort->SetViewPort(PlayerX-9, PlayerY-7,PlayerX+10,PlayerY+7);
}

void CWorldParts::LimitVPLevel()
{
    int MinX = NrOfCols, MinY = NrOfRows, MaxX = -1, MaxY = -1, Teller = 0;
	for (Teller = 0;Teller<ItemCount;Teller++)
	{
		if (Items[Teller]->GetPlayFieldX() < MinX)
			MinX = Items[Teller]->GetPlayFieldX();
		if (Items[Teller]->GetPlayFieldY() < MinY)
			MinY = Items[Teller]->GetPlayFieldY();
		if (Items[Teller]->GetPlayFieldX() > MaxX)
			MaxX = Items[Teller]->GetPlayFieldX();
		if (Items[Teller]->GetPlayFieldY() > MaxY)
            MaxY = Items[Teller]->GetPlayFieldY();
    }
//    printf("limitlevel : %d %d %d %d\n",MinX,MinY,MaxX,MaxY);
    ViewPort->SetVPLimit(MinX,MinY,MaxX,MaxY);
    CenterVPOnPlayer();


}

void CWorldParts::RemoveAll()
{
	int Teller;
	for (Teller=0;Teller<ItemCount;Teller++)
	{
		delete Items[Teller];
		Items[Teller] = NULL;
	}
	ItemCount=0;
}
void CWorldParts::Remove(int PlayFieldXin,int PlayFieldYin)
{
	int Teller1,Teller2;
	for (Teller1=0;Teller1<ItemCount;Teller1++)
	{
		if ((Items[Teller1]->GetPlayFieldX() == PlayFieldXin) && (Items[Teller1]->GetPlayFieldY() == PlayFieldYin))
		{
			delete Items[Teller1];
			for (Teller2=Teller1;Teller2<ItemCount-1;Teller2++)
				Items[Teller2] = Items[Teller2+1];
			ItemCount--;
			Teller1--;
		}
	}
}

void CWorldParts::Remove(int PlayFieldXin,int PlayFieldYin,int Type)
{
	int Teller1,Teller2;
	for (Teller1=0;Teller1<ItemCount;Teller1++)
	{
		if ((Items[Teller1]->GetPlayFieldX() == PlayFieldXin) && (Items[Teller1]->GetPlayFieldY() == PlayFieldYin) && (Items[Teller1]->GetType() == Type))
		{
			delete Items[Teller1];
			for (Teller2=Teller1;Teller2<ItemCount-1;Teller2++)
				Items[Teller2] = Items[Teller2+1];
			ItemCount--;
			Teller1--;
		}
	}
}

void CWorldParts::HistoryAdd()
{
	int Teller;
	for (Teller=0;Teller<ItemCount;Teller++)
		Items[Teller]->HistoryAdd();
}

void CWorldParts::HistoryGoBack()
{
	int Teller;
	for (Teller=0;Teller<ItemCount;Teller++)
		Items[Teller]->HistoryGoBack();
}

void CWorldParts::Add(CWorldPart *WorldPart)
{
	if( ItemCount < NrOfRows*NrOfCols*3 )
	{
		WorldPart->ParentList = this;
		Items[ItemCount] = WorldPart;
		ItemCount++;
		Sort();
	}
}


void CWorldParts::Sort()
{
	int Teller1,Teller2,Index;
	CWorldPart *Part;
	if (!DisableSorting)
	{
		for (Teller1 = 1; Teller1 <ItemCount;Teller1++)
		{
			Index = Items[Teller1]->GetZ();
			Part = Items[Teller1];
			Teller2 = Teller1;
			while ((Teller2 > 0) && (Items[Teller2-1]->GetZ() > Index))
			{
				Items[Teller2] = Items[Teller2 - 1];
				Teller2--;
			}
			Items[Teller2] = Part;
		}
	}

}

void CWorldParts::Save(char *Filename)
{
	FILE *Fp;
	int Teller,BufferPosition=0;
	char *Buffer;
	Buffer = new char[3*ItemCount];
	for (Teller=0;Teller<ItemCount;Teller++)
	{
		Buffer[BufferPosition]= (char) Items[Teller]->GetType();
		Buffer[BufferPosition+1] = (char)Items[Teller]->GetPlayFieldX();
		Buffer[BufferPosition+2]= (char)Items[Teller]->GetPlayFieldY();
		BufferPosition += 3;
	}
	Fp = fopen(Filename,"wb");
	if (Fp)
	{
		fwrite(Buffer,1,3*ItemCount,Fp);
		fclose(Fp);
#ifdef GP2X
		sync();
#endif
	}
	delete[] Buffer;
}

void CWorldParts::Load(char *Filename)
{
	int X,Y,Type;
	FILE *Fp;
	Uint32 StartTime=0;
	int BufferPosition=0;
	long FileSize;
	char *Buffer;
	CWorldPart *Part;
	Fp = fopen(Filename,"rb");
	if(Fp)
	{
		RemoveAll();
		DisableSorting=true;
		StartTime = SDL_GetTicks();
		fseek (Fp , 0 , SEEK_END);
  		FileSize = ftell (Fp);
  		rewind (Fp);
		Buffer = new char[FileSize];
		fread(Buffer,1,FileSize,Fp);
		while(BufferPosition < FileSize)
		{
			Type = (int)Buffer[BufferPosition];
			X =(int)Buffer[BufferPosition+1];
			Y = (int)Buffer[BufferPosition+2];
			BufferPosition +=3;
			switch(Type)
			{
                case IDEmpty:
                    Add( new CEmpty(X,Y));
                    break;
                case IDBox:
                     Add(new CBox(X,Y));
                    break;
                case IDPlayer:
                     Add(new CPlayer(X,Y));
                    break;
                case IDFloor:
                     Add(new CFloor(X,Y));
                    break;
                case IDWall:
                     Add( new CWall(X,Y));
                    break;
                case IDBomb:
                     Add(new CBomb(X,Y));
                    break;
                case IDDiamond:
                     Add( new CDiamond(X,Y));
                    break;

			}
		}
		delete[] Buffer;
		fclose(Fp);
		DisableSorting=false;
		Sort();
		LimitVPLevel();
		CenterVPOnPlayer();
		//printf("Loading:%d\n",SDL_GetTicks() - StartTime);
	}

}

void CWorldParts::Move()
{
	int Teller;
	for (Teller=0;Teller<ItemCount;Teller++)
	{
		Items[Teller]->Move();
	}
}

void CWorldParts::Draw(SDL_Surface *Surface)
{
	int Teller;
	for (Teller=0;Teller<ItemCount;Teller++)
	{
	    if(Items[Teller]->NeedToKill())
	     {
	         Remove(Items[Teller]->GetPlayFieldX(),Items[Teller]->GetPlayFieldY(),Items[Teller]->GetType());
	         //go back one item to prevent skips & segaults
	         Teller--;
	     }
	     else
            if((Items[Teller]->GetPlayFieldX() >= ViewPort->VPMinX) && (Items[Teller]->GetPlayFieldX()-1 <= ViewPort->VPMaxX) &&
                (Items[Teller]->GetPlayFieldY() >= ViewPort->VPMinY) && (Items[Teller]->GetPlayFieldY()-1 <= ViewPort->VPMaxY))
                {
                    Items[Teller]->Draw(Surface);
                }


	}
}

CWorldParts::~CWorldParts()
{
    delete ViewPort;
	int Teller;
	for (Teller=0;Teller<ItemCount;Teller++)
	{
		delete Items[Teller];
		Items[Teller] = NULL;
	}
	//printf("Deleted worldparts\n");
}

//----------------------------------------------------------------------------------------------------------------------------------------

unsigned char HashTable[1004] = {0xE4,0xF6,0xD7,0xA4,0xA5,0xC3,0xA3,0xE5,0xA4,0xF8,0xA3,0xF6,0xB5,0x98,0xA3,0xA7,0xB9,0xA0,0xD9,0xD6,0xE6,0x29,0x49,0xF8,0xB9,0xF0,0x79,0xA6,0x88,0xA8,0xC6,0xC2,0xF2,0xB7,0x79,0xC0,0xF0,0xD0,0xA8,0xE1,0xE1,0xF9,0xA5,0xE5,0xF1,0xE4,0xC7,0xE8,0xD9,0xB7,0xC0,0xE1,0xC0,0xB4,0xA9,0xF8,0xF8,0xF8,0xA1,0xA4,0xD0,0xC6,0x06,0xB9,0xA1,0xB0,0xF0,0xE8,0xB0,0xE5,0xC3,0xC8,0xA1,0xB2,0xF1,0xE2,0xE0,0xF0,0xF8,0xA5,0xE6,0xF2,0xB4,0xF7,0x79,0xB5,0x75,0xB0,0xF1,0xA6,0xC9,0xE2,0xB6,0x68,0xD9,0xA0,0xE7,0xD1,0xB4,0x51,0xD9,0xE9,0x96,0x61,0xD2,0xC2,0xC2,0x06,0x47,0x86,0x88,0xF4,0xC7,0x88,0xA0,0xA9,0xB1,0xD2,0xE9,0xF1,0x93,0xF4,0x35,0xC6,0xB2,0xC8,0xB8,0xB0,0xA8,0xE9,0xC1,0xA4,0xF5,0xD6,0xD1,0xC8,0xD3,0xA9,0xE9,0xA9,0xF4,0xB4,0xA3,0x57,0xE1,0xC5,0xF1,0xB7,0xB6,0xB2,0xE3,0x45,0x64,0xE3,0xD9,0xE5,0xB5,0xE2,0xE8,0xC8,0xB5,0xE7,0xD3,0xB0,0xD2,0xF7,0x81,0xB2,0xE4,0xA8,0x00,0xC9,0xD5,0xD4,0x86,0xC9,0xB1,0xA2,0xA0,0xD3,0xE3,0xD5,0xF7,0xE0,0xF9,0xE5,0xE2,0xD1,0xA5,0xD4,0xF4,0xC6,0xB4,0xD3,0xA2,0xD6,0xD3,0x35,0xE4,0xE3,0xC4,0xB6,0x18,0xD7,0xE7,0xC5,0xE2,0x59,0xA0,0xB5,0xA7,0xC5,0xF6,0xA7,0xC2,0xD7,0xA2,0xB5,0xB9,0xF6,0x81,0xA3,0xA5,0x00,0xA2,0xA8,0xE7,0xD5,0x87,0xC8,0xE9,0xD7,0xE6,0xA0,0x54,0xF9,0xF9,0xF4,0xB4,0xF4,0xE9,0xA7,0xD0,0x74,0xD0,0xE7,0x24,0xD9,0xB0,0xE8,0xE3,0xF1,0xF7,0xF7,0x88,0x97,0xC0,0xE0,0xD2,0xF3,0xC2,0xA1,0xC4,0xE1,0xF8,0xE8,0xB1,0x76,0x97,0xE7,0xB6,0x96,0xA7,0xD8,0xA4,0xF1,0xA0,0xF3,0xE8,0xE7,0x76,0xE3,0xB1,0xC9,0xE4,0xB4,0xF0,0xA4,0xD0,0x28,0xB8,0xF7,0xD9,0xD3,0xE5,0xC9,0x81,0xF7,0xC0,0xD9,0xC9,0xE3,0xA8,0xD6,0xB9,0xA7,0xF9,0xD0,0xD8,0x54,0xF6,0xF9,0xB8,0xC8,0xC9,0xB9,0xF0,0xF4,0xD1,0xA4,0xD0,0xE6,0xA8,0xB6,0xD2,0xB0,0xE9,0xE9,0xE9,0xB7,0xD9,0xD4,0xE6,0xF0,0xA4,0xC7,0xF1,0xC0,0xC2,0xA8,0xA7,0x14,0x25,0xA1,0xB6,0xB9,0xD5,0x78,0xC1,0xD1,0xC6,0x93,0xE8,0xE7,0xE7,0xA9,0xA2,0xA8,0x89,0xA1,0xA6,0xF9,0xA8,0xF7,0xE4,0xD8,0xA2,0xD2,0xB0,0x39,0xD1,0xC5,0x72,0xB0,0xD6,0xC6,0xA0,0xE9,0xE5,0xC1,0xC2,0xF9,0xF3,0xB2,0xC8,0xD0,0xD2,0xD1,0xC1,0xD7,0x53,0x93,0xB4,0xF9,0xB0,0xD0,0xD4,0xF5,0xE4,0xE4,0x13,0xB2,0xE9,0x70,0xB3,0xB2,0xD8,0xF0,0xC0,0xC1,0xB0,0xE8,0xA3,0xE7,0x33,0xB1,0xC0,0xA7,0x17,0xD4,0xA9,0xE5,0xF6,0xF3,0xF3,0xA3,0xE0,0xD5,0xE2,0x71,0xB7,0xB4,0xA9,0xD6,0xC1,0xC5,0xA3,0xB2,0xD7,0xB6,0xE8,0xC5,0xF0,0x10,0xA6,0xB0,0xC3,0xA3,0xA0,0xB2,0xF6,0xB9,0xF4,0xD8,0xD5,0xF8,0xB9,0xE3,0x70,0xD1,0xF1,0xA6,0xE6,0xE5,0xA8,0xD5,0xC9,0xC1,0xA5,0xE5,0x61,0xF4,0xB2,0xA9,0xA4,0xF2,0xE8,0xB7,0xD7,0xA1,0xD0,0xF4,0xF9,0xE4,0xE8,0xC6,0xB3,0xC4,0xB2,0xC9,0xE3,0x49,0xC3,0x62,0xA6,0xF3,0xB0,0x57,0x65,0xC4,0xF6,0xB9,0x23,0xB8,0xB3,0xA9,0xE6,0xB7,0xA5,0xE8,0xF6,0xC9,0xE7,0x63,0xF7,0xD9,0xA5,0xA5,0xD2,0xD4,0xA6,0xF2,0x94,0x04,0xB6,0x85,0xA3,0xD3,0x59,0xA3,0xA8,0x69,0xF1,0xB9,0xD1,0xD1,0xE5,0xE5,0x57,0xB0,0xD5,0x59,0xD4,0xD9,0x52,0xB4,0xA8,0x75,0xF0,0xE0,0x20,0xA1,0xE7,0x69,0xA2,0xC0,0xB1,0xA3,0xE5,0xB9,0xF5,0xB9,0xB5,0xE1,0xF3,0xB7,0x56,0xA1,0xA4,0xA9,0xA4,0xE7,0xC5,0xC7,0xC7,0x32,0xB7,0xA6,0xB4,0xB2,0xF6,0xB1,0xC4,0xB5,0xC4,0xC1,0xE1,0x97,0xB1,0xE3,0x37,0xA6,0xD8,0xC3,0xF1,0xD5,0xE9,0xF2,0xE2,0xF6,0xF3,0xF6,0xF4,0x51,0xE7,0xE9,0xC1,0xE8,0xA8,0x69,0xD3,0xB4,0xD8,0xD6,0xE8,0xD8,0x47,0xE1,0xA8,0xA0,0x94,0xB5,0xC2,0xD3,0xF6,0x28,0xF5,0x21,0xD7,0x48,0xC4,0xD8,0xF2,0xC7,0xC0,0xA7,0xF6,0xC7,0xE7,0x64,0xB5,0xD5,0x97,0xA0,0xD6,0xF7,0xC4,0xF7,0xD0,0xD7,0xA8,0xD7,0x28,0xC2,0xE5,0xD0,0xC2,0xF2,0xC8,0xF9,0xE5,0x08,0xD3,0xA0,0xA0,0xB2,0xD3,0x29,0xF0,0xB1,0xD0,0xB2,0xD7,0xF1,0x47,0xF2,0xD4,0xD3,0x53,0x92,0xC4,0xF9,0xA2,0xE8,0xC3,0xA9,0xF8,0xB1,0xE4,0x90,0xD0,0xE4,0x26,0x18,0xB4,0xF8,0xE7,0xC7,0xE0,0xF9,0xC1,0x36,0xE3,0xC2,0xE6,0xC9,0xC2,0xF8,0xA1,0x26,0xE7,0xC1,0x57,0xE9,0xD1,0xC8,0xC3,0xB2,0xE1,0x86,0xA5,0xE8,0xB4,0xA3,0xB5,0xD7,0x89,0xD5,0xF7,0xD0,0xD6,0xD7,0xB4,0x41,0xE1,0xA2,0xD5,0xF1,0xA8,0xF5,0xF7,0xC8,0xA3,0xB1,0xC0,0xC7,0x83,0x65,0xE6,0x46,0xA8,0xA3,0xE8,0xC5,0xF9,0xC2,0xD6,0xA5,0xD5,0xA5,0xA8,0xB6,0xE0,0xD0,0xC4,0xE1,0xF0,0x08,0x06,0x45,0xA7,0xF1,0xA1,0xE8,0xF3,0xF1,0xE2,0xC4,0xA6,0xF0,0xF3,0xD3,0xB2,0xE5,0xC5,0x85,0xA6,0xC7,0xD5,0xD4,0xD7,0xC2,0xC8,0xC8,0xD2,0xB8,0xA4,0x79,0x06,0xF7,0xE0,0xB6,0xF4,0xF4,0xC2,0xD1,0xB4,0xA4,0xB5,0xA8,0xC0,0xB2,0xB9,0xF7,0xC0,0xD6,0xD6,0xC3,0xF5,0xA3,0xF5,0xA7,0xE5,0xE1,0xB0,0xA5,0xE0,0xC3,0x41,0xD3,0xB2,0xF9,0xF2,0x19,0xB3,0xA7,0x58,0x40,0xB1,0x86,0xD5,0x71,0xF8,0xF6,0xD2,0xF2,0xA5,0xF4,0xB7,0x80,0xE5,0x03,0xF8,0x14,0xF5,0xE4,0xE7,0xE2,0xB9,0xA6,0xB5,0xE2,0xB5,0xD8,0xC7,0xE5,0xC5,0xF9,0xD4,0xD3,0xE5,0xD9,0xE8,0x79,0xD2,0xE3,0xF0,0xA1,0xF8,0xD8,0x31,0x78,0xB9,0xA9,0x41,0xC1,0xB1,0xC6,0x52,0xC5,0xA7,0xD0,0xF9,0xF5,0xA8,0xF6,0xA3,0xA3,0xD0,0xA5,0xF3,0xB1,0xB1,0x31,0xE0,0xC5,0xA7,0x29,0xC0,0xE1,0xD4,0xF7,0x15,0xC2,0xF3,0xA1,0xC9,0xF3,0xE0,0xA5,0xB6,0xE6,0xE5,0xB1,0xA3,0xB3,0xE7,0xF5,0xD8,0x93,0xF8,0xB2,0x94,0xC8,0xD2,0xE2,0xB0,0xF9,0xF1,0xB4,0xE0,0xD5,0x65,0x81,0xC4,0xC7,0xD0,0xE8,0xA7,0xA2,0xF7,0x45,0xA0,0xE9,0xC5,0xB6,0xE2,0xF2,0xE6,0xA7,0xE8,0x28,0xA6,0xE5,0x90,0xF0,0xD6,0xE1,0xC4,0xD2,0xD5,0xF0,0xA5,0xB1,0xC6,0xA0,0xC1,0xF5,0xB1,0xA4,0xD3,0xF3,0xA3,0xF3,0x10,0xE8,0xA9,0xC1,0xB2,0xB8,0xA5,0xD4,0x41,0xB4,0xC6,0xF7,0xC6,0xF9,0xD9,0xD0,0xC2,0xE9,0xD5,0xC7};
SDL_Surface* Screen,*Buffer,*Buffer1;
TTF_Font* font,* BigFont,*MonoFont,*BigFontSkin;
SDL_Joystick *Joystick;
GameStates GameState = GSIntro;
int MusicCount=0,SelectedMusic=0,InstalledLevelPacksCount=0,InstalledLevels=0,SelectedLevel=0,SelectedLevelPack=0,UnlockedLevels=1;
bool TvOutMode=false,LevelEditorMode=false,LevelHasChanged=false,StageReload=false;
SDL_Color MenuBoxColor = {79,80,67,255}, MenuBoxBorderColor = {204,224,207,255}, MenuTextColor = {79,80,7,0};
CWorldParts WorldParts;
Uint32 NextTime=0;
int Volume = 128,StartScreenX=20,StartScreenY=24,InitialStartScreenX = 20,InitialStartScreenY = 24,InstalledSkinsCount=0,SelectedSkin=-1;;
char LevelPackName[21]= "";
char LevelPackFileName[21] = "";
char InstalledLevelPacks[MaxLevelPacks][21];
Mix_Music *Music[MaxMusicFiles];
char InstalledSkins[MaxSkins][FILENAME_MAX];
char SkinName[FILENAME_MAX];
FPSmanager Fpsman;

void SetVolume(const int VolumeIn)
{
	if (GlobalSoundEnabled)
	{
		Mix_Volume(0,VolumeIn);
		Mix_Volume(1,VolumeIn);
		Mix_VolumeMusic(VolumeIn);
	}
}

void IncVolume()
{
	if (Volume < 128)
	{
		Volume += 8;
		SetVolume(Volume);
	}
}

void DecVolume()
{
	if (Volume > 0)
	{
		Volume-= 8;
		SetVolume(Volume);
	}
}


void LoadSettings()
{
    int Teller;
	FILE *Fp;
	Fp = fopen("./settings.dat","rt");
	if (Fp)
	{
		fscanf(Fp,"Volume=%d\n",&Volume);
		if(TvOutMode)
		{
            fscanf(Fp,"StartScreenX=%d\n",&StartScreenX);
            fscanf(Fp,"StartScreenY=%d\n",&StartScreenY);
		}
		else
        {
            fscanf(Fp,"StartScreenX=%d\n",&InitialStartScreenX);
            fscanf(Fp,"StartScreenY=%d\n",&InitialStartScreenY);
            StartScreenX = 0;
            StartScreenY = 0;
        }
        fscanf(Fp,"Skin=%s\n",&SkinName);
        for(Teller=0;Teller<InstalledSkinsCount;Teller++)
        {
            if (strcmp(InstalledSkins[Teller],SkinName) == 0)
            {
                SelectedSkin = Teller;
                break;
            }
            else
                SelectedSkin = -1;
        }
		fclose(Fp);
	}
	else
	{
		Volume = 128;
		StartScreenX = (Screen->w - Buffer->w) / 2;
        StartScreenY = (Screen->h - Buffer->h) / 2;
	}
	SetVolume(Volume);
}

void SaveSettings()
{
	FILE *Fp;
	Fp = fopen("./settings.dat","wt");
	if (Fp)
	{
		fprintf(Fp,"Volume=%d\n",Volume);
		if(TvOutMode)
        {
            fprintf(Fp,"StartScreenX=%d\n",StartScreenX);
            fprintf(Fp,"StartScreenY=%d\n",StartScreenY);
        }
        else
        {
            fprintf(Fp,"StartScreenX=%d\n",InitialStartScreenX);
            fprintf(Fp,"StartScreenY=%d\n",InitialStartScreenY);
        }
        fprintf(Fp,"Skin=%s\n",SkinName);
		fclose(Fp);
#ifdef GP2X
		sync();
#endif
	}
}



void LoadGraphics()
{
	FILE *ColorsFile;
	SDL_Surface *Tmp,*Tmp2;
	int R,G,B,A;
	char FileName[FILENAME_MAX];

    if(IMGGrid)
        SDL_FreeSurface(IMGGrid);
	if(IMGWall)
        SDL_FreeSurface(IMGWall);
	if(IMGBackground)
		SDL_FreeSurface(IMGBackground);
	if(IMGFloor)
		SDL_FreeSurface(IMGFloor);
	if(IMGPlayer)
		SDL_FreeSurface(IMGPlayer);
	if(IMGBox)
		SDL_FreeSurface(IMGBox);
	if(IMGEmpty)
		SDL_FreeSurface(IMGEmpty);

    if(IMGDiamond)
		SDL_FreeSurface(IMGDiamond);
	if(IMGBomb)
		SDL_FreeSurface(IMGBomb);
	if(IMGTitleScreen)
		SDL_FreeSurface(IMGTitleScreen);

    if(IMGIntro1)
        SDL_FreeSurface(IMGIntro1);
    if(IMGIntro2)
        SDL_FreeSurface(IMGIntro2);
    if(IMGIntro3)
        SDL_FreeSurface(IMGIntro3);
    if(IMGExplosion)
        SDL_FreeSurface(IMGExplosion);

    sprintf(FileName,"./skins/%s/grid.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/grid.png");
    if(SCREENPOSMULTIPLY > 1)
    {
        Tmp2 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
        IMGGrid = ScaleSurface(Tmp2,SCREENPOSMULTIPLY);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        IMGGrid = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }
    SDL_SetColorKey(IMGGrid, (SDL_SRCCOLORKEY|SDL_RLEACCEL),SDL_MapRGB(IMGGrid->format,255,0,255));

    sprintf(FileName,"./skins/%s/intro1.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/intro1.png");
    if(SCREENPOSMULTIPLY > 1)
    {
        Tmp2 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
        IMGIntro1 = zoomSurface(Tmp2,SCREENPOSMULTIPLY,SCREENPOSMULTIPLY,1);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        IMGIntro1 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }

    sprintf(FileName,"./skins/%s/intro2.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/intro2.png");
    if(SCREENPOSMULTIPLY > 1)
    {
        Tmp2 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
        IMGIntro2 = zoomSurface(Tmp2,SCREENPOSMULTIPLY,SCREENPOSMULTIPLY,1);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        IMGIntro2 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }

    sprintf(FileName,"./skins/%s/intro3.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/intro3.png");
    if(SCREENPOSMULTIPLY > 1)
    {
        Tmp2 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
        IMGIntro3 = zoomSurface(Tmp2,SCREENPOSMULTIPLY,SCREENPOSMULTIPLY,1);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        IMGIntro3 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }


    sprintf(FileName,"./skins/%s/floor.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/floor.png");
    if(SCREENPOSMULTIPLY > 1)
    {
        Tmp2 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
        IMGFloor = ScaleSurface(Tmp2,SCREENPOSMULTIPLY);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        IMGFloor = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }
    SDL_SetColorKey(IMGFloor, (SDL_SRCCOLORKEY|SDL_RLEACCEL),SDL_MapRGB(IMGFloor->format,255,0,255));

    sprintf(FileName,"./skins/%s/explosion.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/explosion.png");
    if(SCREENPOSMULTIPLY > 1)
    {
        Tmp2 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
        IMGExplosion = ScaleSurface(Tmp2,SCREENPOSMULTIPLY);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        IMGExplosion = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }
    SDL_SetColorKey(IMGExplosion, (SDL_SRCCOLORKEY|SDL_RLEACCEL),SDL_MapRGB(IMGExplosion->format,255,0,255));

    sprintf(FileName,"./skins/%s/wall.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/wall.png");
    if(SCREENPOSMULTIPLY > 1)
    {
        Tmp2 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
        IMGWall = ScaleSurface(Tmp2,SCREENPOSMULTIPLY);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        IMGWall = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }
    SDL_SetColorKey(IMGWall, (SDL_SRCCOLORKEY|SDL_RLEACCEL),SDL_MapRGB(IMGWall->format,255,0,255));

    sprintf(FileName,"./skins/%s/bomb.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/bomb.png");
    if(SCREENPOSMULTIPLY > 1)
    {
        Tmp2 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
        IMGBomb= ScaleSurface(Tmp2,SCREENPOSMULTIPLY);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        IMGBomb = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }
    SDL_SetColorKey(IMGBomb, (SDL_SRCCOLORKEY|SDL_RLEACCEL),SDL_MapRGB(IMGBomb->format,255,0,255));

    sprintf(FileName,"./skins/%s/diamond.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/diamond.png");
    if(SCREENPOSMULTIPLY > 1)
    {
        Tmp2 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
        IMGDiamond = ScaleSurface(Tmp2,SCREENPOSMULTIPLY);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        IMGDiamond = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }
    SDL_SetColorKey(IMGDiamond, (SDL_SRCCOLORKEY|SDL_RLEACCEL),SDL_MapRGB(IMGDiamond->format,255,0,255));

    sprintf(FileName,"./skins/%s/box.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/box.png");
    if(SCREENPOSMULTIPLY > 1)
    {
        Tmp2 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
        IMGBox = ScaleSurface(Tmp2,SCREENPOSMULTIPLY);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        IMGBox = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }
    SDL_SetColorKey(IMGBox, (SDL_SRCCOLORKEY|SDL_RLEACCEL),SDL_MapRGB(IMGBox->format,255,0,255));

//    printf("%s\n",SkinName);
	sprintf(FileName,"./skins/%s/player.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/player.png");
    if(SCREENPOSMULTIPLY > 1)
    {
        Tmp2 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
        IMGPlayer = ScaleSurface(Tmp2,SCREENPOSMULTIPLY);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        IMGPlayer = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }
    SDL_SetColorKey(IMGPlayer, (SDL_SRCCOLORKEY|SDL_RLEACCEL),SDL_MapRGB(IMGPlayer->format,255,0,255));

	sprintf(FileName,"./skins/%s/empty.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/empty.png");
    if(SCREENPOSMULTIPLY > 1)
    {
        Tmp2 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
        IMGEmpty = ScaleSurface(Tmp2,SCREENPOSMULTIPLY);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        IMGEmpty = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }
    SDL_SetColorKey(IMGEmpty, (SDL_SRCCOLORKEY|SDL_RLEACCEL),SDL_MapRGB(IMGEmpty->format,255,0,255));


    sprintf(FileName,"./skins/%s/titlescreen.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/titlescreen.png");
    if(SCREENPOSMULTIPLY > 1)
    {
        Tmp2 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
        IMGTitleScreen = zoomSurface(Tmp2,SCREENPOSMULTIPLY,SCREENPOSMULTIPLY,1);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        IMGTitleScreen = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }

    sprintf(FileName,"./skins/%s/background.png",SkinName);
    if (FileExists(FileName))
        Tmp = IMG_Load(FileName);
    else
        Tmp = IMG_Load("./graphics/background.png");
    if(SCREENPOSMULTIPLY > 1)
    {

        Tmp2 = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
        IMGBackground = zoomSurface(Tmp2,SCREENPOSMULTIPLY,SCREENPOSMULTIPLY,1);
        SDL_FreeSurface(Tmp2);
    }
    else
    {
        IMGBackground = SDL_DisplayFormat(Tmp);
        SDL_FreeSurface(Tmp);
    }
    SDL_SetColorKey(IMGBackground, (SDL_SRCCOLORKEY|SDL_RLEACCEL),SDL_MapRGB(IMGBackground->format,255,0,255));

	sprintf(FileName,"./skins/%s/colors.txt",SkinName);
	ColorsFile = fopen(FileName,"rt");
	if (ColorsFile)
	{
		fscanf(ColorsFile,"[TextColor]\nR=%d\nG=%d\nB=%d\n",&R,&G,&B);
		MenuTextColor.r = R;
		MenuTextColor.g = G;
		MenuTextColor.b = B;
		fscanf(ColorsFile,"[MenuBoxColor]\nR=%d\nG=%d\nB=%d\nA=%d\n",&R,&G,&B,&A);
		MenuBoxColor.r = R;
		MenuBoxColor.g = G;
		MenuBoxColor.b = B;
		MenuBoxColor.unused = A;
		fscanf(ColorsFile,"[MenuBoxBorderColor]\nR=%d\nG=%d\nB=%d\nA=%d\n",&R,&G,&B,&A);
		MenuBoxBorderColor.r = R;
		MenuBoxBorderColor.g = G;
		MenuBoxBorderColor.b = B;
		MenuBoxBorderColor.unused = A;
		fclose(ColorsFile);
	}
	else
	{
		MenuTextColor.r = 0;
		MenuTextColor.g = 80;
		MenuTextColor.b = 0;
		MenuBoxColor.r = 73;
		MenuBoxColor.g = 156;
		MenuBoxColor.b = 73;
		MenuBoxColor.unused = 255;
		MenuBoxBorderColor.r = 0;
		MenuBoxBorderColor.g = 80;
		MenuBoxBorderColor.b = 0;
		MenuBoxBorderColor.unused = 255;
	}
}


void SearchForSkins()
{
	struct dirent *Entry;
	DIR *Directory;
	struct stat Stats;
	int Teller=0;
	char FileName[FILENAME_MAX];
	Directory = opendir("./skins");
	if (Directory)
	{
		Entry=readdir(Directory);
		while(Entry)
		{
			sprintf(FileName,"./skins/%s",Entry->d_name);
			stat(FileName,&Stats);
			if(S_ISDIR(Stats.st_mode))
			{
				//printf("%s\n",Entry->d_name);
				if(strncmp(".", Entry->d_name, 1)  && (Teller< MaxSkins) && (strlen(Entry->d_name) < FILENAME_MAX-1))
				{
					sprintf(InstalledSkins[Teller],"%s",Entry->d_name);
					Teller++;
				}
			}
			Entry=readdir(Directory);
		}
		closedir(Directory);
	}
	InstalledSkinsCount = Teller;
	SelectedSkin=-1;
}

void NextSkin()
{
    char Text[500];
    SDL_Color Color = {255,255,255,255};
    SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
    if(InstalledSkinsCount > 0)
    {
        SelectedSkin++;
        if (SelectedSkin >=InstalledSkinsCount)
            SelectedSkin = -1;
        if(SelectedSkin == -1)
        {
            sprintf(SkinName,"%s","");
            sprintf(Text,"Please Wait Loading Skin:\nDefault Skin");
        }
        else
        {
            sprintf(SkinName,"%s",InstalledSkins[SelectedSkin]);
            sprintf(Text,"Please Wait Loading Skin:\n%s",SkinName);
        }
        boxRGBA(Buffer,0,0,319*SCREENPOSMULTIPLY,48*SCREENPOSMULTIPLY,255,255,255,200);
        boxRGBA(Buffer,1*SCREENPOSMULTIPLY,1*SCREENPOSMULTIPLY,318*SCREENPOSMULTIPLY,47*SCREENPOSMULTIPLY,0,0,0,200);
        WriteText(Buffer,BigFontSkin,Text,strlen(Text),5*SCREENPOSMULTIPLY,0,0,Color,false);
        SDL_FillRect(Buffer1,NULL,SDL_MapRGB(Buffer1->format,0,0,0));
        SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
       /* #ifdef WIN32
            ScreenBufferZoom = zoomSurface(Buffer1,2,2,0);
            SDL_BlitSurface(ScreenBufferZoom,NULL,Screen,NULL);
            SDL_FreeSurface(ScreenBufferZoom);
        #else*/
        SDL_BlitSurface(Buffer1,NULL,Screen,NULL);
       // #endif
        SDL_Flip(Screen);
        LoadGraphics();
    }
}

void PreviousSkin()
{
    char Text[500];
    SDL_Color Color = {255,255,255,255};
    SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
    if(InstalledSkinsCount > 0)
    {
        SelectedSkin--;
        if (SelectedSkin <=-2)
            SelectedSkin = InstalledSkinsCount-1;
        if(SelectedSkin == -1)
        {
            sprintf(SkinName,"%s","");
            sprintf(Text,"Please Wait Loading Skin:\nDefault Skin");
        }
        else
        {
            sprintf(SkinName,"%s",InstalledSkins[SelectedSkin]);
            sprintf(Text,"Please Wait Loading Skin:\n%s",SkinName);
        }
        boxRGBA(Buffer,0,0,319*SCREENPOSMULTIPLY,48*SCREENPOSMULTIPLY,255,255,255,200);
        boxRGBA(Buffer,1*SCREENPOSMULTIPLY,1*SCREENPOSMULTIPLY,318*SCREENPOSMULTIPLY,47*SCREENPOSMULTIPLY,0,0,0,200);
        WriteText(Buffer,BigFontSkin,Text,strlen(Text),5*SCREENPOSMULTIPLY,0,0,Color,false);
        SDL_FillRect(Buffer1,NULL,SDL_MapRGB(Buffer1->format,0,0,0));
        SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
       /* #ifdef WIN32
            ScreenBufferZoom = zoomSurface(Buffer1,2,2,0);
            SDL_BlitSurface(ScreenBufferZoom,NULL,Screen,NULL);
            SDL_FreeSurface(ScreenBufferZoom);
        #else*/
        SDL_BlitSurface(Buffer1,NULL,Screen,NULL);
       // #endif
        SDL_Flip(Screen);
        LoadGraphics();
    }
}


void SearchForMusic()
{
	struct dirent *Entry;
	DIR *Directory;
	struct stat Stats;
	int Teller;
	char FileName[FILENAME_MAX];
	if (GlobalSoundEnabled)
		Music[0] = Mix_LoadMUS("./music/title.mod");
	Teller=1;
	Directory = opendir("./music");
	if (Directory)
	{
		Entry=readdir(Directory);
		while(Entry)
		{
			sprintf(FileName,"./music/%s",Entry->d_name);
			stat(FileName,&Stats);
			if(!S_ISDIR(Stats.st_mode))
			{
				if(strncmp(".", Entry->d_name, 1) && (strcmp("title.mod",Entry->d_name) != 0) && (Teller< MaxMusicFiles))
				{
				//	printf("%s\n",Entry->d_name);
					if (GlobalSoundEnabled)
					{
						Music[Teller] = Mix_LoadMUS(FileName);
						Teller++;
					}
				}
			}
			Entry=readdir(Directory);
		}
		closedir(Directory);
	}
	MusicCount = Teller;
}

void SearchForLevelPacks()
{
	struct dirent *Entry;
	DIR *Directory;
	struct stat Stats;
	int Teller=0;
	char FileName[FILENAME_MAX];
	Directory = opendir("./levelpacks");
	if (Directory)
	{
		Entry=readdir(Directory);
		while(Entry)
		{
			sprintf(FileName,"./levelpacks/%s",Entry->d_name);
			stat(FileName,&Stats);
			if(S_ISDIR(Stats.st_mode))
			{
				//printf("%s\n",Entry->d_name);
				if(strncmp(".", Entry->d_name, 1)  && (Teller< MaxLevelPacks) && (strlen(Entry->d_name) < 21))
				{
					sprintf(InstalledLevelPacks[Teller],"%s",Entry->d_name);
					RemoveUnderScores(InstalledLevelPacks[Teller]);
					Teller++;
				}
			}
			Entry=readdir(Directory);
		}
		closedir(Directory);
	}
	InstalledLevelPacksCount = Teller;
	SelectedLevelPack=0;
	if (InstalledLevelPacksCount > 0)
	{
		sprintf(LevelPackName,"%s",InstalledLevelPacks[SelectedLevelPack]);
		sprintf(LevelPackFileName,"%s",InstalledLevelPacks[SelectedLevelPack]);
		AddUnderScores(LevelPackFileName);

	}
	else
	{
		sprintf(LevelPackName,"%s","");
		sprintf(LevelPackFileName,"%s","");
	}
}

bool StageDone()
{
	bool Temp = false;
	int Teller,NumDiamonds=0;
	for (Teller=0;Teller<WorldParts.ItemCount;Teller++)
		if (WorldParts.Items[Teller]->GetType() == IDDiamond)
		{
			NumDiamonds++;
		}
    return (NumDiamonds ==0);
}

bool AskQuestion(char *Msg)
{
	SDL_Color Color = {0,0,0,0};
	CInput *Input = new CInput(InputDelay);
	SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
	boxRGBA(Buffer,60*SCREENPOSMULTIPLY,80*SCREENPOSMULTIPLY,260*SCREENPOSMULTIPLY,160*SCREENPOSMULTIPLY,MenuBoxColor.r,MenuBoxColor.g,MenuBoxColor.b,MenuBoxColor.unused);
	rectangleRGBA(Buffer,60*SCREENPOSMULTIPLY,80*SCREENPOSMULTIPLY,260*SCREENPOSMULTIPLY,160*SCREENPOSMULTIPLY,MenuBoxBorderColor.r,MenuBoxBorderColor.g,MenuBoxBorderColor.b,MenuBoxBorderColor.unused);
	rectangleRGBA(Buffer,61*SCREENPOSMULTIPLY,81*SCREENPOSMULTIPLY,259*SCREENPOSMULTIPLY,159*SCREENPOSMULTIPLY,MenuBoxBorderColor.r,MenuBoxBorderColor.g,MenuBoxBorderColor.b,MenuBoxBorderColor.unused);
	WriteText(Buffer,font,Msg,strlen(Msg),65*SCREENPOSMULTIPLY,83*SCREENPOSMULTIPLY,2*SCREENPOSMULTIPLY,MenuTextColor,false);
	SDL_FillRect(Buffer1,NULL,SDL_MapRGB(Buffer1->format,0,0,0));
    SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
    SDL_BlitSurface(Buffer1,NULL,Screen,NULL);
    SDL_Flip(Screen);
	{
		while (!( Input->SpecialsHeld[SPECIAL_QUIT_EV] || Input->JoystickHeld[0][GP2X_BUTTON_A] || Input->JoystickHeld[0][GP2X_BUTTON_X] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_A)] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_X)] || Input->KeyboardHeld[SDLK_a] || Input->KeyboardHeld[SDLK_q] || Input->KeyboardHeld[SDLK_y] || Input->KeyboardHeld[SDLK_n] || Input->KeyboardHeld[SDLK_x]))
		{
		    Input->Update();
			if(GlobalSoundEnabled)
			if(!Mix_PlayingMusic())
			{
				Mix_PlayMusic(Music[SelectedMusic],0);
				SetVolume(Volume);
			}
			SDL_framerateDelay(&Fpsman);
		}
		if (Input->SpecialsHeld[SPECIAL_QUIT_EV])
            GameState = GSQuit;
		if (Input->JoystickHeld[0][GP2X_BUTTON_A] ||Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_A)] || Input->KeyboardHeld[SDLK_a] || Input->KeyboardHeld[SDLK_q] || Input->KeyboardHeld[SDLK_y])
			return true;
		else
			return false;

	}
	delete Input;
}

void PrintForm(char *msg)
{
    CInput *Input = new CInput(InputDelay);
	SDL_Color Color = {0,0,0,0};
	SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
	boxRGBA(Buffer,60*SCREENPOSMULTIPLY,80*SCREENPOSMULTIPLY,260*SCREENPOSMULTIPLY,160*SCREENPOSMULTIPLY,MenuBoxColor.r,MenuBoxColor.g,MenuBoxColor.b,MenuBoxColor.unused);
	rectangleRGBA(Buffer,60*SCREENPOSMULTIPLY,80*SCREENPOSMULTIPLY,260*SCREENPOSMULTIPLY,160*SCREENPOSMULTIPLY,MenuBoxBorderColor.r,MenuBoxBorderColor.g,MenuBoxBorderColor.b,MenuBoxBorderColor.unused);
	rectangleRGBA(Buffer,61*SCREENPOSMULTIPLY,81*SCREENPOSMULTIPLY,259*SCREENPOSMULTIPLY,159*SCREENPOSMULTIPLY,MenuBoxBorderColor.r,MenuBoxBorderColor.g,MenuBoxBorderColor.b,MenuBoxBorderColor.unused);
	WriteText(Buffer,font,msg,strlen(msg),65*SCREENPOSMULTIPLY,83*SCREENPOSMULTIPLY,2*SCREENPOSMULTIPLY,MenuTextColor,false);
    SDL_FillRect(Buffer1,NULL,SDL_MapRGB(Buffer1->format,0,0,0));
    SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
    SDL_BlitSurface(Buffer1,NULL,Screen,NULL);
    SDL_Flip(Screen);
    while (!( Input->SpecialsHeld[SPECIAL_QUIT_EV] || Input->JoystickHeld[0][GP2X_BUTTON_A] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_A)] || Input->KeyboardHeld[SDLK_a] || Input->KeyboardHeld[SDLK_q] || Input->KeyboardHeld[SDLK_RETURN] || Input->KeyboardHeld[SDLK_SPACE]))
    {
        Input->Update();
        if(GlobalSoundEnabled)
        if(!Mix_PlayingMusic())
        {
            Mix_PlayMusic(Music[SelectedMusic],0);
            //Mix_HookMusicFinished(MusicFinished);
            SetVolume(Volume);
        }
        SDL_framerateDelay(&Fpsman);
    }

	delete Input;
}

void SaveUnlockData()
{
	FILE *Fp;
	int Teller;
	char Filename[FILENAME_MAX];
	unsigned char LevelHash[4];
	unsigned char HashBuffer[64];
	int CheckSum = 0;
	LevelHash[0] = HashTable[UnlockedLevels-1] ;
	LevelHash[1] = HashTable[UnlockedLevels];
	LevelHash[2] = HashTable[UnlockedLevels+1];
	LevelHash[3] = HashTable[UnlockedLevels+2];
	sprintf(Filename,"./%s.dat",LevelPackFileName);
	for (Teller=0;Teller<4;Teller++)
		LevelHash[Teller] = LevelHash[Teller] ^ LevelPackFileName[strlen(LevelPackFileName)-1];
	for (Teller=0;Teller<strlen(LevelPackFileName);Teller++)
		LevelHash[Teller%4] = LevelHash[Teller%4] ^ LevelPackFileName[Teller];
	LevelHash[0] = LevelHash[0] ^ LevelHash[2];
	LevelHash[1] = LevelHash[1] ^ LevelHash[0];
	LevelHash[2] = LevelHash[2] ^ LevelHash[3];
	LevelHash[3] = LevelHash[3] ^ LevelHash[2];
	for (Teller=0;Teller<64;Teller++)
		if ((Teller+1) % 16 == 0)
		{
			HashBuffer[Teller] = LevelHash[(Teller)/16];
			CheckSum += HashBuffer[Teller];
		}
		else
		{
			HashBuffer[Teller] = rand() % 256;
			CheckSum += HashBuffer[Teller];
		}
	CheckSum = CheckSum ^ 50;
	Fp = fopen(Filename,"wb");
	if (Fp)
	{
		fwrite(HashBuffer,1,64,Fp);
		fwrite(&CheckSum,sizeof(int),1,Fp);
		fclose(Fp);
#ifdef GP2X
		sync();
#endif
	}
}

/*void SaveAllHashes()
{
	FILE *Fp;
	char Filename[FILENAME_MAX];
	char LevelHash[4];
	int Teller;
	sprintf(Filename,"./hashes.txt");
	Fp = fopen(Filename,"wt");
	if (Fp)
	{
		for (Teller=0;Teller<1000;Teller++)
		{
			LevelHash[0] = HashTable[Teller];
			LevelHash[1] = HashTable[Teller+1];
			LevelHash[2] = HashTable[Teller+2];
			LevelHash[3] = HashTable[Teller+3];

			fprintf(Fp,"%d|%d|%d|%d\n",(int)LevelHash[0],(int)LevelHash[1],(int)LevelHash[2],(int)LevelHash[3]);

		}
	fclose(Fp);
	sync();
	}
}
*/

void LoadUnlockData()
{
	FILE *Fp;
	unsigned char LevelHash[4];
	int Teller=0;
	unsigned char HashBuffer[64];
	char Filename[FILENAME_MAX];
	sprintf(Filename,"./%s.dat",LevelPackFileName);
	Fp = fopen(Filename,"rb");
	int CheckSum,ValidCheckSum=0;
	if (Fp)
	{
		fflush(Fp);
		fread(HashBuffer,1,64,Fp);
		fread(&CheckSum,sizeof(int),1,Fp);
		fclose(Fp);
		for (Teller = 0 ;Teller<64;Teller++)
		{
			ValidCheckSum += HashBuffer[Teller];
			if ((Teller+1)%16 == 0)
				LevelHash[Teller/16] = HashBuffer[Teller];
		}
		CheckSum = CheckSum ^ 50;
		if (CheckSum == ValidCheckSum)
		{
			LevelHash[3] = LevelHash[3] ^ LevelHash[2];
			LevelHash[2] = LevelHash[2] ^ LevelHash[3];
			LevelHash[1] = LevelHash[1] ^ LevelHash[0];
			LevelHash[0] = LevelHash[0] ^ LevelHash[2];
			for (Teller=0;Teller<strlen(LevelPackFileName);Teller++)
				LevelHash[Teller%4] = LevelHash[Teller%4] ^ LevelPackFileName[Teller];
			for (Teller=0;Teller<4;Teller++)
				LevelHash[Teller] = LevelHash[Teller] ^ LevelPackFileName[strlen(LevelPackFileName)-1];

			Teller=0;
			while (LevelHash[0] != HashTable[Teller] || LevelHash[1] != HashTable[Teller+1] || 	LevelHash[2] != HashTable[Teller+2] || LevelHash[3] != HashTable[Teller+3] && Teller+3 < 1004)
				Teller++;
			if (Teller < InstalledLevels)
				UnlockedLevels = Teller+1;
			else
				UnlockedLevels = 1;
		}
		else
			UnlockedLevels = 1;
	}
	else
	 	UnlockedLevels = 1;

}


/*void Game2()
{
	//Uint32 StartTime=0,EndTime=0;
	int Teller,Moves=0;
	char ChrDebug[200];
	char Msg[300];
	char FileName[FILENAME_MAX];
	SDL_Event Event;
	SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
	SDL_PollEvent(&Event);
	if (GlobalSoundEnabled)
		Mix_HaltMusic();
	Moves=0;
	Uint8 *KeyState;
	if (MusicCount > 1)
 	{
 		SelectedMusic =	1+rand()%(MusicCount-1);
		if(GlobalSoundEnabled)
		{
			Mix_PlayMusic(Music[SelectedMusic],0);
			//Mix_HookMusicFinished(MusicFinished);
			SetVolume(Volume);
		}
	}
	CPlayer *Player=0;
	for (Teller=0;Teller<WorldParts.ItemCount;Teller++)
	{
		if (WorldParts.Items[Teller]->GetType() == IDPlayer)
		{
			Player = WorldParts.Items[Teller];
			break;
		}
	}
	//should never happen
	if(!Player)
	{
		Player = new CPlayer(0,0);
		WorldParts.Add(Player);
	}

	while (GameState == GSGame)
	{
		if (ReloadMusic)
		{
			ReloadMusic=false;
			Mix_PlayMusic(Music[SelectedMusic],0);
			//Mix_HookMusicFinished(MusicFinished);
			SetVolume(Volume);
		}
		while (SDL_PollEvent(&Event))
		{
			if (Event.type == SDL_JOYBUTTONDOWN)
			{
				switch (Event.jbutton.button)
				{
					case GP2X_BUTTON_VOLUP:
						IncVolume();
						break;
					case GP2X_BUTTON_VOLDOWN:
						DecVolume();
						break;
					case GP2X_BUTTON_SELECT:
						if(!LevelEditorMode)
							GameState = GSStageSelect;
						break;
					case GP2X_BUTTON_START:
						if(LevelEditorMode)
							GameState=GSLevelEditor;
						break;
					case GP2X_BUTTON_R:
						SelectedMusic++;
						if (SelectedMusic >= MusicCount)
							SelectedMusic = 1;
						if(GlobalSoundEnabled)
						{
							Mix_HaltMusic();
							Mix_PlayMusic(Music[SelectedMusic],0);
							//Mix_HookMusicFinished(MusicFinished);
							SetVolume(Volume);
						}
						break;
					case GP2X_BUTTON_L:
						SelectedMusic--;
						if (SelectedMusic <= 0)
							SelectedMusic = MusicCount-1;

						if(GlobalSoundEnabled)
						{
							Mix_HaltMusic();
							Mix_PlayMusic(Music[SelectedMusic],0);
							//Mix_HookMusicFinished(MusicFinished);
							SetVolume(Volume);
						}
						break;
					case GP2X_BUTTON_Y:
						SDL_BlitSurface(IMGBackground,NULL,Buffer,NULL);
						WorldParts.Draw(Buffer);
						if(AskQuestion("You are about to restart this level\nAre you sure you want to restart?\n\nPress (A) to Restart (X) to Cancel"))
						{
							sprintf(FileName,"./levelpacks/%s/level%d.lev",LevelPackFileName,SelectedLevel);
							WorldParts.Load(FileName);
							Moves=0;
							for (Teller=0;Teller<WorldParts.ItemCount;Teller++)
							{
								if (WorldParts.Items[Teller]->GetType() == IDPlayer)
								{
									Player = WorldParts.Items[Teller];
									break;
								}
							}
						}
						break;
					case GP2X_BUTTON_B:
						SDL_BlitSurface(IMGBackground,NULL,Buffer,NULL);
						WorldParts.Draw(Buffer);
						sprintf(Msg,"Level Pack: %s\nLevel: %d/%d - Moves: %d\n\nPress (A) To continue playing",LevelPackName,SelectedLevel,InstalledLevels,Moves);
						PrintForm(Msg);
						SDL_Delay(250);
						break;
				}
			}
		}
		//printf("Start while\n");
		if (!Player->IsMoving)
		{
			KeyState = SDL_GetKeyState(NULL);
			if (SDL_JoystickGetButton(Joystick,GP2X_BUTTON_A))
			{
				WorldParts.HistoryGoBack();
				if (Moves > 0)
					Moves--;
			}
			if (SDL_JoystickGetButton(Joystick,GP2X_BUTTON_RIGHT))
			{
				if (Player->CanMoveTo(Player->GetPlayFieldX() + 1, Player->GetPlayFieldY()))
				{
					WorldParts.HistoryAdd();
					Moves++;
				}
				Player->MoveTo(Player->GetPlayFieldX() + 1, Player->GetPlayFieldY(),false);

			}
			if (SDL_JoystickGetButton(Joystick,GP2X_BUTTON_LEFT))
			{
				if (Player->CanMoveTo(Player->GetPlayFieldX() - 1, Player->GetPlayFieldY()))
				{
					WorldParts.HistoryAdd();
					Moves++;
				}
				Player->MoveTo(Player->GetPlayFieldX() - 1, Player->GetPlayFieldY(),false);
			}
			if (SDL_JoystickGetButton(Joystick,GP2X_BUTTON_UP))
			{
				if (Player->CanMoveTo(Player->GetPlayFieldX() , Player->GetPlayFieldY()-1))
				{
					WorldParts.HistoryAdd();
					Moves++;
				}
				Player->MoveTo(Player->GetPlayFieldX(), Player->GetPlayFieldY() - 1,false);
			}
			if (SDL_JoystickGetButton(Joystick,GP2X_BUTTON_DOWN))
			{
				if (Player->CanMoveTo(Player->GetPlayFieldX() , Player->GetPlayFieldY()+1))
				{
					WorldParts.HistoryAdd();
					Moves++;
				}
				Player->MoveTo(Player->GetPlayFieldX(), Player->GetPlayFieldY() +1,false);
			}
		}

		//SDL_FillRect(Buffer,NULL,SDL_MapRGB(Buffer->format,255,255,255));
		SDL_BlitSurface(IMGBackground,NULL,Buffer,NULL);

		//printf("voor Move all objects\n");
		WorldParts.Move();
		//printf("voor Draw all objects\n");
		WorldParts.Draw(Buffer);
		//StartTime = EndTime;
		//EndTime = SDL_GetTicks();
		//sprintf(ChrDebug,"fps %d",1000 / (EndTime-StartTime));
		//WriteText(Buffer,font,ChrDebug,strlen(ChrDebug),0,0,0,MenuTextColor);
		//printf("voor fps show: Endtime:%d StartTime:%d End-start:%d\n",EndTime,StartTime,EndTime-StartTime);
		//printf("Na Fps Show\n");
		SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
        SDL_BlitSurface(Buffer,NULL,Screen,&Rect);
        SDL_Flip(Screen);
		if (!Player->IsMoving && StageDone(Player))
		{
			SDL_Delay(250);
			if (GlobalSoundEnabled)
				Mix_PlayChannel(-1,Sounds[SND_STAGEEND],0);
			if (LevelEditorMode)
			{
				if (AskQuestion("Congratulations !\nYou Succesfully Solved this level\nDo you want to return to the\nlevel editor ?\n(A) Leveleditor (X) Play Again"))
					GameState = GSLevelEditor;
				else
				{
					sprintf(FileName,"./levelpacks/%s/level%d.lev",LevelPackFileName,SelectedLevel);
					WorldParts.Load(FileName);
					Moves=0;
					for (Teller=0;Teller<WorldParts.ItemCount;Teller++)
					{
						if (WorldParts.Items[Teller]->GetType() == IDPlayer)
						{
							Player = WorldParts.Items[Teller];
							break;
						}
					}
				}

			}
			else
			{
				if (SelectedLevel == UnlockedLevels)
				{
					if ( UnlockedLevels < InstalledLevels)
					{
						sprintf(Msg,"Congratulations !\nYou Succesfully Solved Level %d/%d\nThe next level has now been unlocked!\n\nPress (A) to continue",SelectedLevel,InstalledLevels);
						PrintForm(Msg);
						UnlockedLevels++;
						SelectedLevel++;
						SaveUnlockData();
						GameState = GSStageSelect;
					}
					else
					{
						sprintf(Msg,"Congratulations !\nYou Succesfully Solved Level %d/%d\nlevelpack %s\nis now finished, try out another one!\n\nPress (A) to continue",SelectedLevel,InstalledLevels,LevelPackName);
						PrintForm(Msg);
						GameState = GSTitleScreen;
					}
				}
				else
				{
					sprintf(Msg,"Congratulations !\nYou Succesfully Solved Level %d/%d\n\nPress (A) to continue",SelectedLevel,InstalledLevels);
					PrintForm(Msg);
					GameState = GSStageSelect;

				}
			}
		}
		SDL_Delay(WaitForFrame());
		//printf("Start End while\n");
	}
	if (GlobalSoundEnabled)
		Mix_HaltMusic();
	SelectedMusic++;
	if (SelectedMusic >= MusicCount)
		SelectedMusic = 1;
}

*/

char *GetString(char *NameIn,char *Msg)
{
	char *PackName = new char[21];
	bool End=false,SubmitChanges=false;
	int Teller,MaxSelection=0, Selection = 0,asci=97;
	CInput *Input = new CInput(InputDelay);
	SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
	sprintf(PackName,"%s",NameIn);
	MaxSelection = strlen(NameIn);
	PackName[Selection+1]='\0';
	if (MaxSelection == 0)
		PackName[MaxSelection]=chr(asci);
	char Tekst[100];
	while (!End)
	{
	    if(GlobalSoundEnabled)
		if(!Mix_PlayingMusic())
        {
            Mix_PlayMusic(Music[SelectedMusic],0);
            SetVolume(Volume);
        }
		Input->Update();

        if(Input->SpecialsHeld[SPECIAL_QUIT_EV])
            GameState = GSQuit;

        if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_LEFT] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_LEFT)] || Input->KeyboardHeld[SDLK_LEFT]))
        {
            if (Selection > 0)
            {	Selection--;
                asci = ord(PackName[Selection]);
            }
            Input->Delay();
        }

        if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_RIGHT] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_RIGHT)] || Input->KeyboardHeld[SDLK_RIGHT]))
        {
            if (Selection < 19)
            {
                Selection++;
                if (Selection > MaxSelection)
                {
                    PackName[Selection] = chr(97);
                    PackName[Selection+1] = '\0';
                    MaxSelection=Selection;
                }
                asci = ord(PackName[Selection]);
            }
            Input->Delay();
        }

        if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_UP] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_UP)] || Input->KeyboardHeld[SDLK_UP]))
        {
            asci++;
            if (asci==123)
                asci=32;
            if (asci==33)
                (asci=48);
            if (asci==58)
                asci=97;
                PackName[Selection] = chr(asci);
            Input->Delay();
        }

        if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_DOWN] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_DOWN)] || Input->KeyboardHeld[SDLK_DOWN]))
        {
            asci--;
            if(asci==96)
                    asci=57;
            if(asci==47)
                asci=32;
            if(asci==31)
                asci=122;
            PackName[Selection] = chr(asci);
            Input->Delay();
        }

        if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_A] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_A)] || Input->KeyboardHeld[SDLK_q] || Input->KeyboardHeld[SDLK_a] || Input->KeyboardHeld[SDLK_RETURN]))
        {
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
            End = true;
            SubmitChanges=true;
        }

        if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_X] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_X)] || Input->KeyboardHeld[SDLK_x] || Input->KeyboardHeld[SDLK_ESCAPE] ))
        {
            End=true;
            SubmitChanges=false;
        }
		SDL_BlitSurface(IMGTitleScreen,NULL,Buffer,NULL);
		boxRGBA(Buffer,60*SCREENPOSMULTIPLY,80*SCREENPOSMULTIPLY,260*SCREENPOSMULTIPLY,160*SCREENPOSMULTIPLY,MenuBoxColor.r,MenuBoxColor.g,MenuBoxColor.b,MenuBoxColor.unused);
		rectangleRGBA(Buffer,60*SCREENPOSMULTIPLY,80*SCREENPOSMULTIPLY,260*SCREENPOSMULTIPLY,160*SCREENPOSMULTIPLY,MenuBoxBorderColor.r,MenuBoxBorderColor.g,MenuBoxBorderColor.b,MenuBoxBorderColor.unused);
		rectangleRGBA(Buffer,61*SCREENPOSMULTIPLY,81*SCREENPOSMULTIPLY,259*SCREENPOSMULTIPLY,159*SCREENPOSMULTIPLY,MenuBoxBorderColor.r,MenuBoxBorderColor.g,MenuBoxBorderColor.b,MenuBoxBorderColor.unused);
		WriteText(Buffer,font,Msg,strlen(Msg),65*SCREENPOSMULTIPLY,85*SCREENPOSMULTIPLY,2*SCREENPOSMULTIPLY,MenuTextColor,false);
		WriteText(Buffer,MonoFont,PackName,strlen(PackName),85*SCREENPOSMULTIPLY,110*SCREENPOSMULTIPLY,2*SCREENPOSMULTIPLY,MenuTextColor,false);
		if (Selection > 0)
		{
			sprintf(Tekst," ");
			for (Teller=1;Teller<Selection;Teller++)
				sprintf(Tekst,"%s ",Tekst);
			sprintf(Tekst,"%s_",Tekst);
		}
		else
			sprintf(Tekst,"_");
		WriteText(Buffer,MonoFont,Tekst,strlen(Tekst),85*SCREENPOSMULTIPLY,112*SCREENPOSMULTIPLY,2*SCREENPOSMULTIPLY,MenuTextColor,false);
		sprintf(Tekst,"Use Up,Down,Left,right. A = Ok X = Cancel" );
		WriteText(Buffer,font,Tekst,strlen(Tekst),65*SCREENPOSMULTIPLY,135*SCREENPOSMULTIPLY,2*SCREENPOSMULTIPLY,MenuTextColor,false);
        SDL_FillRect(Buffer1,NULL,SDL_MapRGB(Buffer1->format,0,0,0));
        SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
        SDL_BlitSurface(Buffer1,NULL,Screen,NULL);
        SDL_Flip(Screen);
        SDL_framerateDelay(&Fpsman);
	}
	PackName[MaxSelection+1] = '\0';
	while ((PackName[0] == ' ') && (MaxSelection>-1))
	{
		for (Teller=0;Teller<MaxSelection;Teller++)
			PackName[Teller] = PackName[Teller+1];
		MaxSelection--;
	}
	if (MaxSelection>-1)
		while ((PackName[MaxSelection] == ' ') && (MaxSelection>0))
		{
			PackName[MaxSelection] = '\0';
			MaxSelection--;
		}
	if (!SubmitChanges)
		sprintf(PackName,"%s",NameIn);
	delete Input;
	return PackName;
}



void StageSelect()
{
    SDL_Surface *Tmp;
	SDL_Event Event;
	SDL_PollEvent(&Event);
	int Teller;
	int alpha = 0;
	char *FileName = new char[FILENAME_MAX];
	char Tekst[300];
	char Tekst1[300];
	SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
    Tmp = SDL_DisplayFormat(Buffer);
    CInput *Input = new CInput(InputDelay);
	if (SelectedLevel > 0)
	{
		sprintf(FileName,"./levelpacks/%s/level%d.lev",LevelPackFileName,SelectedLevel);
		WorldParts.Load(FileName);

	}
	else
		WorldParts.RemoveAll();
	while (GameState == GSStageSelect)
	{
        if(GlobalSoundEnabled)
            if (! Mix_PlayingMusic())
            {
                SelectedMusic = 0;
                Mix_PlayMusic(Music[SelectedMusic],0);
				//Mix_HookMusicFinished(MusicFinished);
                SetVolume(Volume);
            }
		SDL_BlitSurface(IMGBackground,NULL,Tmp,NULL);
		WorldParts.Draw(Tmp);
		boxRGBA(Tmp,0,0,319*SCREENPOSMULTIPLY,13*SCREENPOSMULTIPLY,MenuBoxColor.r,MenuBoxColor.g,MenuBoxColor.b,MenuBoxColor.unused);
		rectangleRGBA(Tmp,0,-1,319*SCREENPOSMULTIPLY,13*SCREENPOSMULTIPLY,MenuBoxBorderColor.r,MenuBoxBorderColor.g,MenuBoxBorderColor.b,MenuBoxBorderColor.unused);
		if (SelectedLevel ==0)
			sprintf(Tekst,"Level Pack: %s -> %d Levels - (A) Create New Level",LevelPackName,InstalledLevels);
		else
			if(LevelEditorMode)
				sprintf(Tekst,"Level Pack: %s Level:%d/%d - (A) Edit Level (X) Delete Level",LevelPackName,SelectedLevel,InstalledLevels);
			else
				if(SelectedLevel <= UnlockedLevels)
					sprintf(Tekst,"Level Pack: %s Level:%d/%d - (A) Play Level",LevelPackName,SelectedLevel,InstalledLevels);
				else
					sprintf(Tekst,"Level Pack: %s Level:%d/%d - Level is locked!",LevelPackName,SelectedLevel,InstalledLevels);
		WriteText(Tmp,font,Tekst,strlen(Tekst),2*SCREENPOSMULTIPLY,0,0,MenuTextColor,false);
		Input->Update();

        if(Input->SpecialsHeld[SPECIAL_QUIT_EV])
            GameState = GSQuit;

        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_VOLUP] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_VOLUP)] || Input->KeyboardHeld[SDLK_KP_PLUS]))
        {
            IncVolume();
            Input->Delay();
        }

        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_VOLDOWN] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_VOLMIN)] || Input->KeyboardHeld[SDLK_KP_MINUS]))
        {
            DecVolume();
            Input->Delay();
        }

        if(Input->JoystickHeld[0][GP2X_BUTTON_Y]  || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_Y)] || Input->KeyboardHeld[SDLK_y])
        {
            if(LevelEditorMode)
            {
                SelectedLevel = 0;
                WorldParts.RemoveAll();
                printf("worldparts :%d\n",WorldParts.ItemCount);
                LevelHasChanged = false;
                GameState = GSLevelEditor;
            }
           // else
                //printf("not leveleditor mode\n");
        }

        if(Input->JoystickHeld[0][GP2X_BUTTON_SELECT] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_SELECT)] || Input->KeyboardHeld[SDLK_ESCAPE])
        {
            if(LevelEditorMode)
                GameState= GSLevelEditorMenu;
            else
                GameState= GSTitleScreen;
            WorldParts.RemoveAll();
        }

        if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_X] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_X)] || Input->KeyboardHeld[SDLK_x]))
        {
            if(LevelEditorMode && (SelectedLevel > 0))
            {
                sprintf(Tekst,"Are you sure you want to delete this level:\n%s - Level %d\n\nPress (A) to Delete (X) to Cancel",LevelPackName,SelectedLevel);
                if (AskQuestion(Tekst))
                {
                    sprintf(Tekst,"./levelpacks/%s/level%d.lev",LevelPackFileName,SelectedLevel);
                    //system(Command);
                    remove(Tekst);
                    //printf("%s\n",Command);
                    for(Teller=SelectedLevel;Teller<InstalledLevels;Teller++)
                    {
                        sprintf(Tekst,"./levelpacks/%s/level%d.lev",LevelPackFileName,Teller+1);
                        sprintf(Tekst1,"./levelpacks/%s/level%d.lev",LevelPackFileName,Teller);
                        rename(Tekst,Tekst1);
                      //  system(Command);
                        //printf("%s\n",Command);
                    }
#ifdef GP2X
                    sync();
#endif
                    InstalledLevels--;
                    if (SelectedLevel > InstalledLevels)
                        SelectedLevel = InstalledLevels;
                    if (SelectedLevel==0)
                        WorldParts.RemoveAll();
                    else
                    {
                        sprintf(FileName,"./levelpacks/%s/level%d.lev",LevelPackFileName,SelectedLevel);
                        WorldParts.Load(FileName);
                    }
                }
                Input->Reset();
            }
            Input->Delay();
        }

        if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_A] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_A)] || Input->KeyboardHeld[SDLK_a] || Input->KeyboardHeld[SDLK_q] || Input->KeyboardHeld[SDLK_RETURN] || Input->KeyboardHeld[SDLK_SPACE]))
        {
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
            if(LevelEditorMode)
            {
                LevelHasChanged = false;
                GameState = GSLevelEditor;
            }
            else
                if (SelectedLevel <= UnlockedLevels)
                    GameState = GSGame;
                else
                {
                    sprintf(Tekst,"This Level Hasn't been unlocked yet!\nDo you want to play the last unlocked\nlevel %d/%d\n\nPress (A) to Play (X) to Cancel",UnlockedLevels,InstalledLevels);
                    if	(AskQuestion(Tekst))
                    {
                        SelectedLevel = UnlockedLevels;
                        sprintf(FileName,"./levelpacks/%s/level%d.lev",LevelPackFileName,SelectedLevel);
                        WorldParts.Load(FileName);
                        GameState = GSGame;
                    }
                    Input->Reset();
                }
            Input->Delay();
        }

        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_L] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_L)] || Input->KeyboardHeld[SDLK_PAGEDOWN] || Input->KeyboardHeld[SDLK_l]))
        {
            SelectedLevel -= 5;
            if(LevelEditorMode)
            {
                if (SelectedLevel <= 0)
                {
                    SelectedLevel = 0;
                    WorldParts.RemoveAll();
                }
                else
                {
                    sprintf(FileName,"./levelpacks/%s/level%d.lev",LevelPackFileName,SelectedLevel);
                    WorldParts.Load(FileName);
                }
            }
            else
            {
                if (SelectedLevel < 1)
                    SelectedLevel = 1;
                sprintf(FileName,"./levelpacks/%s/level%d.lev",LevelPackFileName,SelectedLevel);
                WorldParts.Load(FileName);
            }
            Input->Delay();
        }

        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_R] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_R)] || Input->KeyboardHeld[SDLK_PAGEUP] || Input->KeyboardHeld[SDLK_r]))
        {
            SelectedLevel +=5;
            if (SelectedLevel > InstalledLevels)
                    SelectedLevel = InstalledLevels;
            sprintf(FileName,"./levelpacks/%s/level%d.lev",LevelPackFileName,SelectedLevel);
            WorldParts.Load(FileName);
            Input->Delay();
        }

        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_LEFT] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_LEFT)] || Input->KeyboardHeld[SDLK_LEFT]))
        {
            SelectedLevel--;
            if(LevelEditorMode)
            {
                if (SelectedLevel <= 0)
                {
                    SelectedLevel = 0;
                    WorldParts.RemoveAll();
                }
                else
                {
                    sprintf(FileName,"./levelpacks/%s/level%d.lev",LevelPackFileName,SelectedLevel);
                    WorldParts.Load(FileName);
                }
            }
            else
            {
                if (SelectedLevel < 1)
                    SelectedLevel = 1;
                sprintf(FileName,"./levelpacks/%s/level%d.lev",LevelPackFileName,SelectedLevel);
                WorldParts.Load(FileName);
            }

            Input->Delay();
        }

        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_RIGHT] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_RIGHT)] || Input->KeyboardHeld[SDLK_RIGHT]))
        {
            SelectedLevel++;
            if (SelectedLevel > InstalledLevels)
                SelectedLevel = InstalledLevels;
            sprintf(FileName,"./levelpacks/%s/level%d.lev",LevelPackFileName,SelectedLevel);
            WorldParts.Load(FileName);
            Input->Delay();
        }


        if (alpha < 255)
        {
            if(alpha+AlphaInc > MaxAlpha)
            {
                alpha = 255;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
            else
            {
                alpha+=AlphaInc;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
        }
        SDL_BlitSurface(Tmp,NULL,Buffer,NULL);
        SDL_FillRect(Buffer1,NULL,SDL_MapRGB(Buffer1->format,0,0,0));
        SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
        SDL_BlitSurface(Buffer1,NULL,Screen,NULL);
        SDL_Flip(Screen);
        SDL_framerateDelay(&Fpsman);

	}
	delete[] FileName;
	SDL_FreeSurface(Tmp);
	delete Input;
}


void FindLevels()
{
	int Teller=1;
	char *FileName = new char[FILENAME_MAX];
	InstalledLevels = 0;
	sprintf(FileName,"./levelpacks/%s/level%d.lev",LevelPackFileName,Teller);
	while (FileExists(FileName))
	{
		Teller+=25;
		sprintf(FileName,"./levelpacks/%s/level%d.lev",LevelPackFileName,Teller);
	}
	while (!FileExists(FileName) && (Teller >=1) )
	{
		Teller--;
		sprintf(FileName,"./levelpacks/%s/level%d.lev",LevelPackFileName,Teller);
	}
	InstalledLevels=Teller;
	delete[] FileName;
}

void Credits()
{
    int alpha = 0;
    SDL_Surface *Tmp;
	CInput *Input = new CInput(InputDelay);
	char *LevelPackCreator = new char[21];
	char FileName[FILENAME_MAX];
	FILE *Fp;
	SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
    Tmp = SDL_DisplayFormat(Buffer);
	char *Tekst = new char[500];
	sprintf(FileName,"./levelpacks/%s/credits.dat",LevelPackFileName);
	if(InstalledLevelPacks > 0)
	{
		Fp = fopen(FileName,"rt");
		if (Fp)
		{
			fscanf(Fp,"[Credits]\nCreator='%[^']'\n",LevelPackCreator);
			fclose(Fp);
			sprintf(Tekst,"Blips was created by\nWillems Davy - Willems Soft 2008.\nHttp://www.willemssoft.be\n\nLevelpack %s was created\nby %s.",LevelPackName,LevelPackCreator);
		}
		else
			sprintf(Tekst,"Blips was created by\nWillems Davy - Willems Soft 2008.\nHttp://www.willemssoft.be\n\nLevelpack %s was created\nby unknown person.",LevelPackName);
	}
	else
		sprintf(Tekst,"Blips was created by\nWillems Davy - Willems Soft 2008\nHttp://www.willemssoft.be");
	while (GameState == GSCredits)
	{
	    if(GlobalSoundEnabled)
		if(!Mix_PlayingMusic())
        {
            SelectedMusic = 0;
            Mix_PlayMusic(Music[SelectedMusic],0);
            SetVolume(Volume);
        }
		SDL_BlitSurface(IMGTitleScreen,NULL,Tmp,NULL);

		Input->Update();
        if(Input->SpecialsHeld[SPECIAL_QUIT_EV])
            GameState = GSQuit;

        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_VOLUP] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_VOLUP)] || Input->KeyboardHeld[SDLK_KP_PLUS]))
        {
            IncVolume();
            Input->Delay();
        }

        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_VOLDOWN] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_VOLMIN)] || Input->KeyboardHeld[SDLK_KP_MINUS]))
        {
            DecVolume();
            Input->Delay();
        }

		if(Input->JoystickHeld[0][GP2X_BUTTON_SELECT] || Input->JoystickHeld[0][GP2X_BUTTON_START] || Input->JoystickHeld[0][GP2X_BUTTON_A]  ||
           Input->JoystickHeld[0][GP2X_BUTTON_B] || Input->JoystickHeld[0][GP2X_BUTTON_X] || Input->JoystickHeld[0][GP2X_BUTTON_Y] ||
           Input->KeyboardHeld[SDLK_SPACE] || Input->KeyboardHeld[SDLK_RETURN] || Input->KeyboardHeld[SDLK_ESCAPE] || Input->KeyboardHeld[SDLK_a] || Input->KeyboardHeld[SDLK_q] ||
           Input->KeyboardHeld[SDLK_x] || Input->KeyboardHeld[SDLK_y] || Input->KeyboardHeld[SDLK_b] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_A)] ||
           Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_X)] ||  Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_Y)]  || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_B)] )
        {
            GameState=GSTitleScreen;
        }

		boxRGBA(Tmp,60*SCREENPOSMULTIPLY,80*SCREENPOSMULTIPLY,260*SCREENPOSMULTIPLY,160*SCREENPOSMULTIPLY,MenuBoxColor.r,MenuBoxColor.g,MenuBoxColor.b,MenuBoxColor.unused);
		rectangleRGBA(Tmp,60*SCREENPOSMULTIPLY,80*SCREENPOSMULTIPLY,260*SCREENPOSMULTIPLY,160*SCREENPOSMULTIPLY,MenuBoxBorderColor.r,MenuBoxBorderColor.g,MenuBoxBorderColor.b,MenuBoxBorderColor.unused);
		rectangleRGBA(Tmp,61*SCREENPOSMULTIPLY,81*SCREENPOSMULTIPLY,259*SCREENPOSMULTIPLY,159*SCREENPOSMULTIPLY,MenuBoxBorderColor.r,MenuBoxBorderColor.g,MenuBoxBorderColor.b,MenuBoxBorderColor.unused);
		WriteText(Tmp,font,Tekst,strlen(Tekst),65*SCREENPOSMULTIPLY,82*SCREENPOSMULTIPLY,1*SCREENPOSMULTIPLY,MenuTextColor,false);
        if (alpha < 255)
        {
            if(alpha+AlphaInc > MaxAlpha)
            {
                alpha = 255;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
            else
            {
                alpha+=AlphaInc;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
        }
        SDL_BlitSurface(Tmp,NULL,Buffer,NULL);
        SDL_FillRect(Buffer1,NULL,SDL_MapRGB(Buffer1->format,0,0,0));
        SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
        SDL_BlitSurface(Buffer1,NULL,Screen,NULL);
        SDL_Flip(Screen);
        SDL_framerateDelay(&Fpsman);
	}
	delete[] Tekst;
	delete[] LevelPackCreator;
	SDL_FreeSurface(Tmp);
	delete Input;
}


void TitleScreen()
{
	int alpha = 0;
    SDL_Surface *Tmp;
	int Teller, Selection = 1;
	CInput *Input = new CInput(InputDelay);
	bool RPressed = false;
	char Command[FILENAME_MAX];
	SDL_Rect Rect;
	char *Tekst = new char[300];
	Tmp = SDL_DisplayFormat(Buffer);
	while (GameState == GSTitleScreen)
	{
        if(MusicCount > 0)
            if(GlobalSoundEnabled)
                if(!Mix_PlayingMusic())
                {
                    SelectedMusic = 0;
                    Mix_PlayMusic(Music[SelectedMusic],0);
                    SetVolume(Volume);
                }
		SDL_BlitSurface(IMGTitleScreen,NULL,Tmp,NULL);
		Input->Update();

		if(Input->JoystickHeld[0][GP2X_BUTTON_SELECT] || Input->KeyboardHeld[SDLK_ESCAPE] || Input->SpecialsHeld[SPECIAL_QUIT_EV] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_SELECT)])
            GameState = GSQuit;

		if(Input->JoystickHeld[0][GP2X_BUTTON_R] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_R)] || Input->KeyboardHeld[SDLK_r])
        {
            if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_LEFT] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_LEFT)] || Input->KeyboardHeld[SDLK_LEFT]))
            {
                if(StartScreenX - 2 >= 0)
                    StartScreenX -=2;
                Input->Delay();
            }

            if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_RIGHT] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_RIGHT)] || Input->KeyboardHeld[SDLK_RIGHT]))
            {
                if(StartScreenX + 2 + Buffer->w <= Screen->w)
                    StartScreenX +=2;
                Input->Delay();
            }

            if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_UP] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_UP)] || Input->KeyboardHeld[SDLK_UP]))
            {
                if(StartScreenY - 2 >=0)
                    StartScreenY -=2;
                Input->Delay();
            }

            if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_DOWN] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_DOWN)] || Input->KeyboardHeld[SDLK_DOWN]))
            {
                if(StartScreenY + 2 + Buffer->h <= Screen->h)
                    StartScreenY +=2;
                Input->Delay();
            }
        }

        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_L] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_L)] || Input->KeyboardHeld[SDLK_PAGEDOWN] || Input->KeyboardHeld[SDLK_l]))
        {
            PreviousSkin();
            Input->Delay();
        }

        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_VOLUP] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_VOLUP)] || Input->KeyboardHeld[SDLK_KP_PLUS]))
        {
            IncVolume();
            Input->Delay();
        }

        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_VOLDOWN] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_VOLMIN)] || Input->KeyboardHeld[SDLK_KP_MINUS]))
        {
            DecVolume();
            Input->Delay();
        }

        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_LEFT] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_LEFT)] || Input->KeyboardHeld[SDLK_LEFT]))
        {
            if(Selection==3)
                if (InstalledLevelPacksCount > 0)
                    if(SelectedLevelPack > 0)
                    {
                        SelectedLevelPack--;
                        sprintf(LevelPackName,"%s",InstalledLevelPacks[SelectedLevelPack]);
                        sprintf(LevelPackFileName,"%s",InstalledLevelPacks[SelectedLevelPack]);
                        AddUnderScores(LevelPackFileName);
                    }
            Input->Delay();
        }

        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_RIGHT] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_RIGHT)] || Input->KeyboardHeld[SDLK_RIGHT]))
        {
            if (Selection==3)
                if (InstalledLevelPacksCount > 0)
                    if(SelectedLevelPack < InstalledLevelPacksCount-1)
                    {
                        SelectedLevelPack++;
                        sprintf(LevelPackName,"%s",InstalledLevelPacks[SelectedLevelPack]);
                        sprintf(LevelPackFileName,"%s",InstalledLevelPacks[SelectedLevelPack]);
                        AddUnderScores(LevelPackFileName);
                    }
            Input->Delay();
        }

        if((Input->Ready()) && (Input->JoystickHeld[0][GP2X_BUTTON_DOWN] || Input->KeyboardHeld[SDLK_DOWN] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_DOWN)]) &&
           !(Input->JoystickHeld[0][GP2X_BUTTON_R] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_R)] || Input->KeyboardHeld[SDLK_r]))
        {
            if (Selection < 6)
            {
                Selection++;
                if (GlobalSoundEnabled)
                    Mix_PlayChannel(-1,Sounds[SND_MENU],0);
            }

            Input->Delay();
        }

        if((Input->Ready()) && (Input->JoystickHeld[0][GP2X_BUTTON_UP] || Input->KeyboardHeld[SDLK_UP] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_UP)]) &&
           !(Input->JoystickHeld[0][GP2X_BUTTON_R] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_R)] || Input->KeyboardHeld[SDLK_r]))
        {
            if (Selection > 1)
            {
                Selection--;
                if (GlobalSoundEnabled)
                    Mix_PlayChannel(-1,Sounds[SND_MENU],0);
            }
            Input->Delay();
        }

        if(Input->JoystickHeld[0][GP2X_BUTTON_A] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_A)] || Input->KeyboardHeld[SDLK_SPACE] || Input->KeyboardHeld[SDLK_RETURN])
        {
            switch(Selection)
            {
                case 1:
                    if (InstalledLevelPacksCount >0)
                    {
                        FindLevels();
                        if(InstalledLevels > 0)
                        {
                            LoadUnlockData();
                            SelectedLevel=UnlockedLevels;
                            LevelEditorMode=false;
                            GameState=GSStageSelect;
                            if (GlobalSoundEnabled)
                                Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
                        }
                        else
                        {
                            if (GlobalSoundEnabled)
                                Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
                            SDL_BlitSurface(IMGTitleScreen,NULL,Buffer,NULL);
                            sprintf(Tekst,"There are no levels found in levelpack\n%s\n\nPlease create a level for this level pack\nfirst!",LevelPackName);
                            PrintForm(Tekst);
                            Input->Reset();
                        }
                    }
                    break;
                case 2:
                    GameState=GSLevelEditorMenu;
                    LevelEditorMode=true;
                    if (GlobalSoundEnabled)
                        Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
                    break;
                case 4:
                    GameState=GSCredits;
                    if (GlobalSoundEnabled)
                        Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
                    break;
                case 5:
                    GameState = GSSetupUsbJoystickButtons;
                    if (GlobalSoundEnabled)
                        Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
                    break;
                case 6:
                    GameState = GSQuit;
                    if (GlobalSoundEnabled)
                        Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
                    break;

            }

        }
   		boxRGBA(Tmp,60*SCREENPOSMULTIPLY,70*SCREENPOSMULTIPLY,260*SCREENPOSMULTIPLY,165*SCREENPOSMULTIPLY,MenuBoxColor.r,MenuBoxColor.g,MenuBoxColor.b,MenuBoxColor.unused);
		rectangleRGBA(Tmp,60*SCREENPOSMULTIPLY,70*SCREENPOSMULTIPLY,260*SCREENPOSMULTIPLY,165*SCREENPOSMULTIPLY,MenuBoxBorderColor.r,MenuBoxBorderColor.g,MenuBoxBorderColor.b,MenuBoxBorderColor.unused);
		rectangleRGBA(Tmp,61*SCREENPOSMULTIPLY,71*SCREENPOSMULTIPLY,259*SCREENPOSMULTIPLY,164*SCREENPOSMULTIPLY,MenuBoxBorderColor.r,MenuBoxBorderColor.g,MenuBoxBorderColor.b,MenuBoxBorderColor.unused);
		sprintf(Tekst,"Play Selected LevelPack\nLevel Editor\n<%s>\nCredits\nSetup External Joystick\nQuit",LevelPackName);
		WriteText(Tmp,BigFont,Tekst,strlen(Tekst),90*SCREENPOSMULTIPLY,70*SCREENPOSMULTIPLY,2*SCREENPOSMULTIPLY,MenuTextColor,false);
		if (Selection > 1)
		{
			sprintf(Tekst,"\n");
			for(Teller=2;Teller<Selection;Teller++)
				sprintf(Tekst,"%s%s",Tekst,"\n");
			sprintf(Tekst,"%s%s",Tekst,">>");
		}
		else
			sprintf(Tekst,">>");
		WriteText(Tmp,BigFont,Tekst,strlen(Tekst),65*SCREENPOSMULTIPLY,70*SCREENPOSMULTIPLY,2*SCREENPOSMULTIPLY,MenuTextColor,false);
		Rect.w = Buffer->w;
        Rect.h = Buffer->h;
        Rect.x = StartScreenX;
        Rect.y = StartScreenY;
		if (alpha < 255)
        {
            if(alpha+AlphaInc > MaxAlpha)
            {
                alpha = 255;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
            else
            {
                alpha+=AlphaInc;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
        }
        SDL_BlitSurface(Tmp,NULL,Buffer,NULL);
        SDL_FillRect(Buffer1,NULL,SDL_MapRGB(Buffer1->format,0,0,0));
        SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
        SDL_BlitSurface(Buffer1,NULL,Screen,NULL);
        SDL_Flip(Screen);
        SDL_framerateDelay(&Fpsman);
	}
	delete[] Tekst;
	SDL_FreeSurface(Tmp);
	delete Input;
}

void SetupUsbJoystickButtons()
{
    int alpha = 0;
    SDL_Surface *Tmp,*Tmp1;
	int Teller, Selection = 0;
	SDL_Rect Rect;
	char *Tekst = new char[300];
	Tmp1 = SDL_CreateRGBSurface(SDL_SWSURFACE,WINDOW_WIDTH,WINDOW_HEIGHT,16,Screen->format->Rmask,Screen->format->Gmask,Screen->format->Bmask,Screen->format->Amask);
	Tmp = SDL_DisplayFormat(Tmp1);
	SDL_FreeSurface(Tmp1);
	bool done = false;
	CInput *Input = new CInput(10);
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
	while (GameState == GSSetupUsbJoystickButtons)
	{
        if(MusicCount > 0)
            if(GlobalSoundEnabled)
                if(!Mix_PlayingMusic())
                {
                    SelectedMusic = 0;
                    Mix_PlayMusic(Music[SelectedMusic],0);
                    SetVolume(Volume);
                }

		Input->Update();

		if(Input->SpecialsHeld[SPECIAL_QUIT_EV])
            GameState = GSQuit;

		if(Input->JoystickHeld[0][GP2X_BUTTON_SELECT] || Input->KeyboardHeld[SDLK_ESCAPE] ||  Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_SELECT)])
            GameState = GSTitleScreen;

        if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_UP] || Input->KeyboardHeld[SDLK_UP] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_UP)]))
        {
            Selection--;
            if( Selection < 0)
                Selection = NROFBUTTONS-1;
            if (GlobalSoundEnabled)
                    Mix_PlayChannel(-1,Sounds[SND_MENU],0);
            Input->Delay();
        }

        if (Input->JoystickHeld[0][GP2X_BUTTON_START] || Input->KeyboardHeld[SDLK_RETURN])
            JoystickSetup->ResetToDefaults();

        if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_DOWN] || Input->KeyboardHeld[SDLK_DOWN] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_DOWN)]))
        {
            Selection++;
            if( Selection >=NROFBUTTONS)
                Selection = 0;
            if (GlobalSoundEnabled)
                    Mix_PlayChannel(-1,Sounds[SND_MENU],0);
            Input->Delay();
        }

        SDL_BlitSurface(IMGTitleScreen,NULL,Tmp,NULL);
        boxRGBA(Tmp,45*SCREENPOSMULTIPLY,50*SCREENPOSMULTIPLY,295*SCREENPOSMULTIPLY,173*SCREENPOSMULTIPLY,MenuBoxColor.r,MenuBoxColor.g,MenuBoxColor.b,MenuBoxColor.unused);
		rectangleRGBA(Tmp,45*SCREENPOSMULTIPLY,50*SCREENPOSMULTIPLY,295*SCREENPOSMULTIPLY,173*SCREENPOSMULTIPLY,MenuBoxBorderColor.r,MenuBoxBorderColor.g,MenuBoxBorderColor.b,MenuBoxBorderColor.unused);
		rectangleRGBA(Tmp,46*SCREENPOSMULTIPLY,51*SCREENPOSMULTIPLY,294*SCREENPOSMULTIPLY,172*SCREENPOSMULTIPLY,MenuBoxBorderColor.r,MenuBoxBorderColor.g,MenuBoxBorderColor.b,MenuBoxBorderColor.unused);
        JoystickSetup->DrawCurrentSetup(Tmp,font,55*SCREENPOSMULTIPLY,53*SCREENPOSMULTIPLY,155*SCREENPOSMULTIPLY,8*SCREENPOSMULTIPLY,Selection,MenuTextColor,MenuTextColor);
        if (alpha < 255)
        {
            if(alpha+AlphaInc > MaxAlpha)
            {
                alpha = 255;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
            else
            {
                alpha+=AlphaInc;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
        }
        SDL_BlitSurface(Tmp,NULL,Buffer,NULL);
        SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
        SDL_BlitSurface(Buffer,NULL,Screen,&Rect);
        SDL_Flip(Screen);

        if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_A] || Input->KeyboardHeld[SDLK_a] || Input->KeyboardHeld[SDLK_SPACE] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_A)]))
        {
            if (alpha == 255)
            {
                if (GlobalSoundEnabled)
                    Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
                Input->Reset();
                while (!done)
                {
                    Input->Update();
                    SDL_BlitSurface(IMGTitleScreen,NULL,Tmp,NULL);
                    boxRGBA(Tmp,45*SCREENPOSMULTIPLY,50*SCREENPOSMULTIPLY,295*SCREENPOSMULTIPLY,173*SCREENPOSMULTIPLY,MenuBoxColor.r,MenuBoxColor.g,MenuBoxColor.b,MenuBoxColor.unused);
                    rectangleRGBA(Tmp,45*SCREENPOSMULTIPLY,50*SCREENPOSMULTIPLY,295*SCREENPOSMULTIPLY,173*SCREENPOSMULTIPLY,MenuBoxBorderColor.r,MenuBoxBorderColor.g,MenuBoxBorderColor.b,MenuBoxBorderColor.unused);
                    rectangleRGBA(Tmp,46*SCREENPOSMULTIPLY,51*SCREENPOSMULTIPLY,294*SCREENPOSMULTIPLY,172*SCREENPOSMULTIPLY,MenuBoxBorderColor.r,MenuBoxBorderColor.g,MenuBoxBorderColor.b,MenuBoxBorderColor.unused);


                    JoystickSetup->DrawCurrentSetup(Tmp,font,55*SCREENPOSMULTIPLY,53*SCREENPOSMULTIPLY,155*SCREENPOSMULTIPLY,8*SCREENPOSMULTIPLY,Selection,MenuTextColor,MenuBoxBorderColor);
                    SDL_BlitSurface(Tmp,NULL,Buffer,NULL);
                    SDL_FillRect(Screen,NULL,SDL_MapRGB(Screen->format,0,0,0));
                    SDL_BlitSurface(Buffer,NULL,Screen,&Rect);
                    SDL_Flip(Screen);
                    for (Teller = 0;Teller<MAXJOYSTICKBUTTONS;Teller++)
                        if(Input->JoystickHeld[1][Teller])
                        {
                            done = true;
                            JoystickSetup->SetButtonValue(Selection,Teller);
                            break;
                        }
                    if(Input->SpecialsHeld[SPECIAL_QUIT_EV])
                    {
                        GameState = GSQuit;
                        done = true;
                    }

                    if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_A] || Input->KeyboardHeld[SDLK_a] || Input->KeyboardHeld[SDLK_SPACE]))
                        done = true;

                    if(Input->JoystickHeld[0][GP2X_BUTTON_SELECT] || Input->KeyboardHeld[SDLK_ESCAPE] || Input->SpecialsHeld[SPECIAL_QUIT_EV] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_SELECT)])
                        done= true;
                    SDL_framerateDelay(&Fpsman);
                }
                Input->Reset();
                Input->Delay();
                done= false;
            }
        }
        SDL_framerateDelay(&Fpsman);

	}
	delete[] Tekst;
	delete Input;
	SDL_FreeSurface(Tmp);
}


void LevelEditorMenu()
{
	int alpha = 0;
    SDL_Surface *Tmp;
	FILE *ColorsFile,*Fp;
	int Teller, Selection = 1;
	char *PackName,*CreatorName;
	char Command[FILENAME_MAX];
	char FileName[FILENAME_MAX];
	SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
    Tmp = SDL_DisplayFormat(Buffer);
	CInput *Input = new CInput(InputDelay);
	char *Tekst = new char[300];
	while (GameState == GSLevelEditorMenu)
	{
        if(GlobalSoundEnabled)
            if (! Mix_PlayingMusic())
            {
                SelectedMusic = 0;
                Mix_PlayMusic(Music[SelectedMusic],0);
				//Mix_HookMusicFinished(MusicFinished);
                SetVolume(Volume);
            }
		SDL_BlitSurface(IMGTitleScreen,NULL,Tmp,NULL);
		Input->Update();

		if(Input->JoystickHeld[0][GP2X_BUTTON_SELECT] || Input->KeyboardHeld[SDLK_ESCAPE] || Input->SpecialsHeld[SPECIAL_QUIT_EV] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_SELECT)])
            GameState = GSTitleScreen;

		if(Input->JoystickHeld[0][GP2X_BUTTON_R] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_R)] || Input->KeyboardHeld[SDLK_r])
        {
            if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_LEFT] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_LEFT)] || Input->KeyboardHeld[SDLK_LEFT]))
            {
                if(StartScreenX - 2 >= 0)
                    StartScreenX -=2;
                Input->Delay();
            }

            if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_RIGHT] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_RIGHT)] || Input->KeyboardHeld[SDLK_RIGHT]))
            {
                if(StartScreenX + 2 + Buffer->w <= Screen->w)
                    StartScreenX +=2;
                Input->Delay();
            }

            if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_UP] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_UP)] || Input->KeyboardHeld[SDLK_UP]))
            {
                if(StartScreenY - 2 >=0)
                    StartScreenY -=2;
                Input->Delay();
            }

            if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_DOWN] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_DOWN)] || Input->KeyboardHeld[SDLK_DOWN]))
            {
                if(StartScreenY + 2 + Buffer->h <= Screen->h)
                    StartScreenY +=2;
                Input->Delay();
            }
        }

        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_L] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_L)] || Input->KeyboardHeld[SDLK_PAGEDOWN] || Input->KeyboardHeld[SDLK_l]))
        {
            PreviousSkin();
            Input->Delay();
        }

        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_VOLUP] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_VOLUP)] || Input->KeyboardHeld[SDLK_KP_PLUS]))
        {
            IncVolume();
            Input->Delay();
        }

        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_VOLDOWN] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_VOLMIN)] || Input->KeyboardHeld[SDLK_KP_MINUS]))
        {
            DecVolume();
            Input->Delay();
        }

        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_LEFT] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_LEFT)] || Input->KeyboardHeld[SDLK_LEFT]))
        {
            if(Selection==4)
                if (InstalledLevelPacksCount > 0)
                    if(SelectedLevelPack > 0)
                    {
                        SelectedLevelPack--;
                        sprintf(LevelPackName,"%s",InstalledLevelPacks[SelectedLevelPack]);
                        sprintf(LevelPackFileName,"%s",InstalledLevelPacks[SelectedLevelPack]);
                        AddUnderScores(LevelPackFileName);
                    }
            Input->Delay();
        }

        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_RIGHT] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_RIGHT)] || Input->KeyboardHeld[SDLK_RIGHT]))
        {
            if (Selection==4)
                if (InstalledLevelPacksCount > 0)
                    if(SelectedLevelPack < InstalledLevelPacksCount-1)
                    {
                        SelectedLevelPack++;
                        sprintf(LevelPackName,"%s",InstalledLevelPacks[SelectedLevelPack]);
                        sprintf(LevelPackFileName,"%s",InstalledLevelPacks[SelectedLevelPack]);
                        AddUnderScores(LevelPackFileName);
                    }
            Input->Delay();
        }

        if((Input->Ready()) && (Input->JoystickHeld[0][GP2X_BUTTON_DOWN] || Input->KeyboardHeld[SDLK_DOWN] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_DOWN)]) &&
           !(Input->JoystickHeld[0][GP2X_BUTTON_R] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_R)] || Input->KeyboardHeld[SDLK_r]))
        {
            if (Selection < 5)
            {
                Selection++;
                if (GlobalSoundEnabled)
                    Mix_PlayChannel(-1,Sounds[SND_MENU],0);
            }

            Input->Delay();
        }

        if((Input->Ready()) && (Input->JoystickHeld[0][GP2X_BUTTON_UP] || Input->KeyboardHeld[SDLK_UP] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_UP)]) &&
           !(Input->JoystickHeld[0][GP2X_BUTTON_R] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_R)] || Input->KeyboardHeld[SDLK_r]))
        {
            if (Selection > 1)
            {
                Selection--;
                if (GlobalSoundEnabled)
                    Mix_PlayChannel(-1,Sounds[SND_MENU],0);
            }
            Input->Delay();
        }

        if(Input->SpecialsHeld[SPECIAL_QUIT_EV])
            GameState = GSQuit;

        if(Input->JoystickHeld[0][GP2X_BUTTON_A] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_A)] || Input->KeyboardHeld[SDLK_SPACE] || Input->KeyboardHeld[SDLK_RETURN])
        {
            switch(Selection)
            {
                case 1:
                    if (GlobalSoundEnabled)
                        Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
                    PackName = GetString("","Enter the Levelpack name:");
                    Input->Reset();
                    if (strlen(PackName) > 0)
                    {
                        SDL_BlitSurface(IMGTitleScreen,NULL,Buffer,NULL);
                        CreatorName = GetString("","Enter the Levelpack Creator name:");
                        Input->Reset();
                        if(strlen(CreatorName)>0)
                        {
                            sprintf(LevelPackName,"%s",PackName);
                            sprintf(LevelPackFileName,"%s",PackName);
                            AddUnderScores(LevelPackFileName);
                            sprintf(Tekst,"./levelpacks/%s",LevelPackFileName);
                            //system(Command);
#ifdef WIN32
                            mkdir(Tekst);
#else
                            mkdir(Tekst,S_IRWXO|S_IRWXU|S_IRWXG);
#endif
                            //sprintf(Command,"chdmod 777 ./levelpacks/%s",LevelPackFileName);
                            //system(Command);
                            sprintf(FileName,"./levelpacks/%s/credits.dat",LevelPackFileName);
                            Fp = fopen(FileName,"wt");
                            if (Fp)
                            {
                                fprintf(Fp,"[Credits]\nCreator='%s'\n",CreatorName);
                                fclose(Fp);
                            }
#ifdef GP2X
                            sync();
#endif
                            SearchForLevelPacks();
                            for (Teller=0;Teller<InstalledLevelPacksCount;Teller++)
                                if(strcmp(PackName,InstalledLevelPacks[Teller]) == 0)
                                {
                                    SelectedLevelPack = Teller;
                                    sprintf(LevelPackName,"%s",InstalledLevelPacks[SelectedLevelPack]);
                                    sprintf(LevelPackFileName,"%s",InstalledLevelPacks[SelectedLevelPack]);
                                    AddUnderScores(LevelPackFileName);
                                }
                        }
                        delete[] CreatorName;
                    }
                    delete[] PackName;
                    break;
                case 2:
                    if (InstalledLevelPacksCount >0)
                    {
                        FindLevels();
                        SelectedLevel=0;
                        GameState=GSStageSelect;
                        if (GlobalSoundEnabled)
                            Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
                    }
                    break;
                case 3:
                    if (InstalledLevelPacksCount >0)
                    {
                        if (GlobalSoundEnabled)
                            Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
                        sprintf(Tekst,"do you want to delete the selected level pack:\n\"%s\"\nAll Levels in Levelpack will be deleted !!!\n\nPress A to Delete, X to Cancel",InstalledLevelPacks[SelectedLevelPack]);
                        if(AskQuestion(Tekst))
                        {
                            for(Teller=1;Teller<INT_MAX;Teller++)
                            {
                                sprintf(Tekst,"./levelpacks/%s/level%d.lev",LevelPackFileName,Teller);
                                if(FileExists(Tekst))
                                {
                                    remove(Tekst);
                                }
                                else
                                    break;
                            }
                            sprintf(Tekst,"./levelpacks/%s/credits.dat",LevelPackFileName);
                            if(FileExists(Tekst))
                            {
                                remove(Tekst);
                            }
                            sprintf(Tekst,"./levelpacks/%s",LevelPackFileName);
                            rmdir(Tekst);

                            SearchForLevelPacks();
                        }
                        Input->Reset();

                    }
                    break;
                case 5:
                    if (GlobalSoundEnabled)
                        Mix_PlayChannel(-1,Sounds[SND_SELECT],0);
                    GameState = GSTitleScreen;
                    break;
            }
        }
		boxRGBA(Tmp,60*SCREENPOSMULTIPLY,80*SCREENPOSMULTIPLY,260*SCREENPOSMULTIPLY,160*SCREENPOSMULTIPLY,MenuBoxColor.r,MenuBoxColor.g,MenuBoxColor.b,MenuBoxColor.unused);
		rectangleRGBA(Tmp,60*SCREENPOSMULTIPLY,80*SCREENPOSMULTIPLY,260*SCREENPOSMULTIPLY,160*SCREENPOSMULTIPLY,MenuBoxBorderColor.r,MenuBoxBorderColor.g,MenuBoxBorderColor.b,MenuBoxBorderColor.unused);
		rectangleRGBA(Tmp,61*SCREENPOSMULTIPLY,81*SCREENPOSMULTIPLY,259*SCREENPOSMULTIPLY,159*SCREENPOSMULTIPLY,MenuBoxBorderColor.r,MenuBoxBorderColor.g,MenuBoxBorderColor.b,MenuBoxBorderColor.unused);
		sprintf(Tekst,"Create New Levelpack\nLoad Selected LevelPack\nDelete Selected Levelpack\n<%s>\nMain Menu",LevelPackName);
		WriteText(Tmp,BigFont,Tekst,strlen(Tekst),90*SCREENPOSMULTIPLY,77*SCREENPOSMULTIPLY,2*SCREENPOSMULTIPLY,MenuTextColor,false);
		if (Selection > 1)
		{
			sprintf(Tekst,"\n");
			for(Teller=2;Teller<Selection;Teller++)
				sprintf(Tekst,"%s%s",Tekst,"\n");
			sprintf(Tekst,"%s%s",Tekst,">>");
		}
		else
			sprintf(Tekst,">>");
		WriteText(Tmp,BigFont,Tekst,strlen(Tekst),65*SCREENPOSMULTIPLY,77*SCREENPOSMULTIPLY,2*SCREENPOSMULTIPLY,MenuTextColor,false);
		if (alpha < 255)
        {
            if(alpha+AlphaInc > MaxAlpha)
            {
                alpha = 255;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
            else
            {
                alpha+=AlphaInc;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
        }
        SDL_BlitSurface(Tmp,NULL,Buffer,NULL);
        SDL_FillRect(Buffer1,NULL,SDL_MapRGB(Buffer1->format,0,0,0));
        SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
        SDL_BlitSurface(Buffer1,NULL,Screen,NULL);
        SDL_Flip(Screen);
        SDL_framerateDelay(&Fpsman);
	}
	delete[] Tekst;
	SDL_FreeSurface(Tmp);
	delete Input;
}

bool LevelErrorsFound()
{
	int Teller,NumPlayer=0,NumDiamond=0;
	for (Teller=0;Teller<WorldParts.ItemCount;Teller++)
	{
		if(WorldParts.Items[Teller]->GetType() == IDPlayer)
			NumPlayer++;
        if(WorldParts.Items[Teller]->GetType() == IDDiamond)
			NumDiamond++;
	}

	if (NumPlayer==0)
	{
		if (GlobalSoundEnabled)
			Mix_PlayChannel(-1,Sounds[SND_ERROR],0);
		PrintForm("Can not save this level because there's\nno player in the level! Please add a Player\nand try again or cancel saving.\n\nPress 'A' to continue");
		return true;
	}
	else
        if (NumDiamond==0)
        {
            if (GlobalSoundEnabled)
                Mix_PlayChannel(-1,Sounds[SND_ERROR],0);
            PrintForm("Can not save this level because there's\nno star in the level! Please add a star\nand try again or cancel saving.\n\nPress 'A' to continue");
            return true;
        }

	return false;

}

void Game()
{

	SDL_Event Event;
	SDL_Surface *Tmp;
	bool BoxMoving = false,CarryingBox=false,ExplosionsFound =false;
	char *LevelPackCreator = new char[21];
	char FileName[FILENAME_MAX];
	FILE *Fp;
	char Text[500];
	int alpha=0,teller,teller2=0;
	Uint32 Time=0,Time2=0;
	CPlayer *Player=0;
	bool ResetViewPort=false;
	for (teller=0;teller<WorldParts.ItemCount;teller++)
	{
		if (WorldParts.Items[teller]->GetType() == IDPlayer)
		{
			Player = (CPlayer*) WorldParts.Items[teller];
			break;
		}
	}
	//should never happen
	if(!Player)
	{
		Player = new CPlayer(0,0);
		WorldParts.Add(Player);
	}
	SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
	CInput *Input = new CInput(InputDelay);
	Tmp = SDL_DisplayFormat(Buffer);
	SDL_Color Color={255,0,255,255};
	int framecounter=0,fps=0;
	char *Tekst = new char[500];
	//Time2 = SDL_GetTicks();
	Time = SDL_GetTicks();
	if (MusicCount > 1)
 	{

		if(GlobalSoundEnabled)
		{
		    SelectedMusic =	1+rand()%(MusicCount-1);
			Mix_PlayMusic(Music[SelectedMusic],0);
			SetVolume(Volume);
		}
	}
    WorldParts.LimitVPLevel();
    while (GameState == GSGame)
    {
        if(!Mix_PlayingMusic())
        if(GlobalSoundEnabled)
        {
            if (MusicCount > 1)
            {
                //printf("game():2\n");
                SelectedMusic =	1+rand()%(MusicCount-1);
                Mix_PlayMusic(Music[SelectedMusic],0);
                SetVolume(Volume);
                //printf("game():3\n");
            }
            else
                if(MusicCount ==1)
                {
                    //printf("game():4\n");
                    SelectedMusic =	0;
                    Mix_PlayMusic(Music[SelectedMusic],0);
                    SetVolume(Volume);
                    //printf("game():5\n");
                }

        }

        Input->Update();
        //printf("game():6\n");
        if(Input->SpecialsHeld[SPECIAL_QUIT_EV])
            GameState = GSQuit;
        //printf("game():7\n");
        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_VOLUP] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_VOLUP)] || Input->KeyboardHeld[SDLK_KP_PLUS]))
        {
            IncVolume();
            Input->Delay();
        }
        //printf("game():8\n");
        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_VOLDOWN] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_VOLMIN)] || Input->KeyboardHeld[SDLK_KP_MINUS]))
        {
            DecVolume();
            Input->Delay();
        }
        //printf("game():9\n");
        if(Input->JoystickHeld[0][GP2X_BUTTON_SELECT] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_SELECT)] || Input->KeyboardHeld[SDLK_ESCAPE])
        {
            if(!LevelEditorMode)
                GameState = GSStageSelect;
        }
        //printf("game():10\n");
        if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_START] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_START)] || Input->KeyboardHeld[SDLK_RETURN]))
        {
            if(LevelEditorMode)
                GameState=GSLevelEditor;
            else
            {
                sprintf(FileName,"./levelpacks/%s/level%d.lev",LevelPackFileName,SelectedLevel);
                WorldParts.Load(FileName);
            }
            Input->Delay();
        }
        //printf("game():11\n");
        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_L] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_L)] || Input->KeyboardHeld[SDLK_PAGEDOWN] || Input->KeyboardHeld[SDLK_l]))
        {
            NextSkin();
            Input->Delay();
        }
        //printf("game():12\n");
        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_R] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_R)] || Input->KeyboardHeld[SDLK_PAGEUP] || Input->KeyboardHeld[SDLK_r]))
        {
            if (MusicCount > 1)
            {
                SelectedMusic++;
                if (SelectedMusic >= MusicCount)
                    SelectedMusic = 1;
                if(GlobalSoundEnabled)
                {
                    Mix_HaltMusic();
                    Mix_PlayMusic(Music[SelectedMusic],0);
                    //Mix_HookMusicFinished(MusicFinished);
                    SetVolume(Volume);
                }
            }
            Input->Delay();
        }
        //printf("game():13\n");
        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_LEFT] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_LEFT)] || Input->KeyboardHeld[SDLK_LEFT]))
        {
            Input->Delay();
        }
        //printf("game():14\n");
        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_RIGHT] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_RIGHT)] || Input->KeyboardHeld[SDLK_RIGHT]))
        {

            Input->Delay();
        }

        if (Input->JoystickHeld[0][JoystickSetup->GetButtonValue(GP2X_BUTTON_DOWN)] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_DOWN)]|| Input->KeyboardHeld[SDLK_DOWN])
        {
            Input->Delay();
        }

        //move up
        if (Input->JoystickHeld[0][JoystickSetup->GetButtonValue(GP2X_BUTTON_UP)] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_UP)]|| Input->KeyboardHeld[SDLK_UP])
        {
            Input->Delay();
        }

        //printf("game():15\n");
        if(Time+10<SDL_GetTicks())
        {
            //printf("game():16\n");
            if (Input->JoystickHeld[0][GP2X_BUTTON_Y]  || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_Y)] || Input->KeyboardHeld[SDLK_y])
            {
                if(Input->JoystickHeld[0][GP2X_BUTTON_LEFT] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_LEFT)] || Input->KeyboardHeld[SDLK_LEFT])
                    WorldParts.ViewPort->Move(-2*SCREENPOSMULTIPLY,0);
                if(Input->JoystickHeld[0][GP2X_BUTTON_RIGHT] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_RIGHT)] || Input->KeyboardHeld[SDLK_RIGHT])
                    WorldParts.ViewPort->Move(2*SCREENPOSMULTIPLY,0);
                if(Input->JoystickHeld[0][GP2X_BUTTON_UP] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_UP)] || Input->KeyboardHeld[SDLK_UP])
                    WorldParts.ViewPort->Move(0,-2*SCREENPOSMULTIPLY);
                if( (Input->JoystickHeld[0][GP2X_BUTTON_DOWN] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_DOWN)] || Input->KeyboardHeld[SDLK_DOWN]))
                    WorldParts.ViewPort->Move(0,2*SCREENPOSMULTIPLY);
                ResetViewPort = true;
            }
            else
            {
                if (ResetViewPort)
                {
                    WorldParts.CenterVPOnPlayer();
                    ResetViewPort = false;
                }
            }
            //printf("game():17\n");
            if (!Player->IsMoving && !Player->IsDeath && !(Input->JoystickHeld[0][GP2X_BUTTON_Y]  || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_Y)] || Input->KeyboardHeld[SDLK_y]))
            {

                //move down
                if (Input->JoystickHeld[0][JoystickSetup->GetButtonValue(GP2X_BUTTON_DOWN)] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_DOWN)]|| Input->KeyboardHeld[SDLK_DOWN])
                {
                        Player->MoveTo(Player->GetPlayFieldX(),Player->GetPlayFieldY()+1,false);
                }

                //move up
                if (Input->JoystickHeld[0][JoystickSetup->GetButtonValue(GP2X_BUTTON_UP)] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_UP)]|| Input->KeyboardHeld[SDLK_UP])
                {
                        Player->MoveTo(Player->GetPlayFieldX(),Player->GetPlayFieldY()-1,false);
                }
                //move left
                if ( (Input->JoystickHeld[0][GP2X_BUTTON_LEFT] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_LEFT)] || Input->KeyboardHeld[SDLK_LEFT]))
                {
                        Player->MoveTo(Player->GetPlayFieldX()-1,Player->GetPlayFieldY(),false);
                }
                //move right
                if ((Input->JoystickHeld[0][GP2X_BUTTON_RIGHT] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_RIGHT)] || Input->KeyboardHeld[SDLK_RIGHT]))
                {
                        Player->MoveTo(Player->GetPlayFieldX()+1,Player->GetPlayFieldY(),false);
                }

            }
            //printf("game():20\n");
            Time = SDL_GetTicks();

        }



        SDL_BlitSurface(IMGBackground,NULL,Tmp,NULL);
        WorldParts.Draw(Tmp);
        WorldParts.Move();


        sprintf(Text,"Fps:%d",fps);
       // WriteText(Tmp,BigFont,Text,strlen(Text),0,0,0,Color,false);
        //tekenen naar buffer
        Rect.w = Buffer->w;
        Rect.h = Buffer->h;
        Rect.x = StartScreenX;
        Rect.y = StartScreenY;
        if (alpha < 255)
        {
            if(alpha+AlphaInc > MaxAlpha)
            {
                alpha = 255;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
            else
            {
                alpha+=AlphaInc;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
        }
        SDL_BlitSurface(Tmp,NULL,Buffer,NULL);
        SDL_FillRect(Buffer1,NULL,SDL_MapRGB(Buffer1->format,0,0,0));
        SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
        SDL_BlitSurface(Buffer1,NULL,Screen,NULL);
        SDL_Flip(Screen);
        SDL_framerateDelay(&Fpsman);
        framecounter++;
        if(Time2 + 1000 < SDL_GetTicks())
        {
            fps = framecounter;
            framecounter = 0;
            Time2 = SDL_GetTicks();
        }
        if (Player->IsDeath)
        {
            ExplosionsFound =false;
            for(teller=0;teller<WorldParts.ItemCount;teller++)
            {
                if(WorldParts.Items[teller]->GetType() == IDExplosion)
                {
                    ExplosionsFound = true;
                }
            }

            if(!ExplosionsFound)
            if (LevelEditorMode)
            {
                if (AskQuestion("Too bad you died !\nDo you want to return to the\nlevel editor ?\n\n(A) Leveleditor (X) Play Again"))
					GameState = GSLevelEditor;
				else
				{
					sprintf(FileName,"./temp.lev");
					WorldParts.Load(FileName);
					for (teller=0;teller<WorldParts.ItemCount;teller++)
					{
						if (WorldParts.Items[teller]->GetType() == IDPlayer)
						{
							Player = (CPlayer*)WorldParts.Items[teller];
							break;
						}

					}
				}
            }
            else
            {
                if (AskQuestion("Too bad you died !\nDo you want to try again?\n\n(A) Try Again (X) Level Selector"))
				{
                    sprintf(FileName,"./levelpacks/%s/level%d.lev",LevelPackFileName,SelectedLevel);
                    WorldParts.Load(FileName);
                    for (teller=0;teller<WorldParts.ItemCount;teller++)
                    {
                        if (WorldParts.Items[teller]->GetType() == IDPlayer)
                        {
                            Player = (CPlayer*) WorldParts.Items[teller];
                            break;
                        }
                    }
                            //should never happen
                    if(!Player)
                    {
                        Player = new CPlayer(0,0);
                        WorldParts.Add(Player);
                    }
                    Input->Reset();
                }
                else
					GameState = GSStageSelect;

            }
        }
        if ( StageDone())
		{
			SDL_Delay(250);
			if (GlobalSoundEnabled)
				Mix_PlayChannel(-1,Sounds[SND_STAGEEND],0);
			if (LevelEditorMode)
			{
				if (AskQuestion("Congratulations !\nYou Succesfully Solved this level\nDo you want to return to the\nlevel editor ?\n(A) Leveleditor (X) Play Again"))
					GameState = GSLevelEditor;
				else
				{
					sprintf(FileName,"./temp.lev");
					WorldParts.Load(FileName);
					for (teller=0;teller<WorldParts.ItemCount;teller++)
					{
						if (WorldParts.Items[teller]->GetType() == IDPlayer)
						{
							Player = (CPlayer*)WorldParts.Items[teller];
							break;
						}

					}
				}

			}
			else
			{
				if (SelectedLevel == UnlockedLevels)
				{
					if ( UnlockedLevels < InstalledLevels)
					{
						sprintf(Text,"Congratulations !\nYou Succesfully Solved Level %d/%d\nThe next level has now been unlocked!\n\nPress (A) to continue",SelectedLevel,InstalledLevels);
						PrintForm(Text);
						UnlockedLevels++;
						SelectedLevel++;
						SaveUnlockData();
						GameState = GSStageSelect;
					}
					else
					{
						sprintf(Text,"Congratulations !\nYou Succesfully Solved Level %d/%d\nlevelpack %s\nis now finished, try out another one!\n\nPress (A) to continue",SelectedLevel,InstalledLevels,LevelPackName);
						PrintForm(Text);
						GameState = GSTitleScreen;
					}
				}
				else
				{
					sprintf(Text,"Congratulations !\nYou Succesfully Solved Level %d/%d\n\nPress (A) to continue",SelectedLevel,InstalledLevels);
					PrintForm(Text);
					GameState = GSStageSelect;

				}
			}
		}
    }
    Mix_HaltMusic();
    SDL_FreeSurface(Tmp);
    delete Input;
}

void LevelEditor()
{
    SDL_Color Color={255,0,255,255};
	int Teller,Teller2,alpha=0;
	SDL_Surface *Tmp,*Tmp1;
	Uint32 Time2=0;
	bool ShowPosition=true,AnotherPartFound,SamePartFound,PlayerFound=false,ShowGrid=true;
	char Tekst[200];
	char *FileName = new char[FILENAME_MAX];
	CInput *Input = new CInput(InputDelay-5);
	CWorldPart *Part;
	CSelector Selector(&WorldParts);
	int MaxX=0,MaxY=0,MinY=NrOfRows,MinX=NrOfCols,Xi=0,Yi=0;
	int framecounter=0,fps=0;
	SDL_Rect Rect;
    Rect.w = Buffer->w;
    Rect.h = Buffer->h;
    Rect.x = StartScreenX;
    Rect.y = StartScreenY;
    Tmp1 = SDL_CreateRGBSurface(SDL_HWSURFACE,WINDOW_WIDTH,WINDOW_HEIGHT,SCREEN_BPP,Screen->format->Rmask,Screen->format->Gmask,Screen->format->Bmask,Screen->format->Amask);
	Tmp = SDL_DisplayFormat(Tmp1);
	SDL_FreeSurface(Tmp1);
	if (StageReload)
	{
		WorldParts.Load("./temp.lev");
		remove("./temp.lev");
#ifdef GP2X
		sync();
#endif
		StageReload=false;
	}
    for (Teller = 0;Teller< WorldParts.ItemCount;Teller++)
        if(WorldParts.Items[Teller]->GetType() == IDPlayer)
        {
            PlayerFound = true;
            //printf("Player found, item count: %d\n",WorldParts.ItemCount);
            Selector.SetPosition(WorldParts.Items[Teller]->GetPlayFieldX(),WorldParts.Items[Teller]->GetPlayFieldY());
            break;
        }
    WorldParts.ViewPort->SetVPLimit(0,0,NrOfCols-1,NrOfRows-1);
    if (!PlayerFound)
    {
        WorldParts.ViewPort->SetViewPort((NrOfCols/2)-9,(NrOfRows/2)-7,(NrOfCols/2)+10,(NrOfRows/2)+7);
        Selector.SetPosition((NrOfCols/2),(NrOfRows/2));
    }

    //
	while (GameState == GSLevelEditor)
	{
        if(GlobalSoundEnabled)
            if (! Mix_PlayingMusic())
            {
                SelectedMusic = 0;
                Mix_PlayMusic(Music[SelectedMusic],0);
                SetVolume(Volume);
            }

		if (alpha < 255)
        {
            if(alpha+AlphaInc > MaxAlpha)
            {
                alpha = 255;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
            else
            {
                alpha+=AlphaInc;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
        }
		SDL_BlitSurface(IMGBackground,NULL,Tmp,NULL);
		if(ShowGrid)
            SDL_BlitSurface(IMGGrid,NULL,Tmp,NULL);
		WorldParts.Draw(Tmp);
		Selector.Draw(Tmp);
		sprintf(Tekst,"Fps:%d",fps);
        WriteText(Tmp,BigFont,Tekst,strlen(Tekst),0,0,0,Color,false);
		if (ShowPosition)
		{
			sprintf(Tekst,"X: %d - Y: %d",Selector.GetPlayFieldX(),Selector.GetPlayFieldY());
			boxRGBA(Tmp,257*SCREENPOSMULTIPLY,0,319*SCREENPOSMULTIPLY,13*SCREENPOSMULTIPLY,MenuBoxColor.r,MenuBoxColor.g,MenuBoxColor.b,MenuBoxColor.unused);
			rectangleRGBA(Tmp,257*SCREENPOSMULTIPLY,-1,319*SCREENPOSMULTIPLY,13*SCREENPOSMULTIPLY,MenuBoxBorderColor.r,MenuBoxBorderColor.g,MenuBoxBorderColor.b,MenuBoxBorderColor.unused);
			WriteText(Tmp,font,Tekst,strlen(Tekst),259*SCREENPOSMULTIPLY,0,0,MenuTextColor,false);
		}
		SDL_BlitSurface(Tmp,NULL,Buffer,NULL);
        SDL_FillRect(Buffer1,NULL,SDL_MapRGB(Buffer1->format,0,0,0));
        SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
        SDL_BlitSurface(Buffer1,NULL,Screen,NULL);
        SDL_Flip(Screen);

        Input->Update();
        framecounter++;
        if(Time2 + 1000 < SDL_GetTicks())
        {
            fps = framecounter;
            framecounter = 0;
            Time2 = SDL_GetTicks();
        }
        if(Input->SpecialsHeld[SPECIAL_QUIT_EV])
            GameState = GSQuit;

        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_VOLUP] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_VOLUP)] || Input->KeyboardHeld[SDLK_KP_PLUS]))
        {
            IncVolume();
            Input->Delay();
        }

        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_VOLDOWN] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_VOLMIN)] || Input->KeyboardHeld[SDLK_KP_MINUS]))
        {
            DecVolume();
            Input->Delay();
        }

        if(Input->JoystickHeld[0][GP2X_BUTTON_SELECT] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_SELECT)] || Input->KeyboardHeld[SDLK_ESCAPE])
        {
            if (LevelHasChanged)
            {
                if(AskQuestion("The current level isn't saved yet!\nDo you want to save it now ?\n\nPress (A) to save, (X) to cancel saving"))
                {
                    SDL_BlitSurface(IMGBackground,NULL,Buffer,NULL);
                    WorldParts.Draw(Buffer);
                    Selector.Draw(Buffer);
                    if (ShowPosition)
                    {
                        sprintf(Tekst,"X: %d - Y: %d",Selector.GetPlayFieldX(),Selector.GetPlayFieldY());
                        boxRGBA(Buffer,257*SCREENPOSMULTIPLY,0,WINDOW_WIDTH-1,13*SCREENPOSMULTIPLY,MenuBoxColor.r,MenuBoxColor.g,MenuBoxColor.b,MenuBoxColor.unused);
                        rectangleRGBA(Buffer,257*SCREENPOSMULTIPLY,-1,WINDOW_WIDTH,13*SCREENPOSMULTIPLY,MenuBoxBorderColor.r,MenuBoxBorderColor.g,MenuBoxBorderColor.b,MenuBoxBorderColor.unused);
                        WriteText(Buffer,font,Tekst,strlen(Tekst),259*SCREENPOSMULTIPLY,0,0,MenuTextColor,false);
                    }
                    if (!LevelErrorsFound())
                    {
                        if (SelectedLevel==0)
                                sprintf(FileName,"./levelpacks/%s/level%d.lev",LevelPackFileName,InstalledLevels+1);
                        else
                            sprintf(FileName,"./levelpacks/%s/level%d.lev",LevelPackFileName,SelectedLevel);
                        WorldParts.Save(FileName);
                        FindLevels();
                        if (SelectedLevel==0)
                            SelectedLevel = InstalledLevels;
                        LevelHasChanged=false;
                        GameState = GSStageSelect;
                    }
                }
                else
                    GameState = GSStageSelect;
                Input->Reset();
            }
            else
                GameState = GSStageSelect;
        }

        if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_START] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_START)] || Input->KeyboardHeld[SDLK_RETURN]))
        {
            if(!LevelErrorsFound())
            {
                WorldParts.Save("./temp.lev");
                StageReload = true;
                GameState=GSGame;
                WorldParts.CenterVPOnPlayer();
            }
            Input->Reset();
            Input->Delay();
        }

        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_L] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_L)] || Input->KeyboardHeld[SDLK_PAGEDOWN] || Input->KeyboardHeld[SDLK_l]))
        {
            Selector.DecSelection();
            Input->Delay();
        }

        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_R] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_R)] || Input->KeyboardHeld[SDLK_PAGEUP] || Input->KeyboardHeld[SDLK_r]))
        {
            Selector.IncSelection();
            Input->Delay();
        }

        if(Input->JoystickHeld[0][GP2X_BUTTON_A] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_A)] || Input->KeyboardHeld[SDLK_a] || Input->KeyboardHeld[SDLK_q] || Input->KeyboardHeld[SDLK_SPACE])
        {
            SamePartFound = false;
            for(Teller=0;Teller<WorldParts.ItemCount;Teller++)
                if((WorldParts.Items[Teller]->GetPlayFieldX() == Selector.GetPlayFieldX()) &&
                   (WorldParts.Items[Teller]->GetPlayFieldY() == Selector.GetPlayFieldY()))
                {
                    if(WorldParts.Items[Teller]->GetType() == Selector.GetSelection())
                    {
                        SamePartFound=true;
                    }
                    if(Selector.GetSelection() == IDEmpty)
                    {
                        LevelHasChanged=true;
                        break;
                    }
                }
            if(Selector.GetSelection() != IDEmpty)
                if(!LevelHasChanged)
                    LevelHasChanged=!SamePartFound;
            switch(Selector.GetSelection())
            {
                case IDEmpty:
                    for(Teller=0;Teller<WorldParts.ItemCount;Teller++)
                    {
                        if((WorldParts.Items[Teller]->GetPlayFieldX() == Selector.GetPlayFieldX()) &&
                           (WorldParts.Items[Teller]->GetPlayFieldY() == Selector.GetPlayFieldY()))
                            if(WorldParts.Items[Teller]->GetType() == IDFloor)
                            {
                                AnotherPartFound = false;
                                for(Teller2=Teller+1;Teller2<WorldParts.ItemCount;Teller2++)
                                    if((WorldParts.Items[Teller2]->GetPlayFieldX() == Selector.GetPlayFieldX()) &&
                                       (WorldParts.Items[Teller2]->GetPlayFieldY() == Selector.GetPlayFieldY()))
                                        {
                                                WorldParts.Remove(WorldParts.Items[Teller2]->GetPlayFieldX(),WorldParts.Items[Teller2]->GetPlayFieldY(),WorldParts.Items[Teller2]->GetType());
                                                AnotherPartFound = true;
                                                break;
                                        }


                                if (!AnotherPartFound)
                                {
                                    WorldParts.Remove(WorldParts.Items[Teller]->GetPlayFieldX(),WorldParts.Items[Teller]->GetPlayFieldY(),IDFloor);
                                    break;
                                }
                            }
                            else
                            {
                                WorldParts.Remove(Selector.GetPlayFieldX(),Selector.GetPlayFieldY());
                                break;
                            }
                    }
                    break;
                case IDBox:
                    WorldParts.Remove(Selector.GetPlayFieldX(),Selector.GetPlayFieldY(),IDWall);
                    WorldParts.Remove(Selector.GetPlayFieldX(),Selector.GetPlayFieldY(),IDBox);
                    WorldParts.Remove(Selector.GetPlayFieldX(),Selector.GetPlayFieldY(),IDPlayer);
                    WorldParts.Remove(Selector.GetPlayFieldX(),Selector.GetPlayFieldY(),IDBomb);
                    WorldParts.Remove(Selector.GetPlayFieldX(),Selector.GetPlayFieldY(),IDDiamond);
                    WorldParts.Add(new CBox(Selector.GetPlayFieldX(),Selector.GetPlayFieldY()));
                    break;
                case IDPlayer:
                    WorldParts.Remove(Selector.GetPlayFieldX(),Selector.GetPlayFieldY(),IDWall);
                    WorldParts.Remove(Selector.GetPlayFieldX(),Selector.GetPlayFieldY(),IDBox);
                    WorldParts.Remove(Selector.GetPlayFieldX(),Selector.GetPlayFieldY(),IDBomb);
                    WorldParts.Remove(Selector.GetPlayFieldX(),Selector.GetPlayFieldY(),IDDiamond);
                    for(Teller=0;Teller<WorldParts.ItemCount;Teller++)
                    {
                        if (WorldParts.Items[Teller]->GetType() == IDPlayer)
                            WorldParts.Remove(WorldParts.Items[Teller]->GetPlayFieldX(),WorldParts.Items[Teller]->GetPlayFieldY(),IDPlayer);
                    }
                    WorldParts.Add(new CPlayer(Selector.GetPlayFieldX(),Selector.GetPlayFieldY()));
                    break;
                case IDWall:
                    WorldParts.Remove(Selector.GetPlayFieldX(),Selector.GetPlayFieldY());
                    WorldParts.Add(new CWall(Selector.GetPlayFieldX(),Selector.GetPlayFieldY()));
                    break;
                case IDFloor:
                    WorldParts.Remove(Selector.GetPlayFieldX(),Selector.GetPlayFieldY(),IDFloor);
                    WorldParts.Remove(Selector.GetPlayFieldX(),Selector.GetPlayFieldY(),IDWall);
                    WorldParts.Add(new CFloor(Selector.GetPlayFieldX(),Selector.GetPlayFieldY()));
                    break;
                case IDBomb:
                    WorldParts.Remove(Selector.GetPlayFieldX(),Selector.GetPlayFieldY(),IDWall);
                    WorldParts.Remove(Selector.GetPlayFieldX(),Selector.GetPlayFieldY(),IDBox);
                    WorldParts.Remove(Selector.GetPlayFieldX(),Selector.GetPlayFieldY(),IDPlayer);
                    WorldParts.Remove(Selector.GetPlayFieldX(),Selector.GetPlayFieldY(),IDBomb);
                    WorldParts.Remove(Selector.GetPlayFieldX(),Selector.GetPlayFieldY(),IDDiamond);
                    WorldParts.Add(new CBomb(Selector.GetPlayFieldX(),Selector.GetPlayFieldY()));
                    break;
                case IDDiamond:
                    WorldParts.Remove(Selector.GetPlayFieldX(),Selector.GetPlayFieldY(),IDWall);
                    WorldParts.Remove(Selector.GetPlayFieldX(),Selector.GetPlayFieldY(),IDBox);
                    WorldParts.Remove(Selector.GetPlayFieldX(),Selector.GetPlayFieldY(),IDPlayer);
                    WorldParts.Remove(Selector.GetPlayFieldX(),Selector.GetPlayFieldY(),IDBomb);
                    WorldParts.Remove(Selector.GetPlayFieldX(),Selector.GetPlayFieldY(),IDDiamond);
                    WorldParts.Add(new CDiamond(Selector.GetPlayFieldX(),Selector.GetPlayFieldY()));
                    break;


            }
        }

        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_LEFT] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_LEFT)] || Input->KeyboardHeld[SDLK_LEFT]))
        {
            Selector.MoveLeft();
            if(Selector.GetPlayFieldX() <  WorldParts.ViewPort->VPMinX+3)
                WorldParts.ViewPort->Move(-TileWidth,0);
            Input->Delay();
        }

        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_RIGHT] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_RIGHT)] || Input->KeyboardHeld[SDLK_RIGHT]))
        {
            Selector.MoveRight();
            if(Selector.GetPlayFieldX() > WorldParts.ViewPort->VPMaxX - 3)
                WorldParts.ViewPort->Move(TileWidth,0);
            Input->Delay();
        }

        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_UP] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_UP)] || Input->KeyboardHeld[SDLK_UP]))
        {
            Selector.MoveUp();
            if(Selector.GetPlayFieldY() < WorldParts.ViewPort->VPMinY+3)
                WorldParts.ViewPort->Move(0,-TileWidth);
            Input->Delay();
        }

        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_DOWN] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_DOWN)] || Input->KeyboardHeld[SDLK_DOWN]))
        {
            Selector.MoveDown();
            if(Selector.GetPlayFieldY() > WorldParts.ViewPort->VPMaxY-3)
                WorldParts.ViewPort->Move(0,TileWidth);
            Input->Delay();
        }

        if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_B] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_B)] || Input->KeyboardHeld[SDLK_b]))
        {
            if(ShowPosition && ShowGrid)
                ShowGrid = !ShowGrid;
            else
            if(ShowPosition && !ShowGrid)
                ShowPosition = !ShowPosition;
            else
            if(!ShowPosition && ! ShowGrid)
               ShowGrid = !ShowGrid;
            else
            if(!ShowPosition && ShowGrid)
                ShowPosition = !ShowPosition;
            Input->Delay();
        }

        if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_Y] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_Y)] || Input->KeyboardHeld[SDLK_y] || Input->KeyboardHeld[SDLK_BACKSPACE] || Input->KeyboardHeld[SDLK_DELETE]))
        {
            if (WorldParts.ItemCount > 0)
            if(AskQuestion("You are about to delete all parts\nin this level, are you sure\nyou want to do this?\n\nPress (A) to delete, (X) to cancel"))
            {
                LevelHasChanged = true;
                WorldParts.RemoveAll();
            }
            Input->Reset();
            Input->Delay();
        }

        if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_X] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_X)] || Input->KeyboardHeld[SDLK_x] ))
        {
            MinX = NrOfCols-1;
            MinY = NrOfRows-1;
            MaxX = 0;
            MaxY = 0;
            for (Teller = 0;Teller<WorldParts.ItemCount;Teller++)
            {
                if (WorldParts.Items[Teller]->GetPlayFieldX() < MinX)
                    MinX = WorldParts.Items[Teller]->GetPlayFieldX();
                if (WorldParts.Items[Teller]->GetPlayFieldY() < MinY)
                    MinY = WorldParts.Items[Teller]->GetPlayFieldY();
                if (WorldParts.Items[Teller]->GetPlayFieldX() > MaxX)
                    MaxX = WorldParts.Items[Teller]->GetPlayFieldX();
                if (WorldParts.Items[Teller]->GetPlayFieldY() > MaxY)
                    MaxY = WorldParts.Items[Teller]->GetPlayFieldY();
            }
            Xi = ((NrOfCols-1) / 2) - (MaxX + MinX) / 2;
            Yi = ((NrOfRows-1) / 2) - (MaxY + MinY) / 2;
            for (Teller = 0;Teller<WorldParts.ItemCount;Teller++)
            {
                WorldParts.Items[Teller]->SetPosition(WorldParts.Items[Teller]->GetPlayFieldX() + Xi,WorldParts.Items[Teller]->GetPlayFieldY() + Yi);
            }
            WorldParts.LimitVPLevel();
            //center viewport;
            WorldParts.ViewPort->SetViewPort((NrOfCols/2)-9,(NrOfRows/2)-7,(NrOfCols/2)+10,(NrOfRows/2)+7);
            Selector.SetPosition((NrOfCols/2),(NrOfRows/2));
            //if player found center viewport on player !
            for (Teller = 0;Teller<WorldParts.ItemCount;Teller++)
            {
                if(WorldParts.Items[Teller]->GetType() == IDPlayer)
                {
                    Selector.SetPosition(WorldParts.Items[Teller]->GetPlayFieldX(),WorldParts.Items[Teller]->GetPlayFieldY());
                    WorldParts.CenterVPOnPlayer();
                    break;
                }
            }
            WorldParts.ViewPort->SetVPLimit(0,0,NrOfCols-1,NrOfRows-1);
            if (Xi != 0 || Yi!=0)
                LevelHasChanged = true;
            Input->Delay();
        }
        SDL_framerateDelay(&Fpsman);
	}
	delete[] FileName;
	SDL_FreeSurface(Tmp);
	delete Input;
}

void Intro()
{
    SDL_Rect Rect;
    CInput *Input = new CInput(10);
    Uint8 alpha = 0;
    Uint32 Time=0;
    bool RPressed=false;
    int IntroScreenNr = 1;
    SDL_Surface *Tmp;
    Tmp = SDL_DisplayFormat(Buffer);
    Time = SDL_GetTicks();
    while (GameState == GSIntro)
    {
        Input->Update();

        if(Input->JoystickHeld[0][GP2X_BUTTON_SELECT] || Input->JoystickHeld[0][GP2X_BUTTON_START] || Input->JoystickHeld[0][GP2X_BUTTON_A]  ||
           Input->JoystickHeld[0][GP2X_BUTTON_B] || Input->JoystickHeld[0][GP2X_BUTTON_X] || Input->JoystickHeld[0][GP2X_BUTTON_Y] ||
           Input->KeyboardHeld[SDLK_SPACE] || Input->KeyboardHeld[SDLK_RETURN] || Input->KeyboardHeld[SDLK_ESCAPE] || Input->KeyboardHeld[SDLK_a] ||
           Input->KeyboardHeld[SDLK_x] || Input->KeyboardHeld[SDLK_y] || Input->KeyboardHeld[SDLK_b] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_A)] ||
           Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_B)] ||  Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_X)]  || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_Y)] )
        {
            GameState=GSTitleScreen;
        }

        if(Input->SpecialsHeld[SPECIAL_QUIT_EV])
            GameState = GSQuit;

        if(Input->JoystickHeld[0][GP2X_BUTTON_R] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_R)] || Input->KeyboardHeld[SDLK_r])
        {
            if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_LEFT] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_LEFT)] || Input->KeyboardHeld[SDLK_LEFT]))
            {
                if(StartScreenX - 2 >= 0)
                    StartScreenX -=2;
                Input->Delay();
            }

            if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_RIGHT] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_RIGHT)] || Input->KeyboardHeld[SDLK_RIGHT]))
            {
                if(StartScreenX + 2 + Buffer->w <= Screen->w)
                    StartScreenX +=2;
                Input->Delay();
            }

            if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_UP] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_UP)] || Input->KeyboardHeld[SDLK_UP]))
            {
                if(StartScreenY - 2 >=0)
                    StartScreenY -=2;
                Input->Delay();
            }

            if(Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_DOWN] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_DOWN)] || Input->KeyboardHeld[SDLK_DOWN]))
            {
                if(StartScreenY + 2 + Buffer->h <= Screen->h)
                    StartScreenY +=2;
                Input->Delay();
            }
        }

        if (Input->Ready() && (Input->JoystickHeld[0][GP2X_BUTTON_L] || Input->JoystickHeld[1][JoystickSetup->GetButtonValue(BUT_L)] || Input->KeyboardHeld[SDLK_PAGEDOWN] || Input->KeyboardHeld[SDLK_l]))
        {
            PreviousSkin();
            Input->Delay();
        }

        //tekenen naar buffer

        switch(IntroScreenNr)
        {
            case 1 :
                SDL_BlitSurface(IMGIntro1,NULL,Tmp,NULL);
                break;
            case 2 :
                SDL_BlitSurface(IMGIntro2,NULL,Tmp,NULL);
                break;
            case 3 :
                SDL_BlitSurface(IMGIntro3,NULL,Tmp,NULL);
                break;
        }

        if (alpha < 255)
        {
            if(alpha+AlphaInc > MaxAlpha)
            {
                alpha = 255;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
            else
            {
                alpha+=AlphaInc;
                SDL_SetAlpha(Tmp,SDL_SRCALPHA | SDL_RLEACCEL,alpha);
            }
        }




        Rect.w = Buffer->w;
        Rect.h = Buffer->h;
        Rect.x = StartScreenX;
        Rect.y = StartScreenY;


        SDL_FillRect(Buffer1,NULL,SDL_MapRGB(Buffer1->format,0,0,0));
        SDL_BlitSurface(Tmp,NULL,Buffer,NULL);
        SDL_BlitSurface(Buffer,NULL,Buffer1,&Rect);
      /*  #ifdef WIN32
            ScreenBufferZoom = zoomSurface(Buffer1,2,2,0);
            SDL_BlitSurface(ScreenBufferZoom,NULL,Screen,NULL);
            SDL_FreeSurface(ScreenBufferZoom);
        #else*/
        SDL_BlitSurface(Buffer1,NULL,Screen,NULL);
       // #endif

        SDL_Flip(Screen);
        SDL_framerateDelay(&Fpsman);
       /* #ifdef WIN32
        SDL_Delay(10);
        #endif*/
        if(Time + 3500 < SDL_GetTicks())
        {
            alpha = 0;
            IntroScreenNr++;
            if(IntroScreenNr > 3)
                GameState = GSTitleScreen;
            Time = SDL_GetTicks();
        }

    }
    SDL_FreeSurface(Tmp);

}


void UnLoadGraphics()
{
    if(IMGGrid)
        SDL_FreeSurface(IMGGrid);
    if(IMGWall)
        SDL_FreeSurface(IMGWall);
	if(IMGBackground)
		SDL_FreeSurface(IMGBackground);
	if(IMGFloor)
		SDL_FreeSurface(IMGFloor);
	if(IMGPlayer)
		SDL_FreeSurface(IMGPlayer);
	if(IMGBox)
		SDL_FreeSurface(IMGBox);
	if(IMGEmpty)
		SDL_FreeSurface(IMGEmpty);

    if(IMGDiamond)
		SDL_FreeSurface(IMGDiamond);
	if(IMGBomb)
		SDL_FreeSurface(IMGBomb);
	if(IMGTitleScreen)
		SDL_FreeSurface(IMGTitleScreen);

    if(IMGIntro1)
        SDL_FreeSurface(IMGIntro1);
    if(IMGIntro2)
        SDL_FreeSurface(IMGIntro2);
    if(IMGIntro3)
        SDL_FreeSurface(IMGIntro3);
     if(IMGExplosion)
        SDL_FreeSurface(IMGExplosion);


}

void UnloadMusic()
{
	int Teller;
	if (GlobalSoundEnabled)
	{
		Mix_HaltMusic();
		for (Teller=0;Teller < MusicCount;Teller++)
			if (Music[Teller])
				Mix_FreeMusic(Music[Teller]);
	}
	//Mix_FreeMusic(Music[0]);
}

void LoadSounds()
{
	if (GlobalSoundEnabled)
	{
		Sounds[SND_MENU] = Mix_LoadWAV("./sound/menu.wav");
		Sounds[SND_SELECT] = Mix_LoadWAV("./sound/select.wav");
		Sounds[SND_ERROR] = Mix_LoadWAV("./sound/error.wav");
		Sounds[SND_STAGEEND] = Mix_LoadWAV("./sound/stageend.wav");
		Sounds[SND_EXPLODE] = Mix_LoadWAV("./sound/explode.wav");
		Sounds[SND_COLLECT] = Mix_LoadWAV("./sound/collect.wav");
	}
}



void UnloadSounds()
{
	int Teller;
	for (Teller=0;Teller<NrOfSounds;Teller++)
		if(Sounds[Teller])
			Mix_FreeChunk(Sounds[Teller]);
}

void LoadJoystickSettings()
{
    JoystickSetup->AddDefinition(BUT_SELECT,"Go back / Quit",7,7);
    JoystickSetup->AddDefinition(BUT_R,"Next Music / Next Part",5,5);
    JoystickSetup->AddDefinition(BUT_L,"Next Skin / Prev Part",4,4);
    JoystickSetup->AddDefinition(BUT_LEFT,"Left",JOYSTICK_LEFT,JOYSTICK_LEFT);
    JoystickSetup->AddDefinition(BUT_RIGHT,"Right",JOYSTICK_RIGHT,JOYSTICK_RIGHT);
    JoystickSetup->AddDefinition(BUT_DOWN,"Down",JOYSTICK_DOWN,JOYSTICK_DOWN);
    JoystickSetup->AddDefinition(BUT_UP,"Up",JOYSTICK_UP,JOYSTICK_UP);
    JoystickSetup->AddDefinition(BUT_A,"Select / Place part",0,0);
    JoystickSetup->AddDefinition(BUT_X,"Center level",2,2);
    JoystickSetup->AddDefinition(BUT_Y,"New Level / Erase all parts",3,3);
    JoystickSetup->AddDefinition(BUT_B,"Stats / Hide position",1,1);
    JoystickSetup->AddDefinition(BUT_START,"Restart / Test level",6,6);
    JoystickSetup->AddDefinition(BUT_VOLUP,"Volume up",9,9);
    JoystickSetup->AddDefinition(BUT_VOLMIN,"Volume down",8,8);
    JoystickSetup->LoadCurrentButtonValues("./joystick.def");
}

int main(int argc, char **argv)
{
    SDL_Rect Rect;
    SDL_Surface *Icon;
	//mtrace();
#ifndef GP2X
        if (argc ==2)
        {
            SCREENPOSMULTIPLY = atoi(argv[1]);
            if((SCREENPOSMULTIPLY < 1) || (SCREENPOSMULTIPLY > 10))
                SCREENPOSMULTIPLY = 1;
        }
        WINDOW_WIDTH = 320 *SCREENPOSMULTIPLY,
        WINDOW_HEIGHT = 240*SCREENPOSMULTIPLY,
        TileWidth = 16*SCREENPOSMULTIPLY,
        TileHeight = 16*SCREENPOSMULTIPLY;
#endif
	srand((int) time(NULL));
	if (SDL_Init(SDL_INIT_JOYSTICK | SDL_INIT_VIDEO | SDL_INIT_AUDIO ) == 0)
	{
		printf("SDL Succesfully initialized\n");
#ifdef GP2X
		if(SDL_GP2X_GetPhysicalScreenSize(&Rect) == 0)
            Screen = SDL_SetVideoMode( WINDOW_WIDTH, WINDOW_HEIGHT,SCREEN_BPP, SDL_HWSURFACE );
        else
        {
            TvOutMode = true;
            Screen = SDL_SetVideoMode( 360, 288,16, SDL_HWSURFACE );
        }
#else
        Screen = SDL_SetVideoMode( WINDOW_WIDTH, WINDOW_HEIGHT,SCREEN_BPP, SDL_SWSURFACE);
        TvOutMode = false;
        Icon = IMG_Load("./graphics/blips.png");
        SDL_WM_SetIcon(Icon,NULL);
        SDL_WM_SetCaption( "Blips - Created By Willems Soft - Original game by Bryant Brownell", NULL );
#endif
		if(Screen)
		{
		    printf("Succesfully Set %dx%dx16\n",WINDOW_WIDTH,WINDOW_HEIGHT);
		    Buffer = SDL_CreateRGBSurface(SDL_SWSURFACE,WINDOW_WIDTH,WINDOW_HEIGHT,SCREEN_BPP,Screen->format->Rmask,Screen->format->Gmask,Screen->format->Bmask,Screen->format->Amask);
			if (TvOutMode)
                Buffer1 = SDL_CreateRGBSurface(SDL_HWSURFACE,360,288,SCREEN_BPP,Screen->format->Rmask,Screen->format->Gmask,Screen->format->Bmask,Screen->format->Amask);
            else
                Buffer1 = SDL_CreateRGBSurface(SDL_SWSURFACE,WINDOW_WIDTH, WINDOW_HEIGHT,SCREEN_BPP,Screen->format->Rmask,Screen->format->Gmask,Screen->format->Bmask,Screen->format->Amask);
			if (Buffer && Buffer1)
			{
			    SDL_FillRect(Buffer,NULL,SDL_MapRGB(Buffer->format,0,0,0));
                SDL_ShowCursor(SDL_DISABLE);

#ifdef GP2X
                    if (Mix_OpenAudio(22050,AUDIO_S16,MIX_DEFAULT_CHANNELS,128) < 0)
#else
                    if (Mix_OpenAudio(22050,AUDIO_S16,MIX_DEFAULT_CHANNELS,1024) < 0)
#endif

                    {
                        GlobalSoundEnabled = false;
                        printf("Failed to initialise sound!\n");
                        printf("%s\n",Mix_GetError());
                    }
                    else
                    {
                        printf("Audio Succesfully initialised!\n");
                    }
                    if (TTF_Init() == 0)
                    {
                        printf("Succesfully initialized TTF\n");
                        font = TTF_OpenFont("./font.ttf",9*SCREENPOSMULTIPLY);
                        BigFontSkin = TTF_OpenFont("./font.ttf",16*SCREENPOSMULTIPLY);
                        BigFont = TTF_OpenFont("./font2.ttf",20*SCREENPOSMULTIPLY);
                        MonoFont = TTF_OpenFont("./font1.ttf",17*SCREENPOSMULTIPLY);
                        if (font && BigFont && MonoFont && BigFontSkin)
                        {
                            printf("Succesfully Loaded fonts\n");
                            JoystickSetup = new CUsbJoystickSetup();
                            LoadJoystickSettings();
                            SDL_initFramerate(&Fpsman);
                            SDL_setFramerate(&Fpsman,FPS);
                            TTF_SetFontStyle(font,TTF_STYLE_NORMAL);
                            SearchForLevelPacks();
                            SearchForMusic();
                            SearchForSkins();
                            LoadSounds();
                            FindLevels();
                            LoadSettings();
                            LoadGraphics();
                            while (GameState != GSQuit)
                            {
                                switch(GameState)
                                {
                                    case GSIntro :
                                        Intro();
                                        break;
                                    case GSTitleScreen :
                                        TitleScreen();
                                        break;
                                    case GSCredits :
                                        Credits();
                                        break;
                                    case GSGame :
                                        Game();
                                        break;
                                    case GSStageClear:
                                        break;
                                    case GSStageSelect:
                                        StageSelect();
                                        break;
                                    case GSLevelEditor:
                                        LevelEditor();
                                        break;
                                    case GSLevelEditorMenu:
                                        LevelEditorMode = true;
                                        LevelEditorMenu();
                                        break;
                                    case GSSetupUsbJoystickButtons:
                                        SetupUsbJoystickButtons();
                                        break;
                                    default:
                                        break;
                                }
                            }
                            SaveSettings();
                            UnLoadGraphics();
                            UnloadSounds();
                            UnloadMusic();
                            TTF_CloseFont(font);
                            TTF_CloseFont(BigFont);
                            TTF_CloseFont(MonoFont);
                            TTF_CloseFont(BigFontSkin);
                            MonoFont=NULL;
                            font=NULL;
                            BigFont=NULL;
                            JoystickSetup->SaveCurrentButtonValues("./joystick.def");
                            delete JoystickSetup;
                        }
                        else
                        {
                            printf("Failed to Load fonts\n");
                        }
                        TTF_Quit();
                    }
                    else
                    {
                        printf("Failed to initialize TTF\n");
                    }



                SDL_FreeSurface(Buffer);
                Buffer=NULL;
                SDL_FreeSurface(Buffer1);
                Buffer1=NULL;
			}
			else
                printf("Failed To Create Buffer\n");
            SDL_FreeSurface(Screen);
            Screen=NULL;
#ifndef GP2X
            SDL_FreeSurface(Icon);
#endif
		}
		else
		{
			printf("Failed to Set Videomode %dx%dx16\n",WINDOW_WIDTH, WINDOW_HEIGHT);
		}
		SDL_Quit();
	}
	else
	{
		printf("Couldn't initialise SDL!\n");
	}
	WorldParts.RemoveAll();
	// Shouldn't be needed but i just placed it again to be sure all the boardparts are free
	// Call the menu
	//chdir("/usr/gp2x");
	//execl("/usr/gp2x/gp2xmenu", "/usr/gp2x/gp2xmenu", NULL);

	return 0;

}
